unit fFunc00000;

interface

uses WIndows, Classes, Controls, fFunc, GLScene, GLObjects, GLTeapot;

type
	TfrmFunc00000 = class(TfrmFunc)
  protected
    FTeapot: TGLTeapot;
    procedure CREATE_OBJECT; override;
    procedure DO_KeyPress(Sender: TObject; var Key: Char); override;
    procedure DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); override;
    procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00000 }

procedure TfrmFunc00000.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 30, newTime);
  inherited;
end;

procedure TfrmFunc00000.DO_KeyPress(Sender: TObject; var Key: Char);
begin
	with FTeapot do case Key of
    '7' : RotateAbsolute(-10,  0,  0);
    '9' : RotateAbsolute(+10,  0,  0);
    '4' : RotateAbsolute(  0,-10,  0);
    '6' : RotateAbsolute(  0,+10,  0);
    '1' : RotateAbsolute(  0,  0,-10);
    '3' : RotateAbsolute(  0,  0,+10);
	end;
end;

procedure TfrmFunc00000.DO_MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
var
	DistanceRatio: Single;
begin
  DistanceRatio := Power(1.1, WheelDelta/120);
	FCamera.AdjustDistanceToTarget(DistanceRatio);
  Handled := True;
end;

procedure TfrmFunc00000.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
	dx, dy: Integer;
  Ratio: Single;
	V: TVector;
begin
	dx := mx - X;
  dy := my - Y;
	if ssLeft in Shift then
  begin
  	if ssShift in Shift then 	FCamera.RotateObject(FTeapot, dy, dx)
    else 											FCamera.MoveAroundTarget(dy, dx)
	end
  else if Shift = [ssRight] then
  begin
    with FCamera do Ratio := 0.12 * DistanceToTarget / FocalLength;
		V := FCamera.ScreenDeltaToVectorXY(dx, -dy, Ratio);
		FDummy.Position.Translate(V);
		FCamera.TransformationChanged;//Notification
	end;
	mx := X;
  my := Y;
end;

procedure TfrmFunc00000.CREATE_OBJECT;
begin
  inherited;

  FTeapot := TGLTeapot.Create(Self);

  FDummy.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FTeapot);
  FScene.Cameras.AddChild(FCamera);

  with FCamera do
  begin
  	TargetObject := FDummy;
  	DepthOfView := 1000;
  	FocalLength := 500;
    Position.AsVector := VectorMake(0,0,10);
    Up.AsVector := VectorMake(0,1,0);
	end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(0,10,10);
	end;

  with FDummy do
  begin
	  ShowAxes := True;
  	VisibleAtRunTime := True;
  end;

  with FTeapot do
  begin
  	Material.FrontProperties.Diffuse.Color := VectorMake(0.5764706,1,0.5686275,1);
  	Direction.AsVector := VectorMake(0,1,0);
    Scale.AsVector := VectorMake(2,2,2);
    Up.AsVector := VectorMake(0,1,0);
  end;

end;

end.
