unit fFunc00003;

interface

uses Windows, Classes, Controls, Forms, ExtCtrls, Dialogs, fFunc, GLScene,GLObjects, GLGeomObjects, GLTexture;

type
	TfrmFunc00003 = class(TfrmFunc)
  protected
    FPickTimer: TTimer;
    FSphere: TGLSphere;
    FCylinder: TGLCylinder;
    FTorus: TGLTorus;
    FCone: TGLCone;
  	procedure CREATE_OBJECT; override;
    procedure DO_PickTimer(Sender: TObject);
		procedure DO_ObjectsProgress(Sender: TObject; const deltaTime, newTime: Double);
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_ViewerClick(Sender: TObject); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;    
  public
  	CurrentPick: TGLCustomSceneObject;
  end;

implementation

uses VectorGeometry, SysUtils, fMain;

{ TfrmFunc00003 }

procedure TfrmFunc00003.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FTorus, 30, newTime);
  ROTATE_SCENE_OBJECT(FCylinder, 30, newTime);
  ROTATE_SCENE_OBJECT(FCone, 30, newTime);
  ROTATE_SCENE_OBJECT(FSphere, 30, newTime);
//  ROTATE_SCENE_OBJECT(FDummy, 30, newTime, 1);
  inherited;
end;

procedure TfrmFunc00003.DO_ObjectsProgress(Sender: TObject; const deltaTime, newTime: Double);
var
	TargetColor: TColorVector;
begin
	with Sender as TGLCustomSceneObject do
  begin
    // if we are picked, target color is red, else it is black (no emission)
    if Sender = CurrentPick then TargetColor := clrRed else TargetColor := clrBlack;
    // Set new color at 66% between current and target color
    with Material.FrontProperties.Emission do Color := VectorLerp(TargetColor, Color, 0.66)
	end;
end;

procedure TfrmFunc00003.DO_PickTimer(Sender: TObject);
var
	cp: TPoint;
begin
  // get what is under the mouse
  GetCursorPos(cp);
  cp := FViewer.ScreenToClient(cp);
  CurrentPick := (FViewer.Buffer.GetPickedObject(cp.x, cp.y) as TGLCustomSceneObject);
  FPickTimer.Enabled := False;
end;

procedure TfrmFunc00003.DO_ViewerClick(Sender: TObject);
var
	pt: TPoint;
	pick: TGLCustomSceneObject;
begin
	GetCursorPos(pt);
  pt := FViewer.ScreenToClient(pt);
  pick := (FViewer.Buffer.GetPickedObject(pt.X, pt.Y) as TGLCustomSceneObject);
  if Assigned(pick) then
	  PostMessage(TForm(Owner).Handle,WM_NEWACTIVEFUNC,pick.Tag,0);
end;

procedure TfrmFunc00003.DO_ViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	pick: TGLCustomSceneObject;
begin
  pick := (FViewer.Buffer.GetPickedObject(x, y) as TGLCustomSceneObject);
  if Assigned(pick) then
  begin
  	FPickTimer.Enabled := False;
    pick.Material.FrontProperties.Emission.Color := clrYellow;
  end;
end;

procedure TfrmFunc00003.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
	FPickTimer.Enabled := True;
end;

procedure TfrmFunc00003.CREATE_OBJECT;
begin
  inherited;

  FSphere := TGLSphere.Create(Self);
  FCylinder := TGLCylinder.Create(Self);
  FTorus := TGLTorus.Create(Self);
  FCone := TGLCone.Create(Self);

  FSphere.OnProgress := Self.DO_ObjectsProgress;
  FCylinder.OnProgress := Self.DO_ObjectsProgress;
  FTorus.OnProgress := Self.DO_ObjectsProgress;
  FCone.OnProgress := Self.DO_ObjectsProgress;

  with FScene do
  begin
  	Cameras.AddChild(FCamera);
  	Objects.AddChild(FDummy);
  	Objects.AddChild(FLight);
  	FDummy.AddChild(FSphere);
  	FDummy.AddChild(FCylinder);
  	FDummy.AddChild(FTorus);
	 	FDummy.AddChild(FCone);
  end;

  with FViewer do
  begin
  	Buffer.AmbientColor.Color := clrSkyBlue;
  end;
  
  with FCamera do
  begin
  	DepthOfView := 1000;
  	FocalLength := 150;
  	Position.AsVector := VectorMake(0,0,10);
  	TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(50,50,50);
	  FLight.Shining := True;
  end;

 	with FDummy do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,0,0);
//  	ShowAxes := True;
  //	VisibleAtRunTime := True;
  end;

  with FTorus do
  begin
  	Tag := 0;
    Position.AsVector := VectorMake(-2.5,2,0);
    MajorRadius := 0.7;
    MinorRadius := 0.2;
    Rings := 25;
    Sides := 15;
  end;

  with FCylinder do
  begin
  	Tag := 1;
    Height := 1.5;
    Loops := 1;
    Position.AsVector := VectorMake(-2.5,-2,0);
    Slices := 16;
    Stacks := 4;
  end;

  with FCone do
  begin
  	Tag := 2;
    BottomRadius := 1;
    Position.AsVector := VectorMake(2.5,2,0);
    Height := 1.5;
    Loops := 1;
    Slices := 16;
    Stacks := 4;
  end;

  with FSphere do
  begin
  	Tag := 4;
	  Position.AsVector := VectorMake(2.5,-2,0);
    Radius := 1;
    Slices := 16;
    Stacks := 16;
    Start := 0;
    Stop := 360;
  end;

  FPickTimer	:= TTimer.Create(Self);
  with FPickTimer do
  begin
  	Interval := 50;
  	Enabled := False;
  	OnTimer := DO_PickTimer;
  end;

end;

end.
