unit fFunc00007;

interface

uses Windows, Classes, Forms, fFunc, GLScene, GLTexture, GLWindowsFont, GLCanvas;

type
	TfrmFunc00007 = class(TfrmFunc)
  protected
  	FWinFont: TGLWindowsBitmapFont;
  	FDirect: TGLDirectOpenGL;
    procedure CREATE_OBJECT; override;
    procedure DO_Resize(Sender: TObject); override;
    procedure DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo); override;
    procedure DO_Timer(Sender: TObject); override;
  end;

implementation

uses Math, SysUtils, VectorGeometry, GLCrossPlatform;

var Step: Integer = 0;

{ TfrmFunc00007 }

procedure TfrmFunc00007.DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo);
const
  LineCount		: Integer = 1000;
  EllipseCount: Integer = 1000;
  RectCount		: Integer = 1000;
  TextCount		: Integer = 300;
var
	C: TGLCanvas;
  W, H, I, X, Y: Integer;
  R: TRect;
  T: Int64;
  AColor: TColor;
begin
	T := StartPrecisionTimer;
	W := FViewer.ClientWidth;
  H := FViewer.ClientHeight;
	C := TGLCanvas.Create(W, H);
  try
  	C.PenWidth := 1;
  	case Step of
    	0:for I := 0 to LineCount-1 do
      	begin
          C.PenColor := Random(256*256*256);
          C.MoveTo(Random(W), Random(H));
          C.LineTo(Random(W), Random(H));
        end;
      1:for I := 0 to EllipseCount-1 do
      	begin
          C.PenColor := Random(256*256*256);
          C.Ellipse(Random(W), Random(H),
                  	Random(W), Random(H));
        end;
      2:for I := 0 to RectCount-1 do
      	begin
        	C.PenColor := Random(256*256*256);
          R := Rect(Random(W), Random(H),
          					Random(W), Random(H));
          C.FrameRect(R.Left, R.Top, R.Right, R.Bottom);
        end;
      3:for I := 0 to TextCount-1 do
      	begin
          AColor := Random(256*256*256);
          X := Random(W);
          Y := Random(H);
          FWinFont.TextOut(rci, X, Y, ' - HELLO - ', AColor);
        end;
    end;
  finally
  	FreeAndNil(C);
  end;
  TForm(Owner).Caption  := Format('GLCanvas: %.2f msec', [StopPrecisionTimer(T)*1000]);
end;

procedure TfrmFunc00007.DO_Resize(Sender: TObject);
begin
	inherited;
  with FWinFont do Font.Height := Min(-8, -FViewer.ClientWidth div 40);
end;

procedure TfrmFunc00007.DO_Timer(Sender: TObject);
begin
	Step := (Step + 5) mod 4;
  FViewer.Invalidate;
end;

procedure TfrmFunc00007.CREATE_OBJECT;
begin
  inherited;
  FCadencer.Scene := nil;
  FCadencer.OnProgress := nil;
  FTimer.Interval := 1000;
  
  FWinFont := TGLWindowsBitmapFont.Create(Self);
  FDirect  := TGLDirectOpenGL.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDirect);
  
  with FCamera do
  begin
  	DepthOfView := 100;
    FocalLength := 100;
    Position.AsVector := VectorMake(5,3,4);
    SceneScale := 2;
  	TargetObject := FDummy;
  end;

  with FDummy do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
    ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FWinFont do
  begin
  	Font.Height := -20;
    Font.Name := 'ARIAL';
    Font.Size := 15;
    Ranges.Clear;//Clear Ranges first or exception will be raised.
    with Ranges.Add do
    begin
    	StartASCII := #32;
      StopASCII  := #125;
      StartGlyphIdx := 0;
    end;
  end;

  with FDirect do
  begin
  	OnRender := DO_DirectRender;
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

end;

end.
