unit fFunc00009;

interface

uses Windows, Forms, Graphics, fFunc, GLScene, GLObjects, GLParticles, GLBehaviours;

type
	TfrmFunc00009 = class(TfrmFunc)
  protected
	  FParticles: TGLParticles;
    FSprite: TGLSprite;
    procedure CREATE_OBJECT; override;
		procedure DO_ActivateParticle(Sender: TObject; particle: TGLBaseSceneObject);
    procedure DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry, OpenGL1x;

{ TfrmFunc00009 }

procedure TfrmFunc00009.DO_ActivateParticle(Sender: TObject; particle: TGLBaseSceneObject);
var
	r, alpha, cr, sr: Single;
begin
	with TGLSprite(particle) do
  begin
  	alpha := Random*2*PI;
		r := 3*Random;
    SinCos(alpha, r*r, sr, cr);
		Children[0].Position.SetPoint(sr, 3*r-3, cr);
		GetOrCreateInertia(particle).TurnSpeed := Random(90);
    GetOrCreateInertia(particle).PitchSpeed := Random(90);
    GetOrCreateInertia(particle).RollSpeed := Random(90);
		TGLCustomSceneObject(particle).TagFloat := FCadencer.CurrentTime;
	end;
end;

procedure TfrmFunc00009.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	FParticles.CreateParticle;
  inherited;
end;

var
	angle: Single = 0.0;
  
procedure TfrmFunc00009.DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	with TGLCustomSceneObject(Sender) do
		if newTime - TagFloat > 4 then
			FParticles.KillParticle(TGLCustomSceneObject(Sender));
end;

procedure TfrmFunc00009.CREATE_OBJECT;
var
  B: TGLBInertia;
begin
  inherited;
  FParticles := TGLParticles.Create(Self);
  FSprite := TGLSprite.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FParticles);
  FParticles.AddChild(FDummy);
  FDummy.AddChild(FSprite);

  with FCamera do
  begin
  	DepthOfView := 500;
    FocalLength := 50;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(10,3,0);
  	TargetObject := FParticles;
  end;

  with FParticles do
  begin
    ParticlePoolSize := 10;
    OnActivateParticle := DO_ActivateParticle;
  end;

  with FDummy do
  begin
  	B := TGLBInertia.Create(Behaviours);
    with B do
    begin
    	DampingEnabled := False;
      Mass := 1;
      Name := 'Simple Inertia';
      TurnSpeed := 10;
    	OnProgress := DO_DummyProgress;
    end;
    Position.AsVector := VectorMake(0,-2,0);
  end;

  with FSprite do
  begin
  	Material.FrontProperties.Emission.AsWinColor := clLime;
  	AlphaChannel := 1;
    Height := 0.1;
    Width := 0.1;
  end;

end;

end.
 