unit fFunc00010;

interface

uses Classes, Graphics, StdCtrls, ExtCtrls, fFunc, GLObjects, GLGeomObjects, GLTexture;

type
	TfrmFunc00010 = class(TfrmFunc)
  protected
  	FLabel1: TLabel;
  	FPanel: TPanel;
    FRadioBtns: array[0..2] of TRadioButton;
    FLabel2: TLabel;
    FCheckBox:  array[0..1] of TCheckBox;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_RadioBtnClick(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
  protected
  	FSphereA: TGLSphere;	//Unit: GLObjects
    FSphereB: TGLSphere;	//Unit: GLObjects
    FSphereC: TGLSphere;	//Unit: GLObjects
    FDCCentral: TGLDummyCube;
    FSphere: TGLSphere;	//Unit: GLObjects
    FTorus: TGLTorus;
    FCone: TGLCone;
  	procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses GLMisc, VectorGeometry;

{ TfrmFunc00010 }

procedure TfrmFunc00010.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
var
	alpha: Double;
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  alpha := DegToRad(newTime*60);
  FSphereA.Position.SetPoint(1.5*cos(alpha), 1.5*sin(alpha), 1.5*sin(alpha));
  alpha:=alpha+PI/2;
  FSphereB.Position.SetPoint(1.5*cos(alpha), 1.5*sin(alpha), 1.5*sin(alpha));
  alpha:=alpha+PI/2;
  FSphereC.Position.SetPoint(1.5*cos(alpha), 1.5*sin(alpha), 1.5*sin(alpha));
  inherited;
end;

procedure TfrmFunc00010.CREATE_OBJECT;
begin
  inherited;
  FSphereA  := TGLSphere.Create(Self);
  FSphereB  := TGLSphere.Create(Self);
  FSphereC  := TGLSphere.Create(Self);
  FDCCentral:= TGLDummyCube.Create(Self);
  FSphere		:= TGLSphere.Create(Self);
  FTorus		:= TGLTorus.Create(Self);
  FCone	 		:= TGLCone.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FLight);
  FDummy.AddChild(FSphereA);
  FDummy.AddChild(FSphereB);
  FDummy.AddChild(FSphereC);
  FDummy.AddChild(FDCCentral);
  FDCCentral.AddChild(FTorus);
  FDCCentral.AddChild(FCone);
  FDCCentral.AddChild(FSphere);

  with FCamera do
  begin
  	DepthOfView := 150;
    FocalLength := 150;
    Position.AsVector := VectorMake(5,2.5,2.5);
  	TargetObject := FDCCentral;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(20,50,30);
  end;

  with FDummy do
  begin
  	ShowAxes := True;
    VisibleAtRunTime:= True;
  end;

  with FSphereA do
  begin
  	Bottom := -90;
    Material.BlendingMode := bmTransparency;
    Material.FrontProperties.Diffuse.AsWinColor := clYellow;
    Position.AsVector := VectorMake(2,0,0);
    Radius := 0.5;
    Slices := 16;
    Stacks := 16;
    Start := 0;
    Stop := 360;
    Top := 90;
  end;

  with FSphereB do
  begin
    Material.BlendingMode := bmTransparency;
    Material.FrontProperties.Diffuse.AsWinColor := clMoneyGreen;
    Position.AsVector := VectorMake(-2,0,0);
    Radius := 0.5;
    Slices := 16;
    Stacks := 16;
    Start := 0;
    Stop := 360;
    Top := 90;
  end;

  with FDCCentral do
  begin
  	AMalgamate := True;
  end;

  with FTorus do
  begin
  	MajorRadius := 0.8;
  	MinorRadius := 0.1;
    Rings := 25;
    Sides := 15;
    Up.AsVector := VectorMake(0,0,-1);
  end;

  with FCone do
  begin
  	BottomRadius := 0.3;
    Direction.AsVector := VectorMake(0,0,1);
    Height := 2;
    Loops := 1;
    Material.FrontProperties.Diffuse.AsWinColor := clRed;
    Position.AsVector := VectorMake(0,0.3,0);
    Slices := 16;
    Stacks := 4;
  end;

  with FSphere do
  begin
  	Material.BlendingMode := bmAdditive;
    Material.FrontProperties.Diffuse.AsWinColor := clSkyBlue;
    Radius := 0.6;
    Slices := 16;
    Stacks := 16;
    Start := 0;
    Stop := 360;
  end;

end;

procedure TfrmFunc00010.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
	FPanel := TPanel.Create(Self);
  with FPanel do
  begin
    Caption := '';
    Left := 5;
    Top := 5;
    Width := 180;
    Height := 180;
  end;

  FLabel1 := TLabel.Create(Self);
  with FLabel1 do
  begin
  	Caption := 'Central Objects';
  	Font.Style := Font.Style + [fsBold];
    Left := 1;
    Top := 1;
    Parent := FPanel;
  end;

  for I := 0 to Length(FRadioBtns)-1 do
  begin
   	FRadioBtns[I] := TRadioButton.Create(Self);
    with FRadioBtns[I] do
    begin
    	Tag := I;
      Left := 10;
      Height := 20;
      Width := 150;
      Top := FLabel1.Top + FLabel1.Height + 5;
      Top := Top + I * Height;
      case I of
      	0: Caption := 'Sphere, Torus, Cone';
        1: Caption := 'Torus, Sphere, Cone';
        2: Caption := 'Torus, Cone, Sphere';
      end;
      OnClick := DO_RadioBtnClick;
      Parent := FPanel;
    end;
  end;

  FLabel2 := TLabel.Create(Self);
  with FLabel2 do
  begin
  	Caption := 'Orbiting spheres :';
  	Font.Style := Font.Style + [fsBold];
    Left := 1;
    Top := FRadioBtns[2].Top + FRadioBtns[2].Height + 10;
    Parent := FPanel;
  end;

  for I := 0 to Length(FCheckBox)-1 do
  begin
   	FCheckBox[I] := TCheckBox.Create(Self);
    with FCheckBox[I] do
    begin
    	Tag := I;
      Left := 10;
      Height := 20;
      Width := 150;
      Top := FLabel2.Top + FLabel2.Height + 5;
      Top := Top + I * Height;
      case I of
      	0: Caption := 'Additive blending';
        1: Caption := 'osFarthestFirst';
      end;
      OnClick := DO_CheckBoxClick;
      Parent := FPanel;
    end;
  end;

	FPanel.Parent := FViewer;

end;

procedure TfrmFunc00010.DO_RadioBtnClick(Sender: TObject);
begin
	case TComponent(Sender).Tag of
  	0:begin
   			FSphere.MoveUp;
   			FSphere.MoveUp;
    	end;
    1:begin
   			FSphere.MoveUp;
   			FSphere.MoveUp;
   			FSphere.MoveDown;
    	end;
    2:begin
   			FSphere.MoveDown;
   			FSphere.MoveDown;
    	end;
  end;
end;


procedure TfrmFunc00010.DO_CheckBoxClick(Sender: TObject);
begin
	case TComponent(Sender).Tag of
  	0:begin
   			if TCheckBox(Sender).Checked then FSphereA.Material.BlendingMode := bmAdditive
   			else 															FSphereA.Material.BlendingMode := bmTransparency;
   			FSphereB.Material.BlendingMode := FSphereA.Material.BlendingMode;
        FSphereC.Material.BlendingMode := FSphereA.Material.BlendingMode;
    	end;
    1:begin
   			if TCheckBox(Sender).Checked then FDummy.ObjectsSorting := osRenderFarthestFirst
   			else 															FDummy.ObjectsSorting := osNone;
    	end;
  end;
end;

end.
