unit fFunc00011;

interface

uses Classes, Controls, Graphics, fFunc, GLScene, GLObjects, GLPolyhedron, GLBitmapFont, GLHUDObjects;

type
	TfrmFunc00011 = class(TfrmFunc)
  private
 		LastTime: Double;
		PickedObject: TGLBaseSceneObject;
  protected
  	FCube: TGLCube;
    FOctahedron: TGLSphere;
    FDodecahedron: TGLDodecahedron;
    FBmpFont: TGLBitmapFont;
    FHUDText1: TGLHUDText;
    FHUDText2: TGLHUDText;
  	procedure CREATE_OBJECT; override;
		procedure DO_ViewerClick(Sender: TObject); override;
    procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses SysUtils, VectorGeometry, GLBehaviours;

{ TfrmFunc00011 }

procedure TfrmFunc00011.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 10, newTime);
  if not (PickedObject is TGLHUDText) then
	 	if Assigned(PickedObject) then//Torque: x;Ox;x;
				GetOrCreateInertia(PickedObject).ApplyTorque(deltaTime, 200, 0, 0);
  inherited;
end;

procedure TfrmFunc00011.DO_ViewerClick(Sender: TObject);
var
	I: Integer;
	mass: Single;
begin
	if Assigned(PickedObject) and (PickedObject is TGLHUDText) then
  begin
					if PickedObject = FHUDText1 then 	Mass := 1
  	else 	if PickedObject = FHUDText2 then 	Mass := 2
    else 																		Mass := 0;
    for I := 0 to FDummy.Count-1 do
    	GetOrCreateInertia(FDummy.Children[i]).Mass := Mass;
  end;
end;

procedure TfrmFunc00011.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  PickedObject := FViewer.Buffer.GetPickedObject(X, Y);
  if Assigned(PickedObject) then
  	FViewer.Cursor := crHandPoint
  else FViewer.Cursor := crDefault;
end;

procedure TfrmFunc00011.CREATE_OBJECT;
begin
  inherited;
  FCube					:= TGLCube.Create(Self);
  FOctahedron		:= TGLSphere.Create(Self);
  FDodecahedron	:= TGLDodecahedron.Create(Self);
  FBmpFont 			:= TGLBitmapFont.Create(Self);
  FHUDText1			:= TGLHUDText.Create(Self);
  FHUDText2			:= TGLHUDText.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FHUDText1);
  FScene.Objects.AddChild(FHUDText2);
  FDummy.AddChild(FCube);
  FDummy.AddChild(FOctahedron);
  FDummy.AddChild(FDodecahedron);

  with FCamera do
  begin
  	DepthOfView := 100;
    FocalLength := 100;
    Position.AsVector := VectorMake(10,5,0);
    SceneScale := 2;
  	TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(15,10,-10);
  end;

  with FDummy do
  begin
    ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FCube do
  begin
  	Material.FrontProperties.Diffuse.Color := VectorMake(0.498039,1,0,1);
    Position.AsVector := VectorMake(0,0,2);
  end;

  with FDodecahedron do
  begin
  	Material.FrontProperties.Diffuse.AsWinColor := clRed;
  end;

  with FOctahedron do
  begin
  	Material.FrontProperties.Diffuse.Color := VectorMake(0.6666667,0.6666667,1,1);
    Position.AsVector := VectorMake(0,0,-2);
    Radius := 0.5;
    Slices := 4;
    Stacks := 2;
    Start := 0;
    Stop := 360;
  end;

  with FBmpFont do
  begin
  	CharHeight := 30;
    CharWidth := 30;
    GlyphsIntervalX := 1;
    GlyphsIntervalY := 1;
    HSpace := 3;
    VSpace := 6;
    with Ranges.Add do
    begin
    	StartASCII := #32;
      StopASCII := 'Z';
      StartGlyphIdx := 0;
    end;
    with Ranges.Add do
    begin
    	StartASCII := 'a';
      StopASCII := 'z';
      StartGlyphIdx := 33;
    end;
  	Glyphs.LoadFromFile('.\DATA\DARKGOLD_FONT.BMP');
  end;

  with FHUDText1 do
  begin
  	Alignment := taLeftJustify;
    BitmapFont := FBmpFont;
    Position.AsVector := VectorMake(10,600,0);
		Text := 'Set Mass to 1';
  end;

  with FHUDText2 do
  begin
  	Alignment := taLeftJustify;
    BitmapFont := FBmpFont;
    Position.AsVector := VectorMake(700,600,0);
		Text := 'Set Mass to 2';
  end;

  LastTime := Now * 3600 * 24;

	GetOrCreateInertia(FCube.Behaviours).RotationDamping.SetDamping(1,1,0);
  GetOrCreateInertia(FDodecahedron.Behaviours).RotationDamping.SetDamping(10, 1, 0.01);
  GetOrCreateInertia(FOctahedron).RotationDamping.SetDamping(0, 0, 0.01);
  
end;

end.
