unit fFunc00012;

interface

uses Windows, Classes, Graphics, fFunc, GLScene, GLObjects, GLTexture, GLCollision, GLBitmapFont, GLHUDObjects;

type
	TfrmFunc00012 = class(TfrmFunc)
  protected
  	FSphere: TGLSphere;
  	FSphereX: array[0..19] of TGLSphere;
    FSphereY: array[0..19] of TGLSphere;
    FSphereZ: array[0..19] of TGLSphere;
    FCollisionManager: TCollisionManager;
    FBmpFont: TGLBitmapFont;
    FHUDText1: TGLHUDText;
		procedure CREATE_OBJECT; override;
    procedure DO_ON_Collision(Sender: TObject; object1, object2: TGLBaseSceneObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses SysUtils, VectorGeometry;

{ TfrmFunc00012 }

procedure TfrmFunc00012.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 30, newTime);
  FCollisionManager.CheckCollisions;
  inherited;
end;

procedure TfrmFunc00012.DO_ON_Collision(Sender: TObject; object1, object2: TGLBaseSceneObject);
begin
	FHUDText1.Text := 'Collision:' + object1.Name + ' and ' + object2.Name;
end;

procedure TfrmFunc00012.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FBmpFont 					:= TGLBitmapFont.Create(Self);
  FHUDText1					:= TGLHUDText.Create(Self);
  FSphere 					:= TGLSphere.Create(Self);
  FCollisionManager := TCollisionManager.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
	FScene.Objects.AddChild(FSphere);
  FScene.Objects.AddChild(FHUDText1);

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 50;
    Position.AsVector := VectorMake(5,3,2);
    SceneScale := 2;
  	TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(20,20,20);
  end;

  with FDummy do
  begin
    ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FBmpFont do
  begin
  	CharHeight := 30;
    CharWidth := 30;
    GlyphsIntervalX := 1;
    GlyphsIntervalY := 1;
    HSpace := 3;
    VSpace := 6;
    with Ranges.Add do
    begin
    	StartASCII := #32;
      StopASCII := 'Z';
      StartGlyphIdx := 0;
    end;
    with Ranges.Add do
    begin
    	StartASCII := 'a';
      StopASCII := 'z';
      StartGlyphIdx := 33;
    end;
  	Glyphs.LoadFromFile('.\DATA\DARKGOLD_FONT.BMP');
  end;

  with FHUDText1 do
  begin
  	Alignment := taLeftJustify;
    BitmapFont := FBmpFont;
    Position.AsVector := VectorMake(0,600,0);
		Text := 'CheckCollisions() does not work.';
  end;

  Randomize;

  with FSphere do
  begin
  	Name := 'SHR0';
    Material.FrontProperties.Diffuse.Color := VectorMake(1,1,1,0.7);
    Material.BlendingMode := bmTransparency;//Unit: GLTexture;
    Position.AsVector := VectorMake(1.5,1.5,1.5);
    Radius := 1;
    Slices := 32;
    Stacks := 32;
    Start := 0;
    Stop := 360;
    with Behaviours.GetOrCreate(TGLBCollision) as TGLBCollision do
    begin
    	BoundingMode := cbmSphere;
      Manager := FCollisionManager;
    end;
  end;

	for I := 0 to Length(FSphereX)-1 do
  begin
  	FSphereX[I] := TGLSphere.Create(Self);
    FDummy.AddChild(FSphereX[I]);
    with FSphereX[I] do
    begin
  		Name := 'SHRX' + IntToStr(I);
    	Material.BlendingMode := bmAdditive;//bmTransparency;//Unit: GLTexture;
  		Material.FrontProperties.Diffuse.AsWinColor := RGB(Random($100),Random($100),Random($100));
      Material.FrontProperties.Diffuse.Alpha := 0.5;
    	Position.AsVector := VectorMake(I-10,0,0);
    	Radius := 0.3;
    	Slices := 32;
    	Stacks := 32;
    	Start := 0;
    	Stop := 360;
      with Behaviours.GetOrCreate(TGLBCollision) as TGLBCollision do
      begin
        BoundingMode := cbmSphere;
        Manager := FCollisionManager;
      end;
    end;
  end;

	for I := 0 to Length(FSphereY)-1 do
  begin
  	FSphereY[I] := TGLSphere.Create(Self);
    FDummy.AddChild(FSphereY[I]);
    with FSphereY[I] do
    begin
  		Name := 'SHRY' + IntToStr(I);
  		Material.FrontProperties.Diffuse.AsWinColor := RGB(Random($100),Random($100),Random($100));
    	Position.AsVector := VectorMake(0,I-10,0);
    	Radius := 0.3;
    	Slices := 32;
    	Stacks := 32;
    	Start := 0;
    	Stop := 360;
      with Behaviours.GetOrCreate(TGLBCollision) as TGLBCollision do
      begin
        BoundingMode := cbmSphere;
        Manager := FCollisionManager;
      end;
    end;
  end;

	for I := 0 to Length(FSphereZ)-1 do
  begin
  	FSphereZ[I] := TGLSphere.Create(Self);
    FDummy.AddChild(FSphereZ[I]);
    with FSphereZ[I] do
    begin
  		Name := 'SHRZ' + IntToStr(I);
  		Material.FrontProperties.Diffuse.AsWinColor := RGB(Random($100),Random($100),Random($100));
    	Position.AsVector := VectorMake(0,0,I-10);
    	Radius := 0.3;
    	Slices := 32;
    	Stacks := 32;
    	Start := 0;
    	Stop := 360;
      with Behaviours.GetOrCreate(TGLBCollision) as TGLBCollision do
      begin
        BoundingMode := cbmSphere;
        Manager := FCollisionManager;
      end;
    end;
  end;

  with FCollisionManager do
  begin
  	OnCollision := DO_ON_Collision;
  end;
  
end;

end.
