unit fFunc00013;

interface

uses Windows, Classes, Controls, StdCtrls, Graphics, fFunc, GLVectorFileObjects, GLTexture, VectorLists;

type
	TfrmFunc00013 = class(TfrmFunc)
  protected
    FCheckBox: TCheckBox;
    FFreeForm: TGLFreeForm;//Unit: GLVectorFileObjects
    procedure CREATE_OBJECT; override;
    procedure DESTROY_OBJECT; override;
		procedure DO_ViewerAfterRender(Sender: TObject); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    function FileName3DS: string; virtual;
  public
		Angulo: Single;
    Fig: TAffineVectorList;//Unit: VectorLists
    mx2, my2: Integer;
	end;

implementation

uses GLFile3ds, JPEG, VectorGeometry;

{ TfrmFunc00013 }

procedure TfrmFunc00013.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
const
	AMPLITUD_ALETAZO 			= 20;
  FRECUENCIA_DE_PATADA 	= 15;
var
	J: Integer;
  v: TAffineVector;
  f: Single;
begin
 	if ((mx<>mx2) or (my<>my2)) then
 	begin
 		FCamera.MoveAroundTarget(my-my2, mx-mx2);
    mx := mx2;
    my := my2;
 	end;

 	if FCheckBox.Checked then
	begin
   	Angulo := NewTime;
   	for J := 0  to FFreeForm.MeshObjects.Items[0].Vertices.Count -1 do
     	with FFreeForm.MeshObjects.Items[0].Vertices do
     	begin
       	f := sin(Angulo*FRECUENCIA_DE_PATADA) * (Sqr(Items[J][0])/AMPLITUD_ALETAZO);
       	v[0] := 0;
        v[1] := 0;
        v[2] := f;
       	TranslateItem(J, v);
     	end;
   	FFreeForm.StructureChanged;
  end;

  inherited;
end;


procedure TfrmFunc00013.DO_ViewerAfterRender(Sender: TObject);
begin
	FFreeForm.MeshObjects.Items[0].Vertices.Assign(Fig);
end;

procedure TfrmFunc00013.DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mx	:= X;
  my	:= Y;
  mx2	:= X;
  my2	:= Y;
end;

procedure TfrmFunc00013.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if ssLeft in Shift then
  begin
    mx2 := X;
    my2 := Y;
  end;
end;

procedure TfrmFunc00013.DESTROY_OBJECT;
begin
	Fig.Free;
  inherited;
end;

procedure TfrmFunc00013.CREATE_OBJECT;
begin
  inherited;
	FCheckBox	:= TCheckBox.Create(Self);
  FFreeForm := TGLFreeForm.Create(Self);

  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FCamera);
  FDummy.AddChild(FFreeForm);
  FCamera.AddChild(FLight);

  with FCheckBox do
  begin
  	Caption := 'Animacion dinamica';
    Left := 1;
    Top := 1;
    Width := 120;
    Height := 20;
    Color := clBtnFace;
    Checked := True;
    Visible := True;
    Parent := FViewer;
  end;
  
  with FViewer do
  begin
    Buffer.BackgroundColor := RGB(0,0,160);
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 50;
    Position.AsVector := VectorMake(5,3,2);
    SceneScale := 2;
  	TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(0,0,0);
  end;

  with FDummy do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FFreeForm do
  begin
    LoadFromFile(FileName3DS);//Muse uses unit GLFile3ds.
  	AutoScaling.AsVector := VectorMake(1,1,1);
    Direction.AsVector := VectorMake(1,0,0);
    Position.AsVector := VectorMake(0,0,0);
    Scale.AsVector := VectorMake(0.5,0.5,0.5);
    Material.FrontProperties.Ambient.Color := VectorMake(0.1019608,0.7450981,0.9764706,1);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.439216,0.858824,0.576471,1);
    Material.FrontProperties.Emission.Color := VectorMake(0.3,0.3,0.3,1);
    Material.FrontProperties.PolygonMode := pmLines;
    Material.FrontProperties.Shininess := 110;
    Material.FaceCulling := fcNoCull;
    NormalsOrientation:= mnoInvert;
(*
    Material.FrontProperties.PolygonMode := pmLines;
    Material.FrontProperties.Shininess := 100;
    Material.FrontProperties.Ambient.Color := VectorMake(0.1019608,0.7450981,0.9764706,1);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.439216,0.858824,0.576471,1);
    Material.FrontProperties.Emission.Color := VectorMake(0.3,0.3,0.3,1);
    Material.FaceCulling := fcNoCull;
    NormalsOrientation:= mnoInvert;//mnoDefault;//mnoInvert;
*)
    PitchAngle := 90;
    Up.AsVector := VectorMake(-0.9940332,0,-0.1090784);
  end;

 	Fig := TAffineVectorList.Create;
 	Fig.Assign(FFreeForm.MeshObjects.Items[0].Vertices);

end;

function TfrmFunc00013.FileName3DS: string;
begin
	Result := '.\DATA\dolphin.3ds';
end;

end.
