unit fFunc00014;

interface

uses Windows, Classes, Graphics, Controls, StdCtrls, fFunc, GLScene, GLVectorFileObjects,
	GLHeightData, GLTerrainRenderer, VectorLists, GLTexture;

type
	TfrmFunc00014 = class(TfrmFunc)
  protected
    FCheckBox: TCheckBox;
    FFreeForm: TGLFreeForm; 							//Unit: GLVectorFileObjects
    FBitmapHDS: TGLBitmapHDS; 						//Unit: GLHeightData
    FTerrainRenderer: TGLTerrainRenderer;	//Unit: GLTerrainRenderer
  	procedure CREATE_OBJECT; override;
    procedure DESTROY_OBJECT; override;
		procedure DO_ViewerAfterRender(Sender: TObject); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  public
    mx2, my2: Integer;
  end;

var
	Angulo: Single;
	Fig: TAffineVectorList;//Unit: VectorLists
    
implementation

uses GLFile3ds, JPEG, VectorGeometry;

{ TfrmFunc00014 }

procedure TfrmFunc00014.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
const
	AMPLITUD_ALETAZO 			= 20;
  FRECUENCIA_DE_PATADA 	= 15;
var
	J: Integer;
  v: TAffineVector;
  f: Single;
begin
 	if ((mx<>mx2) or (my<>my2)) then
 	begin
 		FCamera.MoveAroundTarget(my-my2, mx-mx2);
    mx := mx2;
    my := my2;
 	end;

 	if FCheckBox.Checked then
	begin
   	Angulo := NewTime;
   	for J := 0  to FFreeForm.MeshObjects.Items[0].Vertices.Count -1 do
     	with FFreeForm.MeshObjects.Items[0].Vertices do
     	begin
       	f := sin(Angulo*FRECUENCIA_DE_PATADA) * (Sqr(Items[J][0])/AMPLITUD_ALETAZO);
       	v[0] := 0;
        v[1] := 0;
        v[2] := f;
       	TranslateItem(J, v);
     	end;
   FFreeForm.StructureChanged;
 end;
 inherited;
end;

procedure TfrmFunc00014.DO_DummyProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	FDummy.Move(3);	
end;

procedure TfrmFunc00014.DO_ViewerAfterRender(Sender: TObject);
begin
	FFreeForm.MeshObjects.Items[0].Vertices.Assign(Fig);
end;

procedure TfrmFunc00014.DO_ViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  mx	:= X;
  my	:= Y;
  mx2	:= X;
  my2	:= Y;
end;

procedure TfrmFunc00014.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
  if ssLeft in Shift then
  begin
  	mx2 := X;
    my2 := Y;
  end;
end;

procedure TfrmFunc00014.DESTROY_OBJECT;
begin
	Fig.Free;
  inherited;
end;

procedure TfrmFunc00014.CREATE_OBJECT;
begin
  inherited;
  FCheckBox				:= TCheckBox.Create(Self);
  FFreeForm				:= TGLFreeForm.Create(Self);
  FBitmapHDS			:= TGLBitmapHDS.Create(Self);
  FTerrainRenderer:= TGLTerrainRenderer.Create(Self);

  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FTerrainRenderer);
  FDummy.AddChild(FCamera);
  FDummy.AddChild(FFreeForm);
  FCamera.AddChild(FLight);

  with FBitmapHDS do
  begin
  	InfiniteWrap := True;
   	MaxPoolSize := 8 * 1024 * 1024;
    Picture := nil;
   	Picture.LoadFromFile('.\DATA\TERRAIN.BMP');
  end;

	with FViewer do
  begin
   	Buffer.BackgroundColor := RGB(0,0,160);
    Buffer.DepthTest := True;
    Buffer.FaceCulling := False;
    Buffer.FogEnable := True;
    Buffer.Lighting := True;
    Buffer.ShadeModel := smFlat;
    with Buffer.FogEnvironment do
    begin
      FogColor.AsWinColor := RGB(0,0,110);
      FogDistance := fdEyePlane;//Unit: GLScene
      FogEnd := 1000;
      FogMode := fmLinear;
      FogStart := 200;
      FogStart := -FogStart;
    end;
  end;

  with FDummy do
  begin
  	Direction.AsVector := VectorMake(-1,0,-4.371139E-8);
    Position.AsVector := VectorMake(1,5,1);
    Scale.AsVector := VectorMake(3,3,3);
    TurnAngle := -90;
  end;

  with FCamera do
  begin
  	DepthOfView := 500;
    FocalLength := 50;
    Position.AsVector := VectorMake(-3,-1,4);
    TargetObject := FDummy;
  end;

  with FFreeForm do
  begin
  	AutoScaling.AsVector := VectorMake(1,1,1);
    Direction.AsVector := VectorMake(0,1,0);

    Material.FrontProperties.Ambient.Color := VectorMake(0.1019608,0.7450981,0.9764706,1);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.439216,0.858824,0.576471,1);
    Material.FrontProperties.Emission.Color := VectorMake(0.3,0.3,0.3,1);
    Material.FrontProperties.PolygonMode := pmLines;
    Material.FrontProperties.Shininess := 110;
    Material.FaceCulling := fcNoCull;
    NormalsOrientation:= mnoInvert;

    RollAngle := 90;
    Scale.AsVector := VectorMake(0.5,0.5,0.5);
    Up.AsVector := VectorMake(-0.9940332,0,-0.1090784);
    UseMeshMaterials := True;
    LoadFromFile('.\DATA\DOLPHIN.3DS');//Muse uses unit GLFile3ds.
  end;

  with FLight do
  begin
  	ConstAttenuation := 1;
    Shining := True;
  end;


  with FTerrainRenderer do
  begin
  	CLODPrecision := 100;
    Direction.AsVector := VectorMake(0,1,0);
    HeightDataSource := FBitmapHDS;
    Material.FaceCulling := fcNoCull;//fcBufferDefault;//Unit: GLTexture
    Material.Texture.Disabled := False;
    MaxCLODTriangles := 65536;
    OcclusionFrameSkip := 0;
    OcclusionTesselate := totTesselateIfVisible;
    QualityDistance := 150;
    QualityStyle := hrsFullGeometry;		//Unit: GLTexture
    Scale.AsVector := VectorMake(4,4,0.25);
    Up.AsVector := VectorMake(0,0,1);
    TileSize := 16;
    TilesPerTexture := 256 / TileSize;
    Material.Texture.Image.LoadFromFile('.\DATA\TEX.JPG');
  end;

  with FCheckBox do
  begin
  	Caption := 'Animacion dinamica';
    Left := 1;
    Top := 1;
    Width := 120;
    Height := 20;
    Color := clBtnFace;
    Checked := True;
    Visible := True;
    Parent := FViewer;
  end;

  FTimer.Interval := 300;
  
 	Fig := TAffineVectorList.Create;
 	Fig.Assign(FFreeForm.MeshObjects.Items[0].Vertices);

end;

end.
