unit fFunc00015;

interface

uses fFunc, GLScene, GLObjects, GLParticles, GLBehaviours, GLTexture, GLLensFlare, GLMisc, GLUtils;

type
	TfrmFunc00015 = class(TfrmFunc)
  protected
    FParticles: TGLParticles;//Unit: GLParticles
    FSprite: TGLSprite;      //Unit: GLObjects
    procedure CREATE_OBJECT; override;
		procedure DESTROY_OBJECT; override;
    procedure DO_Resize(Sender: TObject); override;
    procedure DO_Timer(Sender: TObject); override;
    procedure DO_ActivateParticle(Sender: TObject; particle: TGLBaseSceneObject); virtual;
    procedure DO_SpriteProgress(Sender: TObject; const deltaTime, newTime: Double); virtual;
  end;

implementation

uses SysUtils, VectorGeometry;

{ TfrmFunc00015 }

procedure TfrmFunc00015.CREATE_OBJECT;
begin
  inherited;
  FTimer.Enabled := False;

  Randomize;

  FParticles := TGLParticles.Create(Self);
  FSprite := TGLSprite.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FParticles);
  FParticles.AddChild(FSprite);

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FViewer do
  begin
    Buffer.FaceCulling := True;//////
  end;

  with FTimer do
  begin
  	Interval := 300;
  end;

  with FCamera do
  begin
  	Position.AsVector := VectorMake(15,0,0);
    TargetObject := FParticles;
  end;

  with FParticles do
  begin
  	with TGLBInertia(Behaviours.GetOrCreate(TGLBInertia)) do//Unit: GLBehaviours
    begin
    	Mass := 1;
    	PitchSpeed := 0;
      RollSpeed := 0;
      TurnSpeed := 30;
      OnActivateParticle := DO_ActivateParticle;
      ObjectsSorting := osNone;
    end;
    ObjectsSorting := osNone;//Unit: GLMisc
    Scale.AsVector := VectorMake(2,2,2);
  end;

  with FSprite do
  begin
  	Scale.AsVector := VectorMake(3,3,3);
  	MirrorU := False;
    MirrorV := False;
    NoZWrite := False;
		AlphaChannel := 1;
    ObjectsSorting := osNone;
    Material.BlendingMode := bmAdditive;//Unit: GLTextrue
		Material.BackProperties.Emission.Color := VectorMake(0,0,0,1);
    with Material.FrontProperties do
    begin
    	Ambient.Color := VectorMake(0,0,0,1);
    	Diffuse.Color := VectorMake(0,0,0,1);
    	Emission.Color := VectorMake(0,0,0,1);
    end;
    with Material.Texture do
    begin
    	Disabled := False;
      TextureFormat := tfIntensity;
      TextureMode := tmModulate;
      TextureWrap := twBoth;
      Image.LoadFromFile('.\DATA\FLARE1.BMP');
    end;
    OnProgress := DO_SpriteProgress;
  end;

  FTimer.Enabled := True;

end;

procedure TfrmFunc00015.DESTROY_OBJECT;
begin
	if Assigned(FParticles) then
  begin
  	FParticles.KillParticles;
    FreeAndNil(FParticles);
  end;
  inherited;
end;

procedure TfrmFunc00015.DO_ActivateParticle(Sender: TObject; particle: TGLBaseSceneObject);
begin
  with TGLSprite(particle) do
  begin
    Material.Texture.Disabled := False;
  	Material.BlendingMode := bmTransparency;
    Material.Texture.TextureMode := tmModulate;
  	with Material.FrontProperties do
    begin
	   	Emission.Color := PointMake(Random, Random, Random);
     	Diffuse.Alpha := 0;
    end;
    TagFloat := FCadencer.CurrentTime;
  end;
end;

procedure TfrmFunc00015.DO_Resize(Sender: TObject);
begin
  FCamera.FocalLength := 50 * Width / 280;
end;

procedure TfrmFunc00015.DO_SpriteProgress(Sender: TObject; const deltaTime, newTime: Double);
var
	life: Double;
begin
	with TGLSprite(Sender) do
  begin
    life := (newTime-TagFloat);// calculate for how long we've been living
          if life>10 then FParticles.KillParticle(TGLSprite(Sender))					//old particle to kill
    else 	if life<01 then Material.FrontProperties.Diffuse.Alpha:=life				//baby particles become brighter in their 1st second of life...
    else 									Material.FrontProperties.Diffuse.Alpha:=(9-life)/9;	//...and slowly disappear in the darkness
  end;
end;

procedure TfrmFunc00015.DO_Timer(Sender: TObject);
begin
	if Assigned(FParticles) then
  begin
    with TGLSprite(FParticles.CreateParticle).Position do
    begin
      X := 10 * (Random - 0.5);
      Y := 10 * (Random - 0.5);
      Z := 10 * (Random - 0.5);
    end;
  end;
  inherited;
end;

end.
