unit fFunc00016;

interface

uses Forms, Classes, Graphics, Controls, StdCtrls, ExtCtrls, ComCtrls,
	fFunc, GLScene, GLObjects, GLExtrusion, GLTexture, GLUtils;

type
	TfrmFunc00016 = class(TfrmFunc)
  protected
  	FPanel: TPanel;
  	FLabels: array[0..3] of TLabel;
    FCheckBoxes: array[0..3] of TCheckBox;
    FTrackBars: array[0..2] of TTrackBar;
    FSphere: TGLSphere;
    FRotationSolid: TGLRevolutionSolid;
    procedure SETUP_USER_INTERFACE; override;
    procedure CREATE_OBJECT; override;
    procedure DO_CheckboxClick(Sender: TObject);
    procedure DO_TrackbarChange(Sender: TObject);
		procedure DO_Timer(Sender: TObject); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses JPeg, SysUtils, VectorGeometry;

{ TfrmFunc00016 }

procedure TfrmFunc00016.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;

  FSphere := TGLSphere.Create(Self);
  FRotationSolid := TGLRevolutionSolid.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FRotationSolid);
  FDummy.AddChild(FSphere);
  with FViewer do
  begin
    Buffer.BackgroundColor := clBackground;
    Buffer.DepthTest := True;
    Buffer.FaceCulling := True;
  end;

  with FCamera do
  begin
  	FocalLength := 100;
  	Position.AsVector := VectorMake(4,0,0);
    TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(50,50,50);
    Shining := True;
  end;

  with FDummy do
  begin
//  	ShowAxes := True;
  //  VisibleAtRuntime := True;
  end;

  with FRotationSolid do
  begin
  	with Material.Texture do
    begin
    	Disabled := False;
      MagFilter := maLinear;
      MappingMode := tmmUser;
      MappingSCoordinates.AsVector := VectorMake(1,0,0,0);
      MappingTCoordinates.AsVector := VectorMake(1,0,0,0);
      MinFilter := miLinear;
      NormalMapScale := 0.125;
      TextureMode := tmModulate;
    end;
  	Division := 100;
    NormalDirection := ndOutside;
    Normals := nsSmooth;
  	ShowAxes := True;
  	Parts := [rspOutside];
    Slices := 24;
    SplineMode := lsmCubicSpline;
    StartAngle := 0;
    StopAngle := 360;
    Visible := True;
    Up.AsVector := VectorMake(-0.6524543,0.7578281,0);
    for I := 0 to 11 do
    	with Nodes.Add do
      	case I of
        	0: AsVector := VectorMake(0,0.9,0);
          1: AsVector := VectorMake(0.4,0.8,0);
          2: AsVector := VectorMake(0.45,0.5,0);
          3: AsVector := VectorMake(0.25,0.3,0);
          4: AsVector := VectorMake(0.25,-0.1,0);
        	5: AsVector := VectorMake(0.6,-0.5,0);
          6: AsVector := VectorMake(0.6,-0.9,0);
          7: AsVector := VectorMake(0.59,-0.95,0);
          8: AsVector := VectorMake(0.6,-1,0);
          9: AsVector := VectorMake(0.6,-1,0);
        	10: AsVector := VectorMake(0.5,-1,0);
          11: AsVector := VectorMake(0,-1,0);
        end;
  end;

  with FSphere do
  begin
  	Bottom := -90;
  	Radius := 0.5;
  	Slices := 32;
    Stacks := 32;
    Start := 0;
    Stop := 360;
  end;

  FRotationSolid.Material.Texture.Image.LoadFromFile('.\DATA\ashwood.jpg');
  FSphere.Material.Texture.Image.LoadFromFile('.\DATA\ashwood.jpg');
//	FCadencer.Enabled := False;
//  FCadencer.Scene := nil;

end;

procedure TfrmFunc00016.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  inherited;
end;

procedure TfrmFunc00016.DO_CheckboxClick(Sender: TObject);
begin
	with TCheckBox(Sender) do
		case Tag of
  		0: with FRotationSolid do if Checked then SplineMode := lsmCubicSpline else SplineMode := lsmLines;
    	1: with FRotationSolid do if Checked then Normals := nsSmooth else Normals := nsFlat;
	    2: with FRotationSolid.Material.Texture do Disabled := not Checked;
  	  3: with FRotationSolid.Material.Texture do if Checked then TextureMode := tmModulate else TextureMode := tmDecal;
  	end;
end;

procedure TfrmFunc00016.DO_Timer(Sender: TObject);
begin
  inherited;
  with TForm(Owner) do
  	Caption := Caption + '--' + Format('%d Triangles', [FRotationSolid.TriangleCount]);
end;

procedure TfrmFunc00016.DO_TrackbarChange(Sender: TObject);
var
	TB: TTrackBar;
begin
	TB := TTrackBar(Sender);
	with TB do
  	case Tag of
      0: FRotationSolid.Division := TB.Position;
      1: FRotationSolid.Slices := TB.Position;
    	2:with FRotationSolid do
        begin
        	StopAngle := TB.Position;
   				if TB.Position = 360 then Parts := Parts - [rspStartPolygon, rspStopPolygon]
   				else 											Parts := Parts + [rspStartPolygon, rspStopPolygon];
        end;
    end;
end;

procedure TfrmFunc00016.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
	FPanel := TPanel.Create(Self);
	for I := 0 to Length(FLabels)-1 		do FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FCheckBoxes)-1 do FCheckBoxes[I] := TCheckBox.Create(Self);
  for I := 0 to Length(FTrackBars)-1 	do FTrackBars[I] := TTrackBar.Create(Self);

  with FLabels[0] do
  begin
  	Font.Style := Font.Style + [fsBold];
    Caption := 'Option';
    SetBounds(5,5,100,20);
    Parent := FPanel;
  end;

	for I := 0 to Length(FCheckBoxes)-1 do
  	with FCheckBoxes[I] do
    begin
      case I of
        0: Caption := 'Spline interpolation';
        1: Caption := 'Normals smoothing';
        2: Caption := 'Texture map';
        3: Caption := 'Modulate texture';
      end;
    	Tag := I;
      Checked := True;
      OnClick := DO_CheckboxClick;
      SetBounds(10,25+20*I,130,20);
      Parent := FPanel;
    end;

  for I := 1 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
    	case I of
      	1: Caption := 'Slices';
        2: Caption := 'Divisions';
        3: Caption := 'Stop';
      end;
      SetBounds(10,110+40*(I-1),130,20);
      Parent := FPanel;
    end;

  for I := 0 to Length(FTrackBars)-1 do
  	with FTrackBars[I] do
    begin
    	case I of
      	0:begin Min := 4; Max := 64; Position := 24; Frequency := 16; end;
        1:begin Min := 1; Max := 30; Position := 10; Frequency := 10; end;
        2:begin Min :=30; Max :=360; Position :=360; Frequency := 45; end;
      end;
      Tag := I;
    	Height := 20;
      SliderVisible := True;
      ThumbLength := 10;
      TickMarks := tmBottomRight;
      OnChange := DO_TrackbarChange;
      SetBounds(3,128+40*I,144,20);
      Parent := FPanel;
    end;

  with FPanel do
  begin
  	SetBounds(10,10,150,240);
  	Parent := FViewer;
  end;


end;

end.
