unit fFunc00017;

interface

uses Windows, Classes, Graphics, Controls, fFunc, GLScene,
	GLObjects, GLGeomObjects, GLSpaceText, GLTexture, VectorGeometry;

type
	TfrmFunc00017 = class(TfrmFunc)
  private
    LastMouseWorldPos: TVector;
    MovingOnZ: Boolean;
    CurrentPick: TGLCustomSceneObject;
    function MouseWorldPos(X, Y: Integer): TVector;
  protected
  	FFloor: TGLCube;
    FCubeA: TGLCube;
    FCubeB: TGLCube;
    FArrows: array[0..2] of TGLArrowLine;
    FSpaceTexts: array[0..2] of TGLSpaceText;
  	procedure CREATE_OBJECT; override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); override;
		procedure DO_KeyPress(Sender: TObject; var Key: Char); override;
		procedure DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState); override;
    procedure DO_ViewerAfterRender(Sender: TObject); override;
  end;

const
  SelectionColor: TColorVector = (1, 0, 0, 1.000);

implementation

uses SysUtils;

{ TfrmFunc00017 }

procedure TfrmFunc00017.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FCadencer.Enabled := False;
  FCadencer.Scene := nil;

  FFloor := TGLCube.Create(Self);
  FCubeA := TGLCube.Create(Self);
  FCubeB := TGLCube.Create(Self);
  for I := 0 to Length(FArrows)-1 do
  	FArrows[I] := TGLArrowLine.Create(Self);
  for I := 0 to Length(FSpaceTexts)-1 do
  	FSpaceTexts[I] := TGLSpaceText.Create(Self);


  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FLight);
  FDummy.AddChild(FFloor);
  FDummy.AddChild(FCubeA);
  FDummy.AddChild(FCubeB);
  for I := 0 to Length(FArrows)-1 do
  	FDummy.AddChild(FArrows[I]);
  for I := 0 to Length(FSpaceTexts)-1 do
	  FDummy.AddChild(FSpaceTexts[I]);

	with FViewer do
  begin
    Buffer.DepthTest := True;
    Buffer.FaceCulling := True;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    Direction.AsVector := VectorMake(-0.8944272,0.4472136,0);
    FocalLength := 500;
    Position.AsVector := VectorMake(20,16,12);
    TargetObject := FDummy;
//    TargetObject := FFloor;
   	Up.AsVector := VectorMake(0,0,1);
  end;

  with FDummy do
  begin
  	CubeSize := 0.2;
    EdgeColor.Color := VectorMake(0.8666667,0.8666667,0.9098039,1);
    Position.AsVector := VectorMake(1,1,0.5);
    ShowAxes := True;
    VisibleAtRunTime := True;
  end;

  with FLight do
  begin
  	Ambient.Color := VectorMake(0.5,0.5,0.5,1);
    ConstAttenuation := 0.8;
    Diffuse.Color := VectorMake(0.9137255,0.9137255,0.9137255,1);
    Position.AsVector := VectorMake(16,13,12);
    Shining := True;
  end;

  with FFloor do
  begin
  	CubeDepth := 0.01;
    CubeHeight := 2;
    CubeWidth := 2;
    Position.AsVector := VectorMake(0,0,-1.02);
  end;

  with FCubeA do
  begin
  	Name := 'CubeA';
  	Direction.AsVector := VectorMake(1,0,0);
    with Material do
    begin
    	BlendingMode := bmTransparency;
    	BackProperties.Diffuse.Color := VectorMake(0.509804,0.5176471,0.6901961,0.889);
    	FrontProperties.Diffuse.Color := VectorMake(0.509804,0.5176471,0.6901961,0.859);
    end;
    Position.AsVector := VectorMake(0.1,0.1,-0.9);
    Scale.AsVector := VectorMake(0.2,0.2,0.2);
  end;

  with FCubeB do
  begin
  	Name := 'CubeB';
  	CubeDepth := 0.25;
    CubeHeight := 0.25;
    CubeWidth := 0.25;
    Position.AsVector := VectorMake(-0.4,0.4,-0.5);
  end;

  with FArrows[0] do
  begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.05;
    BottomRadius := 0.01;
    Direction.AsVector := VectorMake(1,0,0);
    HeadStackingStyle := ahssStacked;
    Height := 2.2;
    Loops := 1;
    Normals := nsSmooth;
    Parts := [alLine, alTopArrow];
    Position.AsVector := VectorMake(0.1,-1,-1);
    Slices := 64;
    Stacks := 16;
    TopArrowHeadHeight := 0.2;
    TopArrowHeadRadius := 0.05;
    TopRadius := 0.01;
    Up.AsVector := VectorMake(0,-4.371139E-8,1);
  end;

  with FArrows[1] do
  begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.2;
    BottomRadius := 0.01;
    Direction.AsVector := VectorMake(0,1,0);
    HeadStackingStyle := ahssStacked;
    Height := 2.2;
    Loops := 1;
    Normals := nsSmooth;
    Parts := [alLine, alTopArrow];
    Position.AsVector := VectorMake(-1,0.1,-1);
    Slices := 64;
    Stacks := 16;
    TopArrowHeadHeight := 0.2;
    TopArrowHeadRadius := 0.05;
    TopRadius := 0.01;
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FArrows[2] do
  begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.05;
    BottomRadius := 0.01;
    Direction.AsVector := VectorMake(0,0,1);
    HeadStackingStyle := ahssStacked;
    Height := 2.2;
    Loops := 1;
    Normals := nsSmooth;
    Parts := [alLine, alTopArrow];
    Position.AsVector := VectorMake(-1,-1,-0.5);
    Slices := 64;
    Stacks := 16;
    TopArrowHeadHeight := 0.2;
    TopArrowHeadRadius := 0.05;
    TopRadius := 0.01;
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FSpaceTexts[0] do
  begin
  	Adjust.Horz := haLeft;
    Adjust.Vert := vaBaseLine;
    AllowedDeviation := 0;
    AspectRatio := 0;
    CharacterRange := stcrAlphaNum;
  	Direction.AsVector := VectorMake(-0.7071068,-0.7071068,0);
    Extrusion := 0.3;
    Oblique := 0;
    Position.AsVector := VectorMake(1.6,-1,-1);
    Scale.AsVector := VectorMake(0.3,0.3,0.3);
    Text := 'X';
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FSpaceTexts[1] do
  begin
  	Adjust.Horz := haLeft;
    Adjust.Vert := vaBaseLine;
    AllowedDeviation := 0;
    AspectRatio := 0;
    CharacterRange := stcrAlphaNum;
  	Direction.AsVector := VectorMake(-0.7071068,-0.7071068,0);
    Extrusion := 0.3;
    Oblique := 0;
    Position.AsVector := VectorMake(-1,1.7,-1);
    Scale.AsVector := VectorMake(0.3,0.3,0.3);
    Text := 'Y';
    Up.AsVector := VectorMake(0,0,1);
  end;
  
  with FSpaceTexts[2] do
  begin
  	Adjust.Horz := haLeft;
    Adjust.Vert := vaBaseLine;
    AllowedDeviation := 0;
    AspectRatio := 0;
    CharacterRange := stcrAlphaNum;
  	Direction.AsVector := VectorMake(0.7071068,0.7071068,0);
    Extrusion := 0.3;
    Oblique := 0;
//    Position.AsVector := VectorMake(0.15,-0.1,1.5);
    Position.AsVector := VectorMake(-0.85,-1,1.2);
    Scale.AsVector := VectorMake(0.3,0.3,0.3);
    Text := 'Z';
    Up.AsVector := VectorMake(0,0,1);
  end;

  FCadencer.Scene := FScene;
  FCadencer.Enabled := True;
end;

procedure TfrmFunc00017.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  inherited;
end;

procedure TfrmFunc00017.DO_KeyPress(Sender: TObject; var Key: Char);
begin
  with FCamera do
  	case Key of
    	'2': MoveAroundTarget(+3, +0);
	    '4': MoveAroundTarget(+0, -3);
  	  '6': MoveAroundTarget(+0, +3);
    	'8': MoveAroundTarget(-3, +0);
	    '-': AdjustDistanceToTarget(1.1);
  	  '+': AdjustDistanceToTarget(1 / 1.1);
  end;
end;

procedure TfrmFunc00017.DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Assigned(currentPick) then
  	with currentPick do
    	case Key of
        VK_UP 	: if ssShift in Shift then Translate(0, 0, +0.3) 	else Translate(-0.3, 0, 0);
        VK_DOWN : if ssShift in Shift then Translate(0, 0, -0.3)	else Translate(+0.3, 0, 0);
        VK_LEFT : Translate(0, -0.3, 0);
        VK_RIGHT: Translate(0, +0.3, 0);
  		end;
end;

procedure TfrmFunc00017.DO_MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
	FCamera.AdjustDistanceToTarget(Power(1.1, WheelDelta / 120));
end;

procedure TfrmFunc00017.DO_ViewerAfterRender(Sender: TObject);
var
  ObjPos, WinPos: TAffineVector;
  Canvas: TCanvas;
begin
	Canvas := TCanvas.Create;
  try
  	with Canvas do
    begin
  		Handle := FViewer.RenderDC;
    	Brush.Style := bsClear;
      Font.Name := 'з';//'Verdana';
      Font.Color := clWhite;
      Font.Height := 24;

    	Pen.Color := clRed;
    	Pen.width := 1;
    	MoveTo(0,0);
    	LineTo(200,200);
    end;

    if Assigned(CurrentPick) then
    begin
    	SetVector(ObjPos, CurrentPick.AbsolutePosition);
      Canvas.TextOut(3, 3 + 1 * canvas.TextHeight('A'),
                     'jv: '+Format('New Object Position: Xn: %4.4f, Yn: %4.4f, Zn: %4.4f',
                     [ObjPos[0], ObjPos[1], ObjPos[2]]));
      WinPos := FViewer.Buffer.WorldToScreen(ObjPos);
      Canvas.TextOut(Round(winPos[0]), FViewer.Height-Round(WinPos[1]), CurrentPick.Name);
    end
    else begin
    	Canvas.TextOut(3, 3+Canvas.TextHeight('A'), 'No selected object');
    end;
   finally
      Canvas.Free;
   end;
end;

procedure TfrmFunc00017.DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   pick: TGLBaseSceneObject;
begin
  MovingOnZ:=(ssShift in Shift);
  pick := (FViewer.Buffer.GetPickedObject(X, Y) as TGLCustomSceneObject);
  if Assigned(Pick) then
  begin
  	if (pick.Name <> 'CubeA') and (pick.Name <> 'CubeB') then pick := nil;
  end;
  if pick <> CurrentPick then
  begin
    if Assigned(CurrentPick) then
       CurrentPick.Material.FrontProperties.Emission.Color := clrBlack;
    CurrentPick := TGLCustomSceneObject(pick);
    if Assigned(CurrentPick) then
       CurrentPick.Material.FrontProperties.Emission.Color := SelectionColor;
  end;
  // store mouse pos
  if Assigned(CurrentPick) then
    LastMouseWorldPos := MouseWorldPos(X, Y);
  inherited;
end;

procedure TfrmFunc00017.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
	NewPos: TVector;
begin
  if ssLeft in Shift then
  begin
  	if (ssShift in Shift) <> MovingOnZ then
    begin
    	MovingOnZ := (ssShift in Shift);
      LastMouseWorldPos := MouseWorldPos(X, Y);
    end;
    NewPos := MouseWorldPos(X, Y);
    if Assigned(CurrentPick) and (VectorNorm(LastMouseWorldPos)<>0) then
       CurrentPick.Position.Translate(VectorSubtract(NewPos, LastMouseWorldPos));
    LastMouseWorldPos := newPos;
  end
  else
  	if Shift = [ssRight] then inherited;

end;

function TfrmFunc00017.MouseWorldPos(X, Y: Integer): TVector;
var
	V: TVector;
begin
	Y := FViewer.Height - Y;
  if Assigned(CurrentPick) then
  begin
  	SetVector(V, X, Y, 0);
    if MovingOnZ then FViewer.Buffer.ScreenVectorIntersectWithPlaneXZ(V, CurrentPick.Position.Y, Result)
    else 							FViewer.Buffer.ScreenVectorIntersectWithPlaneXY(V, currentPick.Position.Z, Result);
  end
  else SetVector(Result, NullVector);
end;

end.
