unit fFunc00019;

interface

uses Classes, Controls, fFunc, GLScene, GLObjects, GLGeomObjects, GLTexture, GLMultiPolygon;

type
  TVektor = record
    X, Y, Z: Double;
  end;

	TfrmFunc00019 = class(TfrmFunc)
	private
    FDX: Double;
    FDY: Double;
    FDZ: Double;
    function GetPlane(Side: Integer): TGLMultiPolygon;
    function TransformToPlane(Side: Integer; X, Y, Z: Double): TVektor; overload;
    function TransformToPlane(Side: Integer; V: TVektor): TVektor; overload;
    procedure CreatePlanes;
    procedure ReDraw;
    procedure AddMaterial(Obj: TGLSceneObject);
    procedure MakeHole(Side: Integer; X, Y, Z, D, T: Double; Phi: Double=0; Rho: Double=0);
    procedure SetDX(const Value: Double);
    procedure SetDY(const Value: Double);
    procedure SetDZ(const Value: Double);
  protected
    FLight2: TGLLightSource;
    FMaterialLibrary: TGLMaterialLibrary;
    FPlanes: array[0..5] of TGLMultiPolygon;
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;    
    property Plane[Side: Integer]: TGLMultiPolygon read GetPlane;
    property DX: Double read FDX write SetDX;
    property DY: Double read FDY write SetDY;
    property DZ: Double read FDZ write SetDZ;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00019 }

const
  cOpposite: array[0..5] of Integer = (5,3,4,1,2,0);

function Vektor(X, Y, Z: Double): TVektor;
begin
  Result.X := X;
  Result.Y := Y;
  Result.Z := Z;
end;

procedure TfrmFunc00019.AddMaterial(Obj: TGLSceneObject);
begin
  Obj.Material.MaterialLibrary := FMaterialLibrary;
  Obj.Material.LibMaterialName := 'MatInner';
end;

procedure TfrmFunc00019.CreatePlanes;
var
  I: Integer;

  function MakePlane(X, Y, Z, P, T, W, H: Double): TGLMultiPolygon;
  begin
    Result := TGLMultiPolygon.Create(nil);
    Result.Material.MaterialLibrary := FMaterialLibrary;
    Result.Material.LibMaterialName := 'MatSurface';
    Result.Parts := [ppTop];
    Result.AddNode(0,0,0,0);
    Result.AddNode(0,W,0,0);
    Result.AddNode(0,W,H,0);
    Result.AddNode(0,0,H,0);
    Result.Position.X := X;
    Result.Position.Y := Y;
    Result.Position.Z := Z;
    Result.PitchAngle := P;
    Result.TurnAngle  := T;
  end;

begin
  FDummy.DeleteChildren;
  FPlanes[2] := MakePlane( 0, 0, 0, -90,  0,DX,DZ);
  FPlanes[3] := MakePlane(DX, 0, 0, -90, 90,DY,DZ);
  FPlanes[4] := MakePlane(DX,DY, 0, -90,180,DX,DZ);
  FPlanes[1] := MakePlane( 0,DY, 0, -90,270,DY,DZ);
  FPlanes[5] := MakePlane( 0,DY, 0,-180,  0,DX,DY);
  FPlanes[0] := MakePlane( 0, 0,DZ,   0,  0,DX,DY);
  for I := 0 to 5 do FDummy.AddChild(FPlanes[I]);
end;

procedure TfrmFunc00019.CREATE_OBJECT;
begin
  inherited;
  FCadencer.Enabled := False;
  FCadencer.Scene := nil;

  FMaterialLibrary := TGLMaterialLibrary.Create(Self);
  FLight2 := TGLLightSource.Create(Self);
  FDummy := TGLDummyCube.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FLight2);
  FScene.Objects.AddChild(FDummy);

	with FMaterialLibrary do
  begin
  	with Materials.Add do
    begin
    	Name := 'MatSurface';
      Material.FrontProperties.Diffuse.Color := clrMediumWood;
    end;

  	with Materials.Add do
    begin
    	Name := 'MatInner';
      Material.FrontProperties.Diffuse.Color := clrLightWood;
    end;

  end;
  
  with FCamera do
  begin
  	DepthOfView := 10000;
    Direction.AsVector := VectorMake(0,1,0);
    FocalLength := 100;
    Position.AsVector := VectorMake(200,-1000,200);
    TargetObject := FDummy;
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FLight do
  begin
  	LightStyle := lsOmni;
    Position.AsVector := VectorMake(-500,-500,800);
  end;

  with FLight2 do
  begin
  	Position.AsVector := VectorMake(500,-500,800);
  end;

  with FDummy do
  begin
  	CubeSize := 100;
  	ShowAxes := True;
    VisibleAtRunTime := True;
  end;

  Redraw;

  FCadencer.Scene := FScene;
  FCadencer.Enabled := True;
  
end;

procedure TfrmFunc00019.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 30, newTime);
  inherited;
end;

function TfrmFunc00019.GetPlane(Side: Integer): TGLMultiPolygon;
begin
	Result := FPlanes[Side];
end;

procedure TfrmFunc00019.MakeHole(Side: Integer; X, Y, Z, D, T, Phi, Rho: Double);
var
  R: Double;
  procedure AddPlane(I: Integer);
  begin
    with Plane[I] do
      with TransformToPlane(I,X,Y,Z) do
        Contours.Add.Nodes.AddXYArc(R/cos(Phi*c180divPi),R,0,360,16,AffineVectorMake(X,Y,0));//Unit: VectorGeometry
  end;

var
  Dum: TGLDummyCube;
  Cyl: TGLCylinder;
  through: Boolean;
begin
  Dum := TGLDummyCube.Create(nil);
  Dum.Position.x := X;
  Dum.Position.y := Y;
  Dum.Position.z := Z;
  case Side of
    0 : Dum.PitchAngle := -90;
    1 : Dum.RollAngle  :=  90;
    2 : Dum.RollAngle  := 180;
    3 : Dum.RollAngle  := 270;
    4 : Dum.RollAngle  :=   0;
    5 : Dum.PitchAngle :=  90;
  end;
  Dum.PitchAngle := Dum.PitchAngle + Rho;
  Dum.RollAngle := Dum.RollAngle + Phi;
  R := 0.5*D;
  through := true;
  case Side of
    0 : if (Z-T)<=0 then T := Z else through := false;
    1 : if (X+T)>=DX then T := DX-X else through := false;
    2 : if (Y+T)>=DY then T := DY-Y else through := false;
    3 : if (X-T)<=0 then T := X else through := false;
    4 : if (Y-T)<=0 then T := Y else through := false;
    5 : if (Z+T)>=DZ then T := DZ-Z else through := false;
  end;
  Cyl := TGLCylinder.Create(nil);
  AddMaterial(Cyl);
  Cyl.Position.x := 0;
  Cyl.Position.y := - 0.5*T;
  Cyl.Position.z := 0;
  Cyl.Height := T;
  Cyl.BottomRadius := R;
  Cyl.TopRadius := R;
  Cyl.NormalDirection := ndInside;
  if through then Cyl.Parts := [cySides]
  else Cyl.Parts := [cySides,cyBottom];
  Dum.AddChild(Cyl);
  FDummy.AddChild(Dum);

  AddPlane(Side);
  if through then AddPlane(cOpposite[Side]);

end;

procedure TfrmFunc00019.Redraw;
begin
  DX := 600;
  DY := 400;
  DZ := 19;
  CreatePlanes;
  MakeHole(0,0.5*DX,0.5*DY,DZ,50,DZ);
end;

procedure TfrmFunc00019.SetDX(const Value: Double);
begin
  FDX := Value;
  FDummy.Position.X := -0.5*Value;
end;

procedure TfrmFunc00019.SetDY(const Value: Double);
begin
  FDY := Value;
  FDummy.Position.Y := -0.5*Value;
end;

procedure TfrmFunc00019.SetDZ(const Value: Double);
begin
  FDZ := Value;
  FDummy.Position.Z := -0.5*Value;
end;

function TfrmFunc00019.TransformToPlane(Side: Integer; X, Y, Z: Double): TVektor;
begin
  case Side of
    0 : 	Result := Vektor(X,Y,Z-DZ);
    1 : 	Result := Vektor(DY-Y,Z,X);
    2 : 	Result := Vektor(X,Z,-Y);
    3 : 	Result := Vektor(Y,Z,DX-X);
    4 : 	Result := Vektor(DX-X,Z,DY-Y);
    5 : 	Result := Vektor(X,DY-Y,Z);
  	else 	Result := Vektor(X,Y,Z);
  end;
end;

function TfrmFunc00019.TransformToPlane(Side: Integer; V: TVektor): TVektor;
begin
	with V do Result := TransformToPlane(Side, X, Y, Z);
end;

end.
