unit fFunc00020;

interface

uses Classes, Controls, fFunc, StdCtrls, ExtCtrls, GLObjects, GLTexture, VectorLists;

type
	TfrmFunc00020 = class(TfrmFunc)
  protected
    FPoints1: TGLPoints;
    FPoints2: TGLPoints;
    FPanel: TPanel;
    FCBPointParams: TCheckBox;
    FCBAnimate: TCheckBox;
    FCBRotateDummyCube: TCheckBox;
    procedure SETUP_USER_INTERFACE; override;
  	procedure CREATE_OBJECT; override;
    procedure DO_CheckBoxClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00020 }

const cNbPoints = 180;

procedure TfrmFunc00020.CREATE_OBJECT;
begin
  inherited;

  FPoints1 := TGLPoints.Create(Self);
  FPoints2 := TGLPoints.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FPoints1);
  FDummy.AddChild(FPoints2);

  with FCamera do
  begin
  	DepthOfView := 100;
    FocalLength := 100;
  	Position.AsVector := VectorMake(5,4,3);
    TargetObject := FDummy;
  end;

  with FDummy do
  begin
  	ShowAxes := True;
    VisibleAtRunTime := True;
  end;

  with FPoints1 do
  begin
  	NoZWrite := True;
    with PointParameters do
    begin
    	DistanceAttenuation.AsVector := VectorMake(-1.5,0.7,0);
      Enabled := True;
      FadeTresholdSize := 1;
      MaxSize := 180;
      MinSize := 0;
    end;
    Size := 15;
    Static := False;
    Style := psSmoothAdditive;
  end;

  with FPoints2 do
  begin
  	NoZWrite := True;
    with PointParameters do
    begin
    	DistanceAttenuation.AsVector := VectorMake(-1.5,0.7,0);
      Enabled := True;
      FadeTresholdSize := 1;
      MaxSize := 180;
      MinSize := 0;
    end;
    Size := 30;
    Static := False;
    Style := psSmoothAdditive;
  end;

  FPoints1.Positions.Count := cNbPoints;
  FPoints1.Colors.Add(clrWhite);
  FPoints2.Colors.Add(clrBlue);

  FCadencer.Scene := FScene;
  FCadencer.Enabled := True;

end;

procedure TfrmFunc00020.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
var
  i: Integer;
  f, a, ab, ca, sa: Single;
  p: TAffineVectorList;
  v: TAffineVector;
begin
	if FCBRotateDummyCube.Checked and Assigned(FDummy) then
  begin
  	FDummy.ShowAxes := True;
    FDummy.VisibleAtRunTime := True;
    ROTATE_SCENE_OBJECT(FDummy, 30, newTime);
  end
  else begin
  	FDummy.ShowAxes := False;
    FDummy.VisibleAtRunTime := False;
	end;

  if FCBAnimate.Checked then
  begin
    f := 1+Cos(newTime);
    p := FPoints1.Positions;
    ab:= newTime*0.1;
    for i := 0 to cNbPoints-1 do
    begin
      a := DegToRad(4*i)+ab;
      SinCos(a, sa, ca);
      v[0] := 2*ca;
      v[1] := 2*Cos(f*a);
      v[2] := 2*sa;
      p[i] := v;
    end;
    FPoints2.Positions := FPoints1.Positions;
  end;
  inherited;
end;

procedure TfrmFunc00020.DO_CheckBoxClick(Sender: TObject);
begin
	case TCheckBox(Sender).Tag of
  	0:begin
   			FPoints1.PointParameters.Enabled := FCBPointParams.Checked;
   			FPoints2.PointParameters.Enabled := FCBPointParams.Checked;
    	end;
    1:begin
   			FPoints1.Static := not FCBAnimate.Checked;
   			FPoints2.Static := not FCBAnimate.Checked;
    	end;
    2:;
  end;
end;

procedure TfrmFunc00020.SETUP_USER_INTERFACE;
begin
  FPanel := TPanel.Create(Self);
  FCBPointParams := TCheckBox.Create(Self);
  FCBAnimate := TCheckBox.Create(Self);
  FCBRotateDummyCube := TCheckBox.Create(Self);

  with FCBPointParams do
  begin
  	Tag := 0;
  	Checked := True;
    Caption := 'Point Parameters';
    SetBounds(5,0,150,20);
  	Parent := FPanel;
  end;

  with FCBAnimate do
  begin
  	Tag := 1;
	  Checked := True;
    Caption := 'Animate';
    SetBounds(5,20,150,20);
  	Parent := FPanel;
  end;

  with FCBRotateDummyCube do
  begin
  	Tag := 2;
	  Checked := True;
    Caption := 'Rotate Dummy';
    SetBounds(5,40,150,20);
  	Parent := FPanel;
  end;

	with FPanel do
  begin
  	SetBounds(10,10,160,65);
  	Parent := FViewer;
  end;

end;

end.
