unit fFunc00021;

interface

uses Windows, Classes, Controls, fFunc, GLScene, GLObjects, GLGeomObjects, GLGraph, GLTexture, GLMisc;

type
	TfrmFunc00021 = class(TfrmFunc)
  protected
    FPlane: TGLPlane;					//Unit: GLObjects
    FPoints: TGLPoints; 			//Unit: GLObjects
    FDirect: TGLDirectOpenGL; //Unit: GLScene
    FArrowLine: TGLArrowLine; //Unit: GLGeomObjects
    FXYZGrid: TGLXYZGrid;     //Unit: GLGraph
    procedure CREATE_OBJECT; override;
    procedure DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry, OpenGL1x;

{ TfrmFunc00021 }

procedure TfrmFunc00021.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FPlane := TGLPlane.Create(Self);
  FPoints := TGLPoints.Create(Self);
  FDirect := TGLDirectOpenGL.Create(Self);
  FArrowLine := TGLArrowLine.Create(Self);
  FXYZGrid := TGLXYZGrid.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);

  FDummy.AddChild(FDirect);
  FDummy.AddChild(FPoints);
  FDummy.AddChild(FPlane);
  FDummy.AddChild(FArrowLine);
  FDummy.AddChild(FLight);
  FPlane.AddChild(FXYZGrid);

(*
  FScene.Objects.AddChild(FDirect);
  FScene.Objects.AddChild(FPoints);
  FScene.Objects.AddChild(FPlane);

  FDummy.AddChild(FArrowLine);
  FDummy.AddChild(FLight);

  FPlane.AddChild(FXYZGrid);
*)
  FScene.ObjectsSorting := osNone;
  FScene.VisibilityCulling := vcNone;

  with FCamera do
  begin
  	DepthOfView := 100;
    FocalLength := 100;
    Position.AsVector := VectorMake(7,5,3);
    TargetObject := FDummy;
  end;

  with FDummy do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
//    ShowAxes := True;
  //  VisibleAtRuntime := True;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(50,50,0);
  end;

  with FArrowLine do
  begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.2;
    BottomRadius := 0.1;
    Direction.AsVector := VectorMake(0,1,0);
    HeadStackingStyle := ahssStacked;
    Height := 1;
    Loops := 1;
    Parts := [alLine, alTopArrow];
    Slices := 16;
    Stacks := 4;
  	TopArrowHeadHeight := 0.5;
    TopArrowHeadRadius := 0.2;
    TopRadius := 0.1;
    Up.AsVector := VectorMake(0,0,-1);
  end;

  with FDirect do
  begin
  	OnRender := DO_DirectRender;
  end;

  with FPoints do
  begin
  	NoZWrite := False;
    with PointParameters do
    begin
    	DistanceAttenuation.AsVector := VectorMake(1,0,0);
      Enabled := False;
      FadeTresholdSize := 1;
      MaxSize := 128;
      MinSize := 0;
      Size := 5;
      Static := False;
      Style := psSmooth;
    end;
  end;

  with FPlane do
  begin
  	Direction.AsVector := VectorMake(0,1,0);
    Height := 6;
    with Material do
    begin
    	FaceCulling := fcNoCull;
    	BlendingMode := bmTransparency;
      FrontProperties.Diffuse.Color := VectorMake(0.5,0.5,1,0.8);
    	MaterialOptions := [moNoLighting];
    end;
    NoZWrite := False;
    Style := [psSingleQuad,psTileTexture];
    Up.AsVector := VectorMake(0,0,1);
    Visible := True;
    Width := 6;
    XOffset := 0;
    XScope := 1;
    XTiles := 1;
    YOffset := 0;
    YScope := 1;
    YTiles := 1;
  end;

  with FXYZGrid do //GridS
  begin
  	LineColor.Color := VectorMake(1,1,1,0.5);
    LinePattern := $FFFF;
    LinesStyle := glsSegments;
    LineWidth := 1;
    Parts := [gpX,gpY];
    Visible := True;
    
    with XSamplingScale do
    begin
    	Max := 3;
      Min := -3;
      Origin := 0;
      Step := 0.5;
    end;

    with YSamplingScale do
    begin
    	Max := 3;
      Min := -3;    
      Origin := 0;
      Step := 0.5;
    end;

    with ZSamplingScale do
    begin
    	Max := 0;
      Min := 0;
      Origin := 0;
      Step := 0.1;
    end;

  end;

	for I := 1 to 100 do
  	FPoints.Positions.Add(
    	(Random-0.5)*5,
    	(Random-0.5)*5,
      (Random-0.5)*5);

end;

procedure TfrmFunc00021.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime, 2);
  inherited;
end;

procedure TfrmFunc00021.DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo);
var
  I: Integer;
  mat: TMatrix;
  p, pProj: TVector;
  planePoint, planeNormal: TVector;
  plane: THmgPlane;
begin
  // Here we recover our plane point and normal...
  planePoint  := FPlane.Position.AsVector;
  planeNormal := FPlane.Direction.AsVector;
  // ...which we use to create a plane (equation)
  plane := PlaneMake(planePoint, planeNormal);
  // from that plane equation and our pojection direction
  // (which is here the plane normal)
  mat := MakeParallelProjectionMatrix(plane, planeNormal);

  // save state, turn off lighting and specify the lines color
  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);
  glColor3f(1, 1, 0);

  // we'll be drawing a bunch of lines, to specify a line in OpenGL,
  // you only need to specify the line start and end vertices
  glBegin(GL_LINES);
    for I := 0 to FPoints.Positions.Count-1 do
    begin
       // read the point coordinates, directly from the TGLPoints list
       MakePoint(p, FPoints.Positions.List[I]);
       // project this point on the plane with the matrix
       pProj := VectorTransform(p, mat);
       // specify the two vertices
       glVertex3fv(@p);
       glVertex3fv(@pProj);
    end;
	glEnd;
   // restore state
   glPopAttrib;
end;

procedure TfrmFunc00021.DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
	FPlane.Position.Y := FPlane.Position.Y + WheelDelta*0.001;
end;

procedure TfrmFunc00021.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
	if Shift=[ssLeft] then inherited
  else if Shift=[ssRight] then
  begin
	  FCamera.RotateObject(FPlane, my-Y, mx-X);
  	mx := X;
  	my := Y;
  end;
end;

end.
