unit fFunc00022;

interface

uses Classes, Controls, StdCtrls, ExtCtrls, ComCtrls, fFunc, GLGraph, GLTexture,
	VectorTypes, VectorGeometry, GLMisc;

type
	TfrmFunc00022 = class(TfrmFunc)
  protected
    FHeightField: TGLHeightField;
    FGrids: array[0..2] of TGLXYZGrid;
    FPanel: TPanel;
    FCentered: TCheckBox;
		FCBRotateDummyCube: TCheckBox;
    FLabels: array[0..2] of TLabel;       //XY, XZ, YZ
    FPositions: array[0..2] of TTrackBar;	//XY, XZ, YZ
    procedure SETUP_USER_INTERFACE; override;
  	procedure CREATE_OBJECT; override;
    procedure DO_PositionsChange(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
		procedure DO_HeightField1GetHeight(const X, Y: Single; var Z: Single; var color: TVector4f; var texPoint: TTexPoint);
  end;

implementation

{ TfrmFunc00022 }

procedure TfrmFunc00022.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  
  FHeightField := TGLHeightField.Create(Self);
  for I := 0 to Length(FGrids)-1 do
  	FGrids[I] := TGLXYZGrid.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FHeightField);
  for I := 0 to Length(FGrids)-1 do
	  FHeightField.AddChild(FGrids[I]);

  with FViewer do
  begin
  	Buffer.FogEnable := True;
  end;

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 250;
    Position.AsVector := VectorMake(3,4,8);
    TargetObject := FHeightField;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(30,50,100);
  end;

  with FHeightField do
  begin
  	OnGetHeight := DO_HeightField1GetHeight;
    ColorMode := hfcmNone;
    Direction.AsVector := VectorMake(0,1,0);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.8,0.8,0.8,0.381);
    Options := [hfoTwoSided];
    ShowAxes := True;
    Up.AsVector := VectorMake(1,0,0);
    with XSamplingScale do
    begin
    	Max := 1;
      Min := -1;
      Origin := 0;
      Step := 0.02;
    end;
    with YSamplingScale do
    begin
    	Max := 1;
      Min := -1;
      Origin := 0;
      Step := 0.02;
    end;
  end;

  for I := 0 to Length(FGrids)-1 do
  	with FGrids[I] do
  	begin
    	AntiAliased := False;
  		case I of
    		0:begin LineColor.Color := clrBlue; Parts := [gpX,gpY]; end;
	      1:begin LineColor.Color := clrLime; Parts := [gpX,gpZ]; end;
  	    2:begin LineColor.Color := clrRed;  Parts := [gpY,gpZ]; end;
    	end;
      LinePattern := $FFFF;
      LinesStyle := glsSegments;
      LineWidth := 1;
      with XSamplingScale do begin Max := 1; Min := -1; Origin := 0; Step := 0.1; end;
      with YSamplingScale do begin Max := 1; Min := -1; Origin := 0; Step := 0.1; end;
      with ZSamplingScale do begin Max := 1; Min := -1; Origin := 0; Step := 0.1; end;
    end;

  FCadencer.Scene := FScene;
	FCadencer.Enabled := True;

end;

procedure TfrmFunc00022.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	if FCBRotateDummyCube.Checked and Assigned(FDummy) then
  begin
  	FDummy.ShowAxes := True;
    FDummy.VisibleAtRunTime := True;
    ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  end
  else begin
  	FDummy.ShowAxes := False;
    FDummy.VisibleAtRunTime := False;
	end;
  inherited;
end;

procedure TfrmFunc00022.DO_CheckBoxClick(Sender: TObject);
begin
  if FCentered.Checked then
  begin
	  FGrids[0].ZSamplingScale.Origin := 0;//XY
    FGrids[1].YSamplingScale.Origin := 0;//XZ
    FGrids[2].XSamplingScale.Origin := 0;//YZ
  end
  else begin
	  FGrids[0].ZSamplingScale.Origin := -1;//XY
    FGrids[1].YSamplingScale.Origin := -1;//XZ
    FGrids[2].XSamplingScale.Origin := -1;//YZ
  end;
end;

procedure TfrmFunc00022.DO_HeightField1GetHeight(const X, Y: Single;
  var Z: Single; var color: TVector4f; var texPoint: TTexPoint);
begin
  Z := VectorNorm(x, y);
  Z := cos(Z*12) / (2*(Z*6.28+1));
end;

procedure TfrmFunc00022.DO_PositionsChange(Sender: TObject);
var
	I: Integer;
begin
	I := TTrackBar(Sender).Tag;
  if Assigned(FGrids[I]) then
  	case I of
    	0: FGrids[I].ZSamplingScale.Origin := -(FPositions[I].Position / 10);//XY
      1: FGrids[I].YSamplingScale.Origin := -(FPositions[I].Position / 10);//XZ
      2: FGrids[I].XSamplingScale.Origin := -(FPositions[I].Position / 10);//YZ
    end;
end;

procedure TfrmFunc00022.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  FPanel := TPanel.Create(Self);
  FCentered := TCheckBox.Create(Self);
  FCBRotateDummyCube := TCheckBox.Create(Self);
  for I := 0 to Length(FLabels)-1 do//XY, XZ, YZ
  	FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FPositions)-1 do//XY, XZ, YZ
  	FPositions[I] := TTrackBar.Create(Self);

  with FPanel do
  begin
    SetBounds(0,5,210,200);
  	Parent := FViewer;
  end;

  with FCentered do
  begin
  	OnClick := DO_CheckBoxClick;
  	Checked := True;
    Caption := 'Centered grids';
    SetBounds(5,5,100,20);
    Parent := FPanel;
  end;

  with FCBRotateDummyCube do
  begin
  	Checked := True;
    Caption := 'Rotate Dummy';
    SetBounds(105,5,100,20);
    Parent := FPanel;
  end;

  for I := 0 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
      case I of
        0: Caption := 'XY Grid Position';
        1: Caption := 'XZ Grid Position';
        2: Caption := 'YZ Grid Position';
      end;
     	SetBounds(5,40+50*I,120,18);
      Parent := FPanel;
    end;

  for I := 0 to Length(FPositions)-1 do
  	with FPositions[I] do
    begin
    	Tag := I;
    	Max := 10;
      Min := -10;
      LineSize := 1;
      SliderVisible := True;
      ThumbLength := 10;
      TickMarks:= tmBoth;
      TickStyle := tsAuto;
      OnChange := DO_PositionsChange;
     	SetBounds(5,55+50*I,200,32);
      Parent := FPanel;
    end;

end;

end.
