unit fFunc00023;

interface

uses Classes, Controls, Graphics, StdCtrls, ExtCtrls, ComCtrls, fFunc,
	GLObjects, GLGraph, GLMisc, GLTexture, VectorGeometry;

type
	TfrmFunc00023 = class(TfrmFunc)
  protected
    FLines: TGLLines;
    FSphere: TGLSphere;
    FHeightField: TGLHeightField;
    FPanel: TPanel;
    FRadioButton: array[0..2] of TRadioButton;
    FComboBox: TComboBox;
    FCheckBox: array[0..1] of TCheckBox;
    FTrackBar: array[0..2] of TTrackBar;
    FLabels: array[0..4] of TLabel;
		procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_RadioButtonClick(Sender: TObject);
    procedure DO_ComboBoxChange(Sender: TObject);
    procedure DO_ComboBox1Enter(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
    procedure DO_TrackBarChange(Sender: TObject);
    procedure DO_SphereProgress(Sender: TObject; const deltaTime, newTime: Double);
  public
    procedure Formula1(const x, y: Single; var z: Single; var color: TColorVector; var texPoint: TTexPoint);//Unit: Geometry
    procedure Formula2(const x, y: Single; var z: Single; var color: TColorVector; var texPoint: TTexPoint);
    procedure Formula3(const x, y: Single; var z: Single; var color: TColorVector; var texPoint: TTexPoint);
  end;

implementation

{ TfrmFunc00023 }

procedure TfrmFunc00023.CREATE_OBJECT;
begin
  inherited;
  FLines := TGLLines.Create(Self);
  FSphere:= TGLSphere.Create(Self);
  FHeightField := TGLHeightField.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);

  FDummy.AddChild(FHeightField);
  FHeightField.AddChild(FSphere);
  FSphere.AddChild(FLines);

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 200;
    Position.AsVector := VectorMake(5,5,5);
    TargetObject := FDummy;
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(30,50,100);
  end;

  with FHeightField do
  begin
  	Direction.AsVector := VectorMake(0,1,0);
    Options := [hfoTwoSided];
    Up.AsVector := VectorMake(0,0,-1);
    with XSamplingScale do
    begin
    	Max := 1;
      Min := -1;
      Origin := 0;
      Step := 0.08;
    end;
    with YSamplingScale do
    begin
    	Max := 1;
      Min := -1;
      Origin := 0;
      Step := 0.08;
    end;
   	OnGetHeight := Formula1;
  end;

  with FSphere do
  begin
  	Material.FrontProperties.Diffuse.Color := clrMediumSpringGreen;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(1,1,0);
    Up.AsVector := VectorMake(0,1,0);
    Radius := 0.1;
    Slices := 12;
    Stacks := 9;
    Top := 90;
    TopCap := ctNone;
  	Bottom := -90;
    BottomCap := ctNone;
    OnProgress := DO_SphereProgress;
    Visible := False;
  end;

  with FLines do
  begin
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  	Division := 10;
    NodeColor.Color := clrBlue;

    with TGLLinesNode(Nodes.Add) do
    begin
    	Color.Color := clrBlue;
      X := 0;
      Y := 0;
      Z := 0;
    end;

    with TGLLinesNode(Nodes.Add) do
    begin
    	Color.Color := clrBlue;
      X := 0;
      Y := 0;
      Z := -1.5;
    end;

    NodesAspect := lnaInvisible;
    NodeSize := 1;
    Options := [];
    SplineMode := lsmLines;
  end;

  FCadencer.Scene := FScene;
  FCadencer.Enabled := True;


end;

procedure TfrmFunc00023.DO_SphereProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  // move our little sphere around
  if FSphere.Visible then
  begin
  	FSphere.Position.SetPoint(cos(newTime*2.3), sin(newTime), 1.5);
  	FHeightField.StructureChanged;
  end;
end;

procedure TfrmFunc00023.DO_CheckBoxClick(Sender: TObject);
var
	O: TCheckBox;
begin
	O := TCheckBox(Sender);
  case O.Tag of
  	0:if O.Checked then FHeightField.Options := FHeightField.Options+[hfoTwoSided]
   		else 						 	FHeightField.Options := FHeightField.Options-[hfoTwoSided];
    1:FLight.Shining := O.Checked;
  end;
end;

procedure TfrmFunc00023.DO_ComboBoxChange(Sender: TObject);
begin
  // change per vertex color mode
  case FComboBox.ItemIndex of
    0: FHeightField.ColorMode := hfcmNone;
    1: FHeightField.ColorMode := hfcmEmission;
    2: FHeightField.ColorMode := hfcmDiffuse;
  end;
end;

procedure TfrmFunc00023.DO_RadioButtonClick(Sender: TObject);
var
	O: TRadioButton;
begin
	O := TRadioButton(Sender);
  FSphere.Visible := O.Tag = 2;
  case O.Tag of
  	0:FHeightField.OnGetHeight := Formula1;
    1:FHeightField.OnGetHeight := Formula2;
    2:FHeightField.OnGetHeight := Formula3;
  end;
end;

procedure TfrmFunc00023.DO_TrackBarChange(Sender: TObject);
var
	O: TTrackBar;
begin
	O := TTrackBar(Sender);
	case O.Tag of
  	0:with FHeightField.XSamplingScale do
    	begin
      	Min := -O.Position/10;
      	Max := O.Position /10;
   		end;
  	1:with FHeightField.YSamplingScale do
    	begin
      	Min := -O.Position/10;
      	Max := O.Position /10;
   		end;
  	2:with FHeightField do
    	begin
      	XSamplingScale.Step := O.Position/1000;
      	YSamplingScale.Step := O.Position/1000;
   		end;

	end;
end;

procedure TfrmFunc00023.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
	FPanel := TPanel.Create(Self);
  FComboBox := TComboBox.Create(Self);
  for I := 0 to Length(FRadioButton)-1 	do FRadioButton[I] := TRadioButton.Create(Self);
  for I := 0 to Length(FCheckBox)-1 		do FCheckBox[I] := TCheckBox.Create(Self);
  for I := 0 to Length(FTrackBar)-1 		do FTrackBar[I] := TTrackBar.Create(Self);
  for I := 0 to Length(FLabels)-1 			do FLabels[I] := TLabel.Create(Self);

  for I := 0 to 3 do
  begin
  	with FLabels[I] do
  	begin
    	case I of
      	0: Caption := 'X extents';
        1: Caption := 'Y extents';
        2: Caption := 'Step';
        3: Caption := 'Fomula';
      end;
      SetBounds(3, 10+I*20, 45, 20);
      Parent := FPanel;
  	end;
  end;

  for I := 0 to Length(FTrackBar)-1 do
  begin
    with FTrackBar[I] do
    begin
    	Tag := I;
			case I of
      	0:begin Max := 50; Min := 0; Position := 10; Frequency := 1; end;
        1:begin Max := 50; Min := 0; Position := 10; Frequency := 1; end;
        2:begin Max :=160; Min :=10; Position := 80; Frequency :=10; end;
      end;
      SliderVisible := True;
      TickMarks := tmBottomRight;
      ThumbLength := 10;
      OnChange := DO_TrackBarChange;
      SetBounds(50, 10+I*20, 140, 20);
      Parent := FPanel;
    end;
  end;

  FLabels[3].Font.Style := FLabels[3].Font.Style + [fsBold]; 
	for I := 0 to Length(FRadioButton)-1 do
  	with FRadioButton[I] do
    begin
      Tag := I;
      case I of
      	0: Caption := 'Formula1';
        1: Caption := 'Formula2';
        2: Caption := 'Dynamic';
      end;
      Checked := I = 0; 
      OnClick := DO_RadioButtonClick;
      SetBounds(3, 90+I*20, 70, 20);
      Parent := FPanel;
    end;

	with FCheckBox[0] do
  begin
  	Tag := 0;
  	Caption := 'Two-sided';
    Checked := True;
    OnClick := DO_CheckBoxClick;
    SetBounds(100, 90, 90, 20);
    Parent := FPanel;
  end;

  with FLabels[4] do
  begin
  	Caption := 'Color Mode:';
    SetBounds(100, 115, 90, 20);
    Parent := FPanel;
  end;

  with FComboBox do
  begin
    OnEnter := DO_ComboBox1Enter;//קKException
    OnChange := DO_ComboBoxChange;
    SetBounds(100, 132, 90, 20);
    Parent := FPanel;
  end;

	with FCheckBox[1] do
  begin
  	Tag := 1;
  	Caption := 'Lighting';
    Checked := True;
    OnClick := DO_CheckBoxClick;
    SetBounds(100, 152, 90, 20);
    Parent := FPanel;
  end;

  with FPanel do
  begin
    SetBounds(0,1,200,185);
  	Parent := FViewer;
  end;

end;

procedure TfrmFunc00023.Formula1(const x, y: Single; var z: Single;
  var color: TColorVector; var texPoint: TTexPoint);
begin
  // first formula
  z:=VectorNorm(x, y);
  z:=cos(z*12)/(2*(z*6.28+1));
  VectorLerp(clrBlue, clrRed, (z+1)/2, color);
end;

procedure TfrmFunc00023.Formula2(const x, y: Single; var z: Single;
  var color: TColorVector; var texPoint: TTexPoint);
begin
  // 2nd formula
  z:=0.5*cos(x*6.28)*sin(Sqrt(abs(y))*6.28);
  VectorLerp(clrBlue, clrRed, (z+1)/2, color);
end;

procedure TfrmFunc00023.Formula3(const x, y: Single; var z: Single;
  var color: TColorVector; var texPoint: TTexPoint);
begin
  // 3rd formula, dynamic
  z:=1/(1+VectorNorm(FSphere.position.X-x, FSphere.position.Y-y));
  if ((Round(x*4)+Round(y*4)) and 1)=1 then color:=clrBlue
  else color:=clrYellow;
end;

procedure TfrmFunc00023.DO_ComboBox1Enter(Sender: TObject);
begin
  with FComboBox do
  	if Items.Text = '' then
	  begin
	  	Items.Add('none');
  	  Items.Add('emission');
    	Items.Add('diffuse');
	    ItemIndex := 0;
    end;
end;

end.
