unit fFunc00024;

interface

uses Classes, Controls, Graphics, fFunc, GLScene, GLGeomObjects, GLTexture, GLMisc;

type
	TfrmFunc00024 = class(TfrmFunc)
  protected
    FTorus: TGLTorus;
    FDirect: TGLDirectOpenGL;
    FMaterialLibrary: TGLMaterialLibrary;
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo); override;
	end;

implementation

uses VectorGeometry, JPEG, OpenGL1x;

{ TfrmFunc00024 }

procedure TfrmFunc00024.CREATE_OBJECT;
begin
  inherited;

  FMaterialLibrary := TGLMaterialLibrary.Create(Self);
  with FMaterialLibrary do
  begin
    with AddTextureMaterial('wood', '.\DATA\ashwood.jpg') do
    	Material.FrontProperties.Emission.Color := clrGray50;
    with AddTextureMaterial('stone','.\DATA\walkway.jpg') do
    	Material.FrontProperties.Emission.Color := clrGray50;
  end;

  FTorus := TGLTorus.Create(Self);
  FDirect:= TGLDirectOpenGL.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FLight);
  FDummy.AddChild(FTorus);
  FDummy.AddChild(FDirect);

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FViewer do
  begin
  	Buffer.AmbientColor.Color := clrLime;
  end;

  with FCamera do
  begin
  	CameraStyle := csOrthogonal;
    DepthOfView := 1000;
    Direction.AsVector := VectorMake(0,0,-1);
    FocalLength := 17;
    Position.AsVector := VectorMake(3,2,1);
    TargetObject := FDummy;
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FDummy do
  begin
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(20,20,20);
  end;

  with FTorus do
  begin
  	Direction.AsVector := VectorMake(0,1,0);
    MajorRadius := 1.3;
    MinorRadius := 0.3;
    Rings := 36;
    ShowAxes := True;
    Sides := 256;
    Up.AsVector := VectorMake(0,0,-1);
    with Material do
    begin
    	BlendingMode := bmTransparency;
      FrontProperties.Diffuse.Color := VectorMake(0.42,0.8,0.8,1);
    end;
  end;

  with FDirect do
  begin
  	Blend := False;
    Direction.AsVector := VectorMake(0,1,0);
    Up.AsVector := VectorMake(0,0,-1);
    UseBuildList := False;
    OnRender := DO_DirectRender;
  end;

	FCadencer.Scene := FScene;
  FCadencer.Enabled := True;

end;

procedure TfrmFunc00024.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  inherited;
end;

procedure TfrmFunc00024.DO_DirectRender(Sender: TObject; var rci: TRenderContextInfo);
var
	material: TGLLibMaterial;
begin
  // disable face culling
  glDisable(GL_CULL_FACE);
  // 1st quad, textured with 'wood', using standard method
  FMaterialLibrary.ApplyMaterial('wood', rci);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 1);  glVertex3f(+0.5, 0.5, -0.5);
    glTexCoord2f(0, 0);  glVertex3f(-0.5, 0.5, -0.5);
    glTexCoord2f(1, 0);  glVertex3f(-0.5, 0.0, +0.5);
    glTexCoord2f(1, 1);  glVertex3f(+0.5, 0.0, +0.5);
  glEnd;
  FMaterialLibrary.UnApplyMaterial(rci);

  // 2nd quad, textured with 'stone'
  // we "manually" apply the material, this can be usefull if you want to have
  // some dynamic material control
  material := FMaterialLibrary.Materials.GetLibMaterialByName('stone');
  material.Material.Apply(rci);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 1);  glVertex3f(+0.5, -0.5, -0.5);
    glTexCoord2f(0, 0);  glVertex3f(+0.5, +0.0, +0.5);
    glTexCoord2f(1, 0);  glVertex3f(-0.5, +0.0, +0.5);
    glTexCoord2f(1, 1);  glVertex3f(-0.5, -0.5, -0.5);
  glEnd;
  material.Material.UnApply(rci);

  // enable face culling again
  glEnable(GL_CULL_FACE);

end;

end.
