unit fFunc00025;

interface

uses Classes, Controls, Dialogs, Forms, fFunc, StdCtrls, GLTeapot, GLMisc, GLTexture;

type
	TfrmFunc00025 = class(TfrmFunc)
  protected
    FButton: TButton;
    FTeapot: TGLTeapot;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_ButtonClick(Sender: TObject);
		procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
	end;

implementation

uses SysUtils, VectorGeometry, Jpeg, OpenGL1x;

{ TfrmFunc00025 }

procedure TfrmFunc00025.CREATE_OBJECT;
begin
  inherited;
  FTeapot := TGLTeapot.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FTeapot);

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    Direction.AsVector := VectorMake(0,0,-1);
    FocalLength := 400;
    Position.AsVector := VectorMake(1,3,5);
    TargetObject := FTeapot;///////////////
  end;

  with FDummy do
	begin
//  	ShowAxes := True;
  	VisibleAtRuntime := True;
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(7,5,3);
  end;

  with FTeapot do
	begin
  	Scale.AsVector := VectorMake(1.5,1.5,1.5);
  	Material.Texture.TextureMode := tmReplace;
  end;

end;

procedure TfrmFunc00025.SETUP_USER_INTERFACE;
begin
  FButton := TButton.Create(Self);
	with FButton do
  begin
  	Caption := 'Apply Cube Environment Map';
  	OnClick := DO_ButtonClick;
    SetBounds(5,5,160,27);
    Parent := FViewer;
  end;
end;

procedure TfrmFunc00025.DO_ButtonClick(Sender: TObject);
begin
  // Cube map warning message
  // If you don't check and turn off cube maps yourself in your apps when
  // cube maps aren't supported, GLScene will just turn off texturing
  // (ie. no error generated, just a different output)
  if not GL_ARB_texture_cube_map then
  begin
  	ShowMessage('Your graphics board does not support cube maps...');
    Exit;
  end;
  // Our cube map images are here
//  SetCurrentDir(ExtractFilePath(Application.ExeName)+'.\DATA');

  with FTeapot.Material.Texture do
  begin
    // We need a CubeMapImage, which unlike the "regular Images" stores
    // multiple images.
    ImageClassName:=TGLCubeMapImage.ClassName;
    with Image as TGLCubeMapImage do
    begin
      // Load all 6 texture map components of the cube map
      // The 'PX', 'NX', etc. refer to 'positive X', 'negative X', etc.
      // and follow the RenderMan specs/conventions
      Picture[cmtPX].LoadFromFile('.\DATA\cm_left.jpg');
      Picture[cmtNX].LoadFromFile('.\DATA\cm_right.jpg');
      Picture[cmtPY].LoadFromFile('.\DATA\cm_top.jpg');
      Picture[cmtNY].LoadFromFile('.\DATA\cm_bottom.jpg');
      Picture[cmtPZ].LoadFromFile('.\DATA\cm_back.jpg');
      Picture[cmtNZ].LoadFromFile('.\DATA\cm_front.jpg');
    end;
    // Select reflection cube map environment mapping
    // This is the mode you'll most commonly use with cube maps, normal cube
    // map generation is also supported (used for diffuse environment lighting)
    MappingMode:=tmmCubeMapReflection;
    // That's all folks, let us see the thing!
    Disabled:=False;
  end;
  FButton.Visible := False;
end;


procedure TfrmFunc00025.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy, 15, newTime);
  ROTATE_SCENE_OBJECT(FTeapot, -15, newTime);
  inherited;
end;

end.
