unit fFunc00026;

interface

uses
	Classes, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls, fFunc, GLScene, GLObjects,
  GLTexture, GLGeomObjects, GLTeapot, GLSkydome, GLUtils;

type
	TfrmFunc00026 = class(TfrmFunc)
  private
    procedure GenerateCubeMap;
  protected
    FMemoryViewer: TGLMemoryViewer;//Unit: GLScene
    FCubeMapCamera: TGLCamera;
    FTorus: TGLTorus;
    FSphere: TGLSphere;
    FCylinder: TGLCylinder;
    FTeapot: TGLTeapot;
    FSkyDome: TGLSkyDome;//Unit: GLSkydome
    FCube: TGLCube;

    FPanel: TPanel;
    FCheckBox: TCheckBox;

  	procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  public
    CubeMapWarnDone: Boolean;
  end;

implementation

uses VectorGeometry, OpenGL1x;

{ TfrmFunc00026 }

procedure TfrmFunc00026.CREATE_OBJECT;
begin
  inherited;
  FMemoryViewer := TGLMemoryViewer.Create(Self);
  FTorus := TGLTorus.Create(Self);
  FTeapot := TGLTeapot.Create(Self);
  FSkyDome := TGLSkyDome.Create(Self);
  FCubeMapCamera := TGLCamera.Create(Self);
  FCylinder := TGLCylinder.Create(Self);
  FSphere := TGLSphere.Create(Self);
  FCube := TGLCube.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FLight);
  FDummy.AddChild(FSkyDome);
  FDummy.AddChild(FTeapot);
  FDummy.AddChild(FTorus);
  FTeapot.AddChild(FCubeMapCamera);
  FTorus.AddChild(FCylinder);
  FTorus.AddChild(FSphere);
  FTorus.AddChild(FCube);

  with FScene do
  begin
  end;

  with FCamera do
  begin
  	Position.AsVector := VectorMake(10,4,8);
    FocalLength := 100;
    TargetObject := FTorus;
  end;

  with FMemoryViewer do
  begin
  	Buffer.BackgroundColor := clMaroon;
    Camera := FCubeMapCamera;
    Height := 128;
    Width := 128;
  end;

  with FDummy do
  begin
  end;

  with FLight do
  begin
  	Diffuse.Color := VectorMake(0.886,0.886,0.886,1);
    Position.AsVector := VectorMake(50,50,50);
  end;

  with FSkyDome do
  begin
  	with Bands.Add do
    begin
    	Slices := 6;
      Stacks := 1;
      StartAngle := 0;
      StopAngle := 15;
      StartColor.Color := clrWhite;
      StopColor.Color := clrBlue;
    end;
  	with Bands.Add do
    begin
    	Slices := 6;
      Stacks := 4;
      StartAngle := 15;
      StopAngle := 90;
      StartColor.Color := clrBlue;
      StopColor.Color := VectorMake(0.51,0.51,1,1);
    end;
  	with Bands.Add do
    begin
    	Slices := 6;
      Stacks := 1;
      StartAngle := -90;
      StopAngle := -5;
      StartColor.Color := VectorMake(0,0.251,0,1);
      StopColor.Color := VectorMake(0,0.502,0,1);
    end;
  	with Bands.Add do
    begin
    	Slices := 6;
      Stacks := 1;
      StartAngle := -5;
      StopAngle := 0;
      StartColor.Color := VectorMake(0,0.502,0,1);
      StopColor.Color := clrWhite;
    end;
    Direction.AsVector := VectorMake(0,1,0);
    Up.AsVector := VectorMake(-0.9486833,0,0.3162278);
  end;

  with FTeapot do
  begin
  	Direction.AsVector := VectorMake(0.7071067,0,0.7071067);
//    Material.Texture.ImageClassName := 'CubeMap Image';////////////////////////////Exception here
    Material.Texture.MappingMode := tmmCubeMapReflection;
    Material.Texture.MinFilter := miLinear;
    Material.Texture.TextureMode := tmReplace;
    Scale.AsVector := VectorMake(8,8,8);
    Visible := True;
  end;

  with FCubeMapCamera do
  begin
  	DepthOfView := 50;
    FocalLength := 25;
    TargetObject := FSphere;
  end;

  with FTorus do
  begin
  	Direction.AsVector := VectorMake(0,1,0);
  	MajorRadius := 5;
    MinorRadius := 0.3;
    Rings := 64;
    Sides := 32;
    Up.AsVector := VectorMake(1,0,0);
  end;

  with FCylinder do
  begin
  	BottomRadius := 0.3;
  	Direction.AsVector := VectorMake(0,1,0);
    Height := 5;
    Loops := 1;
    Material.FrontProperties.Emission.Color := clrBlue;
    Parts := [cySides,cyBottom,cyTop];
    Position.AsVector := VectorMake(5,0,0);
    Slices := 64;
    Stacks := 1;
    TopRadius := 0.3;
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FSphere do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.502,1,0,1);
    Material.FrontProperties.Emission.Color := VectorMake(0,0.549,0.137,1);
    Position.AsVector := VectorMake(-5,0,0);
    Radius := 1.3;
    Slices := 32;
    Stacks := 32;
  end;

  with FCube do
  begin
  	CubeDepth := 1.8;
    CubeHeight := 1.8;
    CubeWidth := 1.8;
    Direction.AsVector := VectorMake(0,0,1);
    Material.FrontProperties.Diffuse.Color := VectorMake(0.22,0.22,0.22,1);
    Material.FrontProperties.Emission.Color := VectorMake(0.502,0,0,1);
    Position.AsVector := VectorMake(0,5,0);
  end;

  with FCadencer do
  begin
  	Scene := FScene;
    Enabled := True;
  end;

end;

procedure TfrmFunc00026.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  if FCheckBox.Checked then
  begin
    // make things move
    FTeapot.Position.Y	:= 2*Sin(newTime);
    ROTATE_SCENE_OBJECT(FTorus,15,newTime,4);
    ROTATE_SCENE_OBJECT(FCube,90,newTime,1);
    // generate the cube map
    GenerateCubeMap;
  end;
  inherited;
end;

procedure TfrmFunc00026.SETUP_USER_INTERFACE;
begin
  FPanel := TPanel.Create(Self);
  FCheckBox := TCheckBox.Create(Self);

  with FCheckBox do
  begin
  	Caption := 'Animate && Dynamically generate cube map';
  	Checked := True;
    SetBounds(3,3,220,20);
  	Parent := FPanel;
  end;

  with FPanel do
  begin
    SetBounds(0,1,230,26);
  	Parent := FViewer;
  end;

end;

procedure TfrmFunc00026.GenerateCubeMap;
begin
EXIT;////////////////////////////does not work
  // Don't do anything if cube maps aren't supported
  if not GL_ARB_texture_cube_map then
  begin
  	if not cubeMapWarnDone then
       ShowMessage('Your graphics hardware does not support cube maps...');
    cubeMapWarnDone := True;
    Exit;
  end;
  // Here we generate the new cube map, from CubeMapCamera (a child of the
  // teapot in the scene hierarchy)
  with FTeapot do
  begin
    // hide the teapot while rendering the cube map
    Visible := False;
    // render cube map to the teapot's texture
    FMemoryViewer.RenderCubeMapTextures(Material.Texture);
    // teapot visible again
    Material.Texture.Disabled := False;
    Visible := True;
  end;

end;


end.
