unit fFunc00028;

interface

uses Graphics, StdCtrls, ExtCtrls, Forms, fFunc, GLObjects, GLTexture, GLMisc;

type
	TfrmFunc00028 = class(TfrmFunc)
  protected
    FMaterialLibrary: TGLMaterialLibrary;
    FCube: TGLCube;

    FPanel: TPanel;
    FButton: TButton;
    FCBAnimate: TCheckBox;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE(); override;
    procedure DO_Close(Sender: TObject; var Action: TCloseAction); override;
    procedure DO_ButtonClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  public
    TimeToNextFrame: Double;
	end;

implementation

uses SysUtils, VectorGeometry;

{ TfrmFunc00028 }

procedure TfrmFunc00028.CREATE_OBJECT;
begin
  inherited;
  FMaterialLibrary := TGLMaterialLibrary.Create(Self);
  FCube	:= TGLCube.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FCube);

  with FMaterialLibrary do
  begin
  end;

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

	with FViewer do
  begin
  	Buffer.AmbientColor.Color := clrWhite; //clrYellow;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 300;
    TargetObject := FDummy;
    Position.AsVector := VectorMake(5,4,3);
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(15,10,7);
  end;

  with FDummy do
  begin
//		ShowAxes := True;
  //  VisibleAtRuntime := True;
  end;

  with FCube do
  begin
  	Material.FrontProperties.Diffuse.Color := clrAqua;
  end;

	with FCadencer do
  begin
  end;

end;

procedure TfrmFunc00028.DO_Close(Sender: TObject; var Action: TCloseAction);
begin
	FCBAnimate.Checked := False;
  inherited;
end;

procedure TfrmFunc00028.SETUP_USER_INTERFACE;
begin
  FPanel := TPanel.Create(Self);
  FButton := TButton.Create(Self);
  FCBAnimate := TCheckBox.Create(Self);

  with FPanel do
  begin
    SetBounds(0,1,150,54);
  	Parent := FViewer;
  end;

  with FButton do
  begin
  	Caption := 'Generate Anim Frames';
  	OnClick := DO_ButtonClick;
    SetBounds(2,1,146,27);
  	Parent := FPanel;
  end;

  with FCBAnimate do
  begin
  	Enabled := True;
  	Caption := 'Animate';
    SetBounds(2,30,65,20);
  	Parent := FPanel;
  end;

end;

procedure TfrmFunc00028.DO_ButtonClick(Sender: TObject);
const
	ChinNum: array[0..9] of string = ('s','','L','','v','','','m','','h');
var
  I: Integer;
  bmp: TBitmap;
begin
  // We generate a handful of bitmaps from scratch
  // you could also load them from a set of files, extract from an AVI etc.
  for I := 0 to 9 do
  begin
    bmp := TBitmap.Create;
    try
    	bmp.PixelFormat := pf24bit;
    	bmp.Width := 60;
    	bmp.Height := 60;
    	bmp.Canvas.Font.Name:='з';//Arial';
    	bmp.Canvas.Font.Height := 56;
   		bmp.Canvas.TextOut(2, 2, ChinNum[I]);
      //bmp.Canvas.TextOut(15, 5, IntToStr(I));
    	FMaterialLibrary.AddTextureMaterial('IMG'+IntToStr(I), bmp);
    finally
    	bmp.Free;
  	end;
  end;
  // Initialize our loop
  FCube.Material.MaterialLibrary := FMaterialLibrary;
  FCube.Material.LibMaterialName := 'IMG0';
  FMaterialLibrary.Tag := 0;
  // GUI update
  FCBAnimate.Enabled := True;
  FButton.Enabled := False;
end;

procedure TfrmFunc00028.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
  // cube turns slowly
  FCube.Turn(deltaTime*3);
  // cycle textures
  if FCBAnimate.Checked then
  begin
    // coutdown to next frame
    TimeToNextFrame := TimeToNextFrame - deltaTime;
    // if it's time for the next frame
    if TimeToNextFrame < 0 then
  	begin
    	// first, update frame counter (the Tag property in our sample)
      // (such a loop is a little overkill, yeah)
      while TimeToNextFrame < 0 do
      begin
      	TimeToNextFrame := TimeToNextFrame+0.2;
        FMaterialLibrary.Tag:=(FMaterialLibrary.Tag+1) mod 10;
      end;
      // then, we update the material reference
      FCube.Material.LibMaterialName := 'IMG' + IntToStr(FMaterialLibrary.Tag);
    end;
  end;

  ROTATE_SCENE_OBJECT(FDummy, 30, newTime, 7);

  inherited;

end;

end.
