unit fFunc00029;

interface

uses Graphics, StdCtrls, ExtCtrls, fFunc, GLObjects, GLGeomObjects, GLTexture;

type
	TfrmFunc00029 = class(TfrmFunc)
  protected
    FMaterialLibrary: TGLMaterialLibrary;
    FCube: TGLCube;
    FTorus: TGLTorus;
    FSphere: array[0..1] of TGLSphere;
    FAnnulus: array[0..1] of TGLAnnulus;
    FPanel: TPanel;
    FBUBind: TButton;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE(); override;
    procedure DO_ButtonClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

  THiddenLineShader = class (TGLShader)
  private
    BackgroundColor, LineColor: TColorVector;
    PassCount: Integer;
  public
    procedure DoApply(var rci : TRenderContextInfo; Sender : TObject); override;
    function DoUnApply(var rci : TRenderContextInfo) : Boolean; override;
  end;

  TOutLineShader = class (TGLShader)
  private
    BackgroundColor, LineColor: TColorVector;
    OutlineSmooth, Lighting: Boolean;
    OutlineWidth, Oldlinewidth: Single;
    PassCount: Integer;
  public
    procedure DoApply(var rci : TRenderContextInfo; Sender : TObject); override;
    function DoUnApply(var rci : TRenderContextInfo) : Boolean; override;
  end;

implementation

uses VectorGeometry, OpenGL1x;

{ TfrmFunc00029 }

procedure TfrmFunc00029.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FMaterialLibrary := TGLMaterialLibrary.Create(Self);

  with FMaterialLibrary.Materials.Add do Name := 'LibMaterial';
  with FMaterialLibrary.Materials.Add do Name := 'LibMaterial1';
  with FMaterialLibrary.Materials.Add do Name := 'LibMaterial2';

  FTorus := TGLTorus.Create(Self);
  FCube := TGLCube.Create(Self);
  for I := 0 to Length(FSphere)-1 do
  	FSphere[I] := TGLSphere.Create(Self);
  for I := 0 to Length(FAnnulus)-1 do
  	FAnnulus[I] := TGLAnnulus.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FTorus);
  FDummy.AddChild(FCube);
  for I := 0 to Length(FSphere)-1  do FDummy.AddChild(FSphere[I]);
  for I := 0 to Length(FAnnulus)-1 do FDummy.AddChild(FAnnulus[I]);

  with FScene do
	begin
  end;

  with FViewer do
	begin
    Buffer.BackgroundColor := cl3DDkShadow;
  	Buffer.AmbientColor.Color := clrGray20;
  end;

  with FCamera do
	begin
  	DepthOfView := 100;
    FocalLength := 100;
    TargetObject := FDummy;
  	Direction.AsVector := VectorMake(0,0,-1);
  	Position.AsVector := VectorMake(14,10,6);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(10,8,7);
  end;

  with FDummy do begin{}end;

  with FTorus do
	begin
  	MajorRadius := 2.5;
    MinorRadius := 1.5;
    Rings := 25;
    Sides := 15;
  	Direction.AsVector := VectorMake(0,1,0);
  	Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0,-1);
    with Material do
    begin
    	MaterialLibrary := FMaterialLibrary;
    	LibMaterialName := 'LibMaterial';
    end;
  end;

  with FCube do
	begin
  	Scale.AsVector := VectorMake(2,2,2);
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,0,7);
    Up.AsVector := VectorMake(0,1,0);
    with Material do
    begin
    	MaterialLibrary := FMaterialLibrary;
    	LibMaterialName := 'LibMaterial2';
    end;
  end;

  with FSphere[0] do
	begin
  	Radius := 0.5;
    Slices := 16;
    Stacks := 16;
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
    ShowAxes := True;
  end;

  with FSphere[1] do
	begin
  	Radius := 1.5;
    Slices := 16;
    Stacks := 16;
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,0,-7);
    Up.AsVector := VectorMake(0,1,0);
    with Material do
    begin
    	MaterialLibrary := FMaterialLibrary;
    	LibMaterialName := 'LibMaterial1';
    end;
  end;

  with FAnnulus[0] do
	begin
  	BottomInnerRadius := 0.3;
    BottomRadius := 0.5;
  	TopInnerRadius := 0.3;
    TopRadius := 0.5;
    Slices := 16;
    Stacks := 4;
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(7,0,0);
    Up.AsVector := VectorMake(0,1,0);
    Scale.AsVector := VectorMake(2,2,2);
    with Material do
    begin
    	MaterialLibrary := FMaterialLibrary;
    	LibMaterialName := 'LibMaterial1';
    end;
  end;

  with FAnnulus[1] do
	begin
  	BottomInnerRadius := 0.3;
    BottomRadius := 0.5;
  	TopInnerRadius := 0.3;
    TopRadius := 0.5;
    Slices := 16;
    Stacks := 4;
  	Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(-7,0,0);
    Up.AsVector := VectorMake(0,1,0);
    Scale.AsVector := VectorMake(2,2,2);
    with Material do
    begin
    	MaterialLibrary := FMaterialLibrary;
    	LibMaterialName := 'LibMaterial2';
    end;
  end;

end;

procedure TfrmFunc00029.DO_ButtonClick(Sender: TObject);
var
	shader1: THiddenLineShader;
  shader2, shader3: TOutLineShader;
begin
  FBUBind.Enabled:=False;
  // instantiate our shaders
  shader1:=THiddenLineShader.Create(Self);
  shader1.BackgroundColor:=ConvertWinColor(FViewer.Buffer.BackgroundColor);
  shader1.LineColor:=clrBlue;

  shader2:=TOutLineShader.Create(Self);
  with shader2 do
  begin
    BackgroundColor:=ConvertWinColor(FViewer.Buffer.BackgroundColor);
    Outlinesmooth:=true;
    OutLineWidth:=2;
    Lighting:=false;
    LineColor:=clrBlack;
  end;

  shader3:=TOutLineShader.Create(Self);
  with shader3 do
  begin
    BackgroundColor:=ConvertWinColor(FViewer.Buffer.BackgroundColor);
    Outlinesmooth:=false;
    OutLineWidth:=4;
    Lighting:=true;
    LineColor:=clrRed;
  end;

 	// binds the shaders to the materials
	FMaterialLibrary.Materials[0].Shader := shader1;
 	FMaterialLibrary.Materials[1].Shader := shader2;
 	FMaterialLibrary.Materials[2].Shader := shader3;

end;

procedure TfrmFunc00029.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FDummy, 15, newTime, 2);
  inherited;
end;

procedure TfrmFunc00029.SETUP_USER_INTERFACE;
begin
  inherited;
  FPanel := TPanel.Create(Self);
  FBUBind := TButton.Create(Self);

  with FPanel do
  begin
    SetBounds(0,1,100,30);
  	Parent := FViewer;
  end;

  with FBUBind do
  begin
  	Caption := 'Bind Shaders';
    OnClick := DO_ButtonClick;
    SetBounds(1,1,97,26);
  	Parent := FPanel;
  end;

end;

{ THiddenLineShader }

procedure THiddenLineShader.DoApply(var rci: TRenderContextInfo; Sender: TObject);
begin
  // new object getting rendered, 1st pass
  PassCount:=1;
  // backup state
  glPushAttrib(GL_ENABLE_BIT);
  // disable lighting, this is a solid fill
  glDisable(GL_LIGHTING);
  rci.GLStates.SetGLPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
  // use background color
  glColor3fv(@BackgroundColor);
  // enable and adjust polygon offset
  glEnable(GL_POLYGON_OFFSET_FILL);
  glPolygonOffset(1, 2);
end;

function THiddenLineShader.DoUnApply(var rci: TRenderContextInfo): Boolean;
begin
  case PassCount of
    1:begin
        // 1st pass completed, we setup for the second
        PassCount:=2;
        // switch to wireframe and its color
        rci.GLStates.SetGLPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
        glColor3fv(@LineColor);
        // disable polygon offset
        glDisable(GL_POLYGON_OFFSET_LINE);
        Result:=True;
      end;
    2:begin
        // restore state
        glPopAttrib;
        // we're done
        Result:=False;
      end;
    else
      // doesn't hurt to be cautious
      Assert(False);
      Result:=False;
	end;

end;

{ TOutLineShader }

procedure TOutLineShader.DoApply(var rci: TRenderContextInfo; Sender: TObject);
begin
  PassCount:=1;
  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);

  if outlineSmooth then
  begin
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
    glEnable(GL_LINE_SMOOTH);
  end else glDisable(GL_LINE_SMOOTH);

  glGetFloatv(GL_LINE_WIDTH,@oldlinewidth);
  glLineWidth(OutLineWidth);
  glPolygonMode(GL_BACK, GL_LINE);
  glCullFace(GL_FRONT);
  glDepthFunc(GL_LEQUAL);
  glColor3fv(@lineColor);
end;

function TOutLineShader.DoUnApply(var rci: TRenderContextInfo): Boolean;
begin
  case PassCount of
  	1:begin
     		PassCount:=2;
     		if lighting then 	glEnable(GL_LIGHTING)
        else 							glColor3fv(@backGroundColor);
		    glDepthFunc(GL_LESS);
    		glCullFace(GL_BACK);
		    glPolygonMode(GL_BACK, GL_FILL);
     		Result:=True;
  		end;
  	2:begin
     		glPopAttrib;
     		glLineWidth(oldLineWidth);
     		Result:=False;
  		end;
  	else
  		Assert(False);
  		Result:=False;
  end;
end;

end.
