unit fFunc00030;

interface

uses Classes, Controls, Graphics, StdCtrls, ExtCtrls, fFunc, Dialogs, GLWindowsFont,
	GLScene, GLGui, GLWindows, GLTexture;

type
	TfrmFunc00030 = class(TfrmFunc)
  protected
    FWindowsBitmapFont: TGLWindowsBitmapFont;	//Unit: GLWindowsFont
    FMaterialLibrary: TGLMaterialLibrary;
    FGLForm: TGLForm;                         //GLWindows
    FGLEdit: TGLEdit;
    FGLLabel: TGLLabel;
    FGLButton: TGLButton;
    FGuiLayout: TGLGuiLayout;									//Unit: GLGui
		FPanel: TPanel;
    FButton: array[0..1] of TButton;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_ViewerMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState); override;
    procedure DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState); override;
		procedure DO_KeyPress(Sender: TObject; var Key: Char); override;
    procedure DO_GLButtonClick(Sender: TObject);
    procedure DO_GLButtonMouseEnter(Sender: TObject);
    procedure DO_GLButtonMouseLeave(Sender: TObject);
    procedure DO_ButtonClick(Sender: TObject);
	end;

implementation

uses SysUtils, GLCrossPlatform, VectorGeometry, GLMisc;

{ TfrmFunc00030 }

procedure TfrmFunc00030.CREATE_OBJECT;
begin
	inherited;
  FWindowsBitmapFont := TGLWindowsBitmapFont.Create(Self);
  with FWindowsBitmapFont do
  begin
  end;

  FMaterialLibrary := TGLMaterialLibrary.Create(Self);
  with FMaterialLibrary do
  	with AddTextureMaterial('Gui','.\DATA\DefaultSkin.bmp') do 
    begin
    	Material.Texture.Disabled := False;
	  end;

  FGuiLayout := TGLGuiLayout.Create(Self);
  with FGuiLayout do
  begin
  	BitmapFont := FWindowsBitmapFont;
    FileName := '.\DATA\default.layout';

    //GuiComponents.Add

    Material.LibMaterialName := 'Gui';
    Material.MaterialLibrary := FMaterialLibrary;
  end;

  FGLForm := TGLForm.Create(Self);
  FGLEdit := TGLEdit.Create(Self);
  FGLLabel := TGLLabel.Create(Self);
  FGLButton := TGLButton.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FGLForm);
	FGLForm.AddChild(FGLEdit);
  FGLForm.AddChild(FGLLabel);
  FGLForm.AddChild(FGLButton);

  with FScene do
	begin
  	ObjectsSorting := osNone;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 150;
  	Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(3,3,3);
    Up.AsVector := VectorMake(0,1,0);
    TargetObject := FDummy;
  end;

  with FLight do
	begin
    Position.AsVector := VectorMake(10,10,10);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FGLForm do
	begin
  	AlphaChannel := 0.5;
    Left := 100;
    Top := 100;
    Height := 170;
    Width := 300;
    BitmapFont := FWindowsBitmapFont;
    Caption := 'GUI Form';
    //DefaultColor := clMaroon;
		DefaultColor := clWhite;
    GuiLayout := FGuiLayout;
    GuiLayoutName := 'form';
    NoZWrite := False;
    Position.AsVector := VectorMake(100,100,0);
    TitleColor := clWhite;
    TitleOffset := 2;
  end;

  with FGLButton do
	begin
    OnButtonClick := DO_GLButtonClick;
    OnMouseEnter := DO_GLButtonMouseEnter;
    OnMouseLeave := DO_GLButtonMouseLeave;
  	AllowUp := True;
    AlphaChannel := 1;
    BitmapFont := FWindowsBitmapFont;
    DefaultColor := clWhite;
    FocusedColor := clRed;
    Caption := 'Set Title';
    GuiLayout := FGuiLayout;
    GuiLayoutName := 'panel';
    GuiLayoutNamePressed := 'panel';
    Height := 21;
    Left := 180;
    LogicHeight := 0;
    LogicWidth := 0;
    Position.AsVector := VectorMake(180,220,0);
    Rotation := 0;
    Top := 200;
    Width := 140;
  end;

  with FGLEdit do
	begin
    AlphaChannel := 1;
    OnMouseEnter := DO_GLButtonMouseEnter;
    OnMouseLeave := DO_GLButtonMouseLeave;
    BitmapFont := FWindowsBitmapFont;
    DefaultColor := clWhite;
    FocusedColor := clRed;
    Caption := 'New Title';
    EditChar := '*';
    GuiLayout := FGuiLayout;
    GuiLayoutName := 'panel';
    Height := 21;
    Left := 200;
    NoZWrite := False;
    Position.AsVector := VectorMake(200,170,0);
    Rotation := 0;
    Top := 170;
    Width := 180;
  end;

  with FGLLabel do
	begin
    AlphaChannel := 1;
    BitmapFont := FWindowsBitmapFont;
    DefaultColor := clWhite;
    Caption := 'Title';
    GuiLayout := FGuiLayout;
    GuiLayoutName := 'panel';
    Left := 110;
    Top := 170;
    Width := 80;
    Height := 21;
    NoZWrite := False;
    Position.AsVector := VectorMake(Left,Top,0);
    TextLayout := tlCenter;//TGLTextLayout: Unit: GLCrossPlatform
  end;

end;

procedure TfrmFunc00030.DO_ButtonClick(Sender: TObject);
var
	fd: TFontDialog;
begin
	case TButton(Sender).Tag of
  	0:begin
        fd := TFontDialog.Create(Self);
        try
          fd.Font := FWindowsBitmapFont.Font;
          if fd.Execute then
            FWindowsBitmapFont.Font := fd.Font;
        finally
          FreeAndNil(fd);
        end;
    	end;
    1:if Assigned(FGLForm) then
      	FGLForm.Visible := not FGLForm.Visible;
  end;
end;

procedure TfrmFunc00030.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	FGLForm.DoChanges;// make things move a little
  ROTATE_SCENE_OBJECT(FDummy,15,newTime,7);
  inherited;
end;

procedure TfrmFunc00030.DO_GLButtonClick(Sender: TObject);
Var
  S: String;
begin
  S := FGLForm.Caption;
  FGLForm.Caption := FGLEdit.Caption;
  FGLEdit.Caption := S;
end;

procedure TfrmFunc00030.DO_GLButtonMouseEnter(Sender: TObject);
begin
	FViewer.Cursor := crHandPoint;
end;

procedure TfrmFunc00030.DO_GLButtonMouseLeave(Sender: TObject);
begin
  FViewer.Cursor := crDefault;
end;

procedure TfrmFunc00030.DO_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  FGLForm.KeyDown(Sender,Key,Shift);
end;

procedure TfrmFunc00030.DO_KeyPress(Sender: TObject; var Key: Char);
begin
  FGLForm.KeyPress(Sender,Key);
end;

procedure TfrmFunc00030.DO_KeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  FGLForm.KeyUp(Sender,Key,Shift);
end;

procedure TfrmFunc00030.DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FGLForm.MouseDown(Sender,TGLMouseButton(Button),Shift,X,Y);
  if Shift = [ssRight] then inherited;
end;

procedure TfrmFunc00030.DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
	FGLForm.MouseMove(Sender,Shift,X,Y);
  if Shift = [ssRight] then inherited;
end;

procedure TfrmFunc00030.DO_ViewerMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FGLForm.MouseUp(Sender,TGLMouseButton(Button),Shift,X,Y);
end;

procedure TfrmFunc00030.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  inherited;
  FPanel := TPanel.Create(Self);
  for I := 0 to Length(FButton)-1 do
	  FButton[I] := TButton.Create(Self);

  with FPanel do
  begin
    SetBounds(0,1,120,53);
  	Parent := FViewer;
  end;

  for I := 0 to Length(FButton)-1 do
  	with FButton[I] do
	  begin
      case I of
      	0: Caption := 'Set New Font';
        1: Caption := 'GLForm on/off';
      end;
  		Tag := I;
    	OnClick := DO_ButtonClick;
    	SetBounds(1,I*26+1,117,25);
  		Parent := FPanel;
  	end;
end;

end.
