unit fFunc00031;

interface

uses Classes, fFunc, GLTexture, GLBitmapFont, GLTeapot, GLHUDObjects, GLTimeEventsMGR;

type
	TfrmFunc00031 = class(TfrmFunc)
  protected
    FBitmapFont: TGLBitmapFont;
    FTimeEventsMGR: TGLTimeEventsMGR;
    FTeapot: TGLTeapot;
    FHUDTexts: array[0..3] of TGLHUDText;//Unit: GLHUDObjects
    procedure CREATE_OBJECT; override;
    procedure DO_OnTimeEvents(event: TTimeEvent);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
	end;

implementation

uses SysUtils, GLMisc, GLUtils, VectorGeometry, VectorTypes, GLCrossPlatform;

const
	FadeOutMax = 100;
  FadeInMax  = 100;
  OverallTrans = 0.7;

var
  FadeOutCount: Integer;
  FadeInCount: Integer;
  OriginalColor: TVector4f;

{ TfrmFunc00031 }

procedure TfrmFunc00031.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  
  FBitmapFont := TGLBitmapFont.Create(Self);
  with FBitmapFont do
  begin
  	CharHeight := 32;
    CharWidth := 27;
    GlyphsAlpha := tiaDefault;
    GlyphsIntervalX := 7;
    GlyphsIntervalY := 0;
    HSpace := 0;
    VSpace := 1;
    MagFilter := maLinear;//Unit: GLUtils
    MinFilter := miLinear;//Unit: GLUtils
    for I := 0 to 3 do
    	with Ranges.Add do
	    	case I of
  	    	0:begin StartASCII := 'A'; StopASCII := 'H'; StartGlyphIdx := 0; end;
    	    1:begin StartASCII := 'I'; StopASCII := 'P'; StartGlyphIdx := 8; end;
          2:begin StartASCII := 'Q'; StopASCII := 'X'; StartGlyphIdx :=16; end;
          3:begin StartASCII := '!'; StopASCII := '!'; StartGlyphIdx :=27; end;
      	end;
   	Glyphs.LoadFromFile('.\DATA\toonfont.bmp');
  end;

  FTimeEventsMGR := TGLTimeEventsMGR.Create(Self);
  with FTimeEventsMGR do
  begin
  	Cadencer := FCadencer;
    Enabled := True;
    for I := 0 to 2 do
	    with Events.Add do
  	  begin
    		Name := 'Event' + IntToStr(I);
    		Enabled := True;
        case I of
        	0:begin StartTime := 1.5; EndTime := 3; EventType := etContinuous;end;
          1:begin StartTime := 0	; EndTime := 0; EventType := etOneShot; 	end;
          2:begin StartTime := 2	; EndTime :=10; EventType := etContinuous;end;
        end;
        OnEvent := DO_OnTimeEvents;
	    end;
  end;

  FTeapot := TGLTeapot.Create(Self);
  for I := 0 to Length(FHUDTexts)-1 do
  	FHUDTexts[I] := TGLHUDText.Create(Self);

	FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FTeapot);
  for I := 0 to Length(FHUDTexts)-1 do
  	FDummy.AddChild(FHUDTexts[I]);

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 300;
  	TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(4,4,4);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(2,2,2);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FTeapot do
	begin
  	Material.FrontProperties.Diffuse.Color := VectorMake(0.6117647,0.6980392,1,1);
    Scale.AsVector := VectorMake(1.5,1.5,1.5);
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  for I := 0 to Length(FHUDTexts)-1 do
  	with FHUDTexts[I] do
		begin
    	Tag := I;
    	Alignment := taCenter;
      Layout := tlTop;
      BitmapFont := FBitmapFont;
      case I of
				0:Text := 'FADING OUT';
        1:Text := 'THE END';
        2:Text := 'RED COLOR';
        3:Text := 'TRANSPARENT';
      end;
      case I of
      	0:ModulateColor.Color := VectorMake(1,1,1,0.7);
        1:ModulateColor.Color := VectorMake(1,0.5,0,0.7);
        2:ModulateColor.Color := VectorMake(1,0,0,0.7);
        3:ModulateColor.Color := VectorMake(0.85,0.85,0.1,0.4);
      end;
      Direction.AsVector := VectorMake(0,0,1);
      Up.AsVector := VectorMake(0,1,0);
      Position.AsVector := VectorMake(180,I*40+50,0);
  	end;
end;

procedure TfrmFunc00031.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FDummy, 15, newTime, 7);
  inherited;
end;

procedure TfrmFunc00031.DO_OnTimeEvents(event: TTimeEvent);
var
	S: string;
	I: Integer;
  A: Single;
  NewColor: TVector4f;
begin
	S := event.Name;
	I := StrToInt(Copy(S,6,Length(S)-5));
  A := (FadeOutCount / FadeOutMax) * OverallTrans;
	case I of
  	0:begin
   			if FadeOutCount < 0 then EXIT;
   			FHUDTexts[0].ModulateColor.Alpha := A;
   			Dec(FadeOutCount);
    	end;
    1:begin
   			FadeOutCount := FadeOutMax;
   			FadeInCount := 0;
   			OriginalColor := FHUDTexts[1].ModulateColor.Color;
   			FHUDTexts[0].ModulateColor.Alpha := A;
   			FHUDTexts[1].ModulateColor.Color := VectorMake(1, 1, 1, 0);
    	end;
    2:begin
   			if FadeInCount >= FadeInMax then EXIT;
   			NewColor := VectorScale(OriginalColor, FadeInCount/FadeInMax);
   			FHUDTexts[1].ModulateColor.Color := NewColor;
   			Inc(FadeInCount);
    	end;
  end;
end;

end.

