unit fFunc00032;

interface

uses
	Classes, Controls, StdCtrls, Graphics, ExtCtrls, fFunc, GLGraph,
	GLTexture, VectorTypes, VectorGeometry;

type
	TfrmFunc00032 = class(TfrmFunc)
  protected
    FHeightField: TGLHeightField;//Unit: GLGraph
    FPanel: TPanel;
    FLabels: array[0..3] of TLabel;
    FRadioButtons: array[0..1] of TRadioButton;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_HeightFieldGetHeight(const x, y: Single; var z: Single; var color: TVector4f; var texPoint: TTexPoint);
  public
    Grid: array[-5..5, -5..5] of TColor;
  end;

implementation

uses SysUtils, GLMisc;

{ TfrmFunc00032 }

procedure TfrmFunc00032.CREATE_OBJECT;
var
	X, Y: Integer;
begin
  inherited;
  FHeightField := TGLHeightField.Create(Self);
  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FHeightField);

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

	with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 50;
    TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(5,4,3);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Ambient.Color := clrGray15;
    Diffuse.Color := clrGray85;
    Position.AsVector := VectorMake(7,15,5);
    Direction.AsVector := VectorMake(0,0,-1);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
//    VisibleAtRuntime := True;
  end;

  with FHeightField do
	begin
  	ColorMode := hfcmAmbientAndDiffuse;
    Options := [hfoTwoSided];
    Scale.AsVector := VectorMake(0.6,0.6,0.5);
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0,-1);

    OnGetHeight := DO_HeightFieldGetHeight;
    
    with XSamplingScale do
    begin
    	Max := 5.5;
      Min := -5.5;
      Origin := 0;
      Step := 0.2;
    end;

    with YSamplingScale do
    begin
    	Max := 5.5;
      Min := -5.5;
      Origin := 0;
      Step := 0.2;
    end;

  end;

  // initialize grid color to white/gray (checked pattern)
  for X := -5 to 5 do
  	for Y:= -5 to 5 do
    	if ((X xor Y) and 1) = 0 then Grid[X, Y] := clWhite
    	else                          Grid[X, Y] := clSilver;

end;

procedure TfrmFunc00032.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	if FRadioButtons[1].Checked then
		ROTATE_SCENE_OBJECT(FDummy,15,newTime,2);
  inherited;
end;

procedure TfrmFunc00032.DO_HeightFieldGetHeight(const x, y: Single;
	var z: Single; var color: TVector4f; var texPoint: TTexPoint);
var
	ix, iy : Integer;
begin
   // Nothing fancy here, the color is directly taken from the grid,
   // and the z function is a basic cosinus. The '+0.01' are to take
   // rounding issues out of the equation.
   ix := Round(ClampValue(x+0.01, -5, 5));
   iy := Round(ClampValue(y+0.01, -5, 5));
   Color := ConvertWinColor(Grid[ix, iy]);
   z := Cos(VectorLength(x, y)*1.5);
end;

procedure TfrmFunc00032.DO_ViewerMouseDown(Sender: TObject;
	Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
	v: TAffineVector;
  ix, iy: Integer;
begin
  if FRadioButtons[1].Checked then inherited
  else if FRadioButtons[0].Checked then
  begin
   	mx := X;
   	my := Y;
    // In Paint mode
    // get absolute 3D coordinates of the point below the mouse
    v := FViewer.Buffer.PixelRayToWorld(X, Y);
    // convert to heightfield local coordinates
    v := FHeightField.AbsoluteToLocal(v);
    // convert that local coords to grid pos
    ix := Round(v[0]);
    iy := Round(v[1]);
    // if we are in the grid...
    if (ix>=-5) and (ix<=5) and (iy>=-5) and (iy<=5) then
    begin
    	// show last coord in the caption bar
      if Assigned(FLabels[3]) then
      	FLabels[3].Caption := Format('Last Coord: X=%d, Y=%d', [ix, iy]);
      // and paint blue or red depending on the button
      if Button = mbLeft then Grid[ix, iy] := clBlue
      else 										Grid[ix, iy] := clRed;
      // Height field changed, rebuild it!
      FHeightField.StructureChanged;
    end;
  end;
end;

procedure TfrmFunc00032.DO_ViewerMouseMove(Sender: TObject;
	Shift: TShiftState; X, Y: Integer);
begin
  if FRadioButtons[1].Checked then inherited
  else if FRadioButtons[0].Checked then
  begin
  	// in paint mode, paint if a button is pressed
    			if ssLeft in Shift 	then DO_ViewerMouseDown(Sender, mbLeft,  Shift, x, y)
    else 	if ssRight in Shift then DO_ViewerMouseDown(Sender, mbRight, Shift, x, y);
  end;
end;

procedure TfrmFunc00032.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  inherited;
  FPanel := TPanel.Create(Self);
  for I := 0 to Length(FLabels)-1 do
  	FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FRadioButtons)-1 do
  	FRadioButtons[I] := TRadioButton.Create(Self);

  with FPanel do
  begin
  	SetBounds(0,1,120,250);
    Parent := FViewer;
  end;

  for I := 0 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
    	AutoSize := False;
    	Font.Name := 'ARIAL';
    	if I in [0,1] then Font.Height := 20;
      if I in [0,1] then Font.Style := Font.Style + [fsBold];
      if I = 2 then WordWrap := True;
      if I = 3 then Font.Color := clBlue;

      case I of
      	0: Caption := 'HeightField';
        1: Caption := 'Picking';
        2: Caption := 'This demo uses a crude method for ' +
        						  'heightfield picking based on the Z' +
                      '-Buffer. This method can be easily' +
                      ' adapted for a variety of objects ' +
                      'and 2.5D problems.';
        3: Caption := 'Last Coord: X=*, Y=*';
      end;
      case I of
      	0:SetBounds(1,0,118,20);
      	1:SetBounds(1,25,118,20);
        2:SetBounds(1,110,118,120);
        3:SetBounds(1,230,118,20);
      end;

      Parent := FPanel;
    end;

  for I := 0 to Length(FRadioButtons)-1 do
  	with FRadioButtons[I] do
    begin
    	Checked := I = 0;
    	case I of
      	0: Caption := 'Paint';
        1: Caption := 'Rotate';
      end;
      case I of
      	0:SetBounds(1,50,118,20);
      	1:SetBounds(1,75,118,20);
      end;
      Parent := FPanel;
    end;

end;

end.
