unit fFunc00033;

interface

uses Controls, Graphics, StdCtrls, ExtCtrls, fFunc, GLObjects, GLGeomObjects, GLVectorFileObjects,
	GLFeedback, GLPolyhedron, GLTexture;

type
	TfrmFunc00033 = class(TfrmFunc)
  protected
    FFreeForm: TGLFreeForm;//Unit: GLVectorFileObjects
    FFeedback: TGLFeedback;//Unit: GLFeedback
    FMeshObjects: array[0..1] of TGLDummyCube;
    FCube: TGLCube;
    FSphere: TGLSphere;
    FDodecahedron: TGLDodecahedron;//Unit: GLPolyhedron
    FPanel: TPanel;
    FButton: TButton;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_ButtonClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;


implementation

uses VectorGeometry, GLMisc;

{ TfrmFunc00033 }

procedure TfrmFunc00033.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FFreeForm := TGLFreeForm.Create(Self);
  FFeedback := TGLFeedback.Create(Self);
  for I := 0 to Length(FMeshObjects)-1 do
  	FMeshObjects[I] := TGLDummyCube.Create(Self);
  FCube := TGLCube.Create(Self);
  FSphere := TGLSphere.Create(Self);
  FDodecahedron := TGLDodecahedron.Create(Self);

  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FCamera);
    			FCamera.AddChild(FLight);
  FScene.Objects.AddChild(FFreeForm);
  FScene.Objects.AddChild(FFeedback);
  		FFeedback.AddChild(FMeshObjects[0]);
    			FMeshObjects[0].AddChild(FCube);
      		FMeshObjects[0].AddChild(FDodecahedron);
    	FFeedback.AddChild(FMeshObjects[1]);
    			FMeshObjects[1].AddChild(FSphere);

  with FViewer do
  begin
  	Buffer.AmbientColor.Color := clrGray20;
  end;

  with FScene do
  begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FDummy do
  begin
//  	ShowAxes := True;
  //  VisibleAtRuntime := True;
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0); 
    Up.AsVector := VectorMake(0,1,0);       
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 100;
    TargetObject := FDummy;
  	Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(1,2,3); 
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
  begin
    Position.AsVector := VectorMake(0,0,0);
  end;

  with FFreeForm do
  begin
  	UseMeshMaterials := True;
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FFeedback do
  begin
  	Active := False;/////////
    MaxBufferSize := 1048576;
    Mode := fm3DColorTexture;
    Visible := False;////////
  end;

  for I := 0 to Length(FMeshObjects)-1 do
	  with FMeshObjects[I] do
   	begin
      Direction.AsVector := VectorMake(0,0,1);
      Position.AsVector := VectorMake(0,0,0);
      Up.AsVector := VectorMake(0,1,0);
      Visible := False;
    end;

  with FCube do
  begin
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(-0.65,-0.65,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FDodecahedron do
  begin
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0.65,-0.65,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FSphere do
  begin
		Radius := 0.5;
    Slices := 16;
    Stacks := 16;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0.65,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

end;

procedure TfrmFunc00033.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FFreeForm,-30,newTime,2);
  inherited;
end;

procedure TfrmFunc00033.SETUP_USER_INTERFACE;
begin
  FPanel := TPanel.Create(Self);
  FButton := TButton.Create(Self);

  with FPanel do
  begin
    SetBounds(0,1,150,30);
  	Parent := FViewer;
  end;

  with FButton do
  begin
  	Caption := 'Build Mesh';
    Cursor := crHandPoint;
  	OnClick := DO_ButtonClick;
    SetBounds(2,1,146,27);
  	Parent := FPanel;
  end;

end;

procedure TfrmFunc00033.DO_ButtonClick(Sender: TObject);
var
	mo: TMeshObject;
  fg: TFGIndexTexCoordList;
begin
  // Clear our freeform of any meshes
  FFreeForm.MeshObjects.Clear;
  // Set feedback to active, will feedback render child
  // objects into it's buffer
  FFeedback.Active:=True;
  // Process the first mesh object (GLCube and GLDodecahedron)
  // Make the objects visible that we want to buffer
  FMeshObjects[0].Visible:=True;
  // Render the feedback object to buffer it's child object
  // that are visible
  FViewer.Buffer.Render(FFeedback);
  // Hide the child objects we rendered
  FMeshObjects[0].Visible:=False;
  // Create a new mesh object in our freeform
  mo:=TMeshObject.CreateOwned(FFreeForm.MeshObjects);
  mo.Mode:=momTriangles;
  // Process the feedback buffer for polygon data
  // and build a mesh (normals are recalculated
  // since feedback only yields position and
  // texcoords)
  FFeedback.BuildMeshFromBuffer(mo.Vertices, mo.Normals, mo.TexCoords, nil);
  // Process the second mesh object (GLSphere)
  // (comments from first mesh object apply here also)
  FMeshObjects[1].Visible:=True;
  FViewer.Buffer.Render(FFeedback);
  FMeshObjects[1].Visible:=False;
  // Vertex indices are required for smooth normals
  mo:=TMeshObject.CreateOwned(FFreeForm.MeshObjects);
  mo.Mode:=momFaceGroups;
  fg:=TFGIndexTexCoordList.CreateOwned(mo.FaceGroups);
  fg.Mode:=fgmmTriangles;
  FFeedback.BuildMeshFromBuffer(mo.Vertices, mo.Normals, fg.TexCoords, fg.VertexIndices);
  // Deactivate the feedback object
  FFeedback.Active:=False;
  FFreeForm.StructureChanged;
end;


end.
