unit fFunc00035;

interface

uses Classes, Controls, ExtCtrls, Graphics, fFunc, GLScene, GLObjects,
	GLGeomObjects, GLVectorFileObjects, GLTexture;

type
	TfrmFunc00035 = class(TfrmFunc)
  protected
    FFreeForm: TGLFreeForm;
    FSphere: TGLSphere;
    FArrowLine: TGLArrowLine;
    FPanel: TPanel;
  	procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
  end;

implementation

uses VectorGeometry, GLMisc, VectorTypes;

{ TfrmFunc00035 }

procedure TfrmFunc00035.CREATE_OBJECT;
begin
  inherited;
  FFreeForm := TGLFreeForm.Create(Self);
  FSphere := TGLSphere.Create(Self);
  FArrowLine := TGLArrowLine.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FFreeForm);
  FScene.Objects.AddChild(FSphere);
      FSphere.AddChild(FArrowLine);

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	CameraStyle := csOrthogonal;/////////
    DepthOfView := 100;
    FocalLength := 10;
    TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(13,3,9);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
    Position.AsVector := VectorMake(50,60,70);
  end;

  with FDummy do
	begin
		ShowAxes := True;
    VisibleAtRuntime := True;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FFreeForm do
	begin
  	NormalsOrientation := mnoInvert;
    Scale.AsVector := VectorMake(0.07,0.07,0.07);
		ShowAxes := True;
    UseMeshMaterials := True;
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FSphere do
	begin
  	Material.FrontProperties.Emission.Color := clrYellow;
    Material.FrontProperties.Emission.Alpha := 0.6;
    Slices := 16;
    Stacks := 16;
    Radius := 0.3;
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(-0.95,0,0.32);
  end;

  with FArrowLine do
	begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.2;
    BottomRadius := 0.05;
    HeadStackingStyle := ahssStacked;
    Height := 1;
    Loops := 1;
    Parts := [alLine,alTopArrow];
    Slices := 16;
    Stacks := 4;
  	TopArrowHeadHeight := 0.5;
    TopArrowHeadRadius := 0.2;
    TopRadius := 0.1;
    Material.BlendingMode := bmTransparency;
    Material.FrontProperties.Diffuse.Color := VectorMake(0.8,0.8,0.8,0.6);
    Material.FrontProperties.Emission.Color := VectorMake(0.251,0.251,0,1);
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0.1);
    Up.AsVector := VectorMake(0,1,0);
  end;

  // Load mushroom mesh
  FFreeForm.LoadFromFile('.\DATA\mushroom.3ds');

end;

procedure TfrmFunc00035.DO_ViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   rayStart, rayVector, iPoint, iNormal : TVector;
begin
	if Shift = [ssRight] then inherited
  else begin
    // retrieve raycasting data:
    //    rayStart is obtained for camera and screen position
    //    rayVector is the camera direction (i.e direction to target since our camera is targeted)
    // (note that (0, 0) is lower left for the Screen function, whereas Delphi
    //  uses top-left as origin, hence the Y inversion)
    SetVector(rayStart, FViewer.Buffer.OrthoScreenToWorld(x, FViewer.Height-y));
    SetVector(rayVector, FCamera.AbsoluteVectorToTarget);
    NormalizeVector(rayVector);
    // Here we require RayCast intersection
    if FFreeForm.RayCastIntersect(rayStart, rayVector, @iPoint, @iNormal) then begin
      // got one, move the sphere there and orient it appropriately
      FSphere.Position.AsVector:=iPoint;
      FSphere.Direction.AsVector:=VectorNormalize(iNormal);
      // make it visible
      FSphere.Visible:=True;
    end else begin
      // hide it if we did not hit
      FSphere.Visible:=False;
    end;
  end;
end;

procedure TfrmFunc00035.DO_ViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
	if Shift = [ssRight] then inherited
  else begin
		// when mouse moves, recompute intersection
		if Shift<>[] then DO_ViewerMouseDown(Sender, mbLeft, Shift, x, y);
  end;
end;

procedure TfrmFunc00035.SETUP_USER_INTERFACE;
begin
  inherited;
  FPanel := TPanel.Create(Self);
  FPanel.Caption := 'Orthogonal View';
  FPanel.Font.Name := 'ARIAL';
  FPanel.Font.Height := 20;
  FPanel.Font.Color := clBlue;
  FPanel.Font.Style := FPanel.Font.Style + [fsBold];
  FPanel.SetBounds(0,1,150,25);
  FPanel.Parent := FViewer;
end;

end.

