unit fFunc00036;

interface

uses Classes, Controls, ExtCtrls, Graphics, fFunc00035, GLScene, GLObjects,
	GLGeomObjects, GLVectorFileObjects, GLTexture;

type
	TfrmFunc00036 = class(TfrmFunc00035)
  protected
		procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
  end;

implementation

uses VectorGeometry, GLMisc, VectorTypes;

{ TfrmFunc00036 }

procedure TfrmFunc00036.CREATE_OBJECT;
begin
  inherited;
	with FCamera do
  begin
  	CameraStyle := csPerspective;
    DepthOfView := 1000;
    FocalLength := 250;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(10,10,10);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FFreeForm do
  begin
  	Material.FrontProperties.Ambient.Color := clrRed;
  	Material.FrontProperties.Diffuse.Color := clrLime;
 		Material.FrontProperties.Diffuse.Alpha := 0.7;

  end;

end;

procedure TfrmFunc00036.DO_ViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   rayStart, rayVector, iPoint, iNormal : TVector;
begin
	if Shift = [ssRight] then inherited
  else begin
    // retrieve raycasting data:
    //    rayStart is the eye (camera) position
    //    rayVector is computed from screen position
    // (note that (0, 0) is lower left for the Screen function, whereas Delphi
    //  uses top-left as origin, hence the Y inversion)
    SetVector(rayStart, FCamera.AbsolutePosition);
    SetVector(rayVector, FViewer.Buffer.ScreenToVector(AffineVectorMake(x, FViewer.Height-y, 0)));
    NormalizeVector(rayVector);
    // Here we request RayCast intersection
    if FFreeForm.RayCastIntersect(rayStart, rayVector, @iPoint, @iNormal) then begin
      // got one, move the sphere there and orient it appropriately
      FSphere.Position.AsVector:=iPoint;
      FSphere.Direction.AsVector:=VectorNormalize(iNormal);
      // make it visible
      FSphere.Visible:=True;
    end else begin
      // hide it if we did not hit
      FSphere.Visible:=False;
    end;
  end;
end;

procedure TfrmFunc00036.DO_ViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
	if Shift = [ssRight] then inherited
  else begin
		if Shift<>[] then DO_ViewerMouseDown(Sender, mbLeft, Shift, x, y);
  end;
end;

procedure TfrmFunc00036.SETUP_USER_INTERFACE;
begin
  inherited;
  FPanel.Caption := 'Perspective View';
  FPanel.Font.Color := clRed;
end;

end.
