unit fFunc00037;

interface

uses Classes, Controls, Forms, StdCtrls, ExtCtrls, ComCtrls, fFunc,
	GLScene, GLObjects, GLGeomObjects, GLVectorFileObjects,
  GLTexture,VectorGeometry;

type
	TfrmFunc00037 = class(TfrmFunc)
  private
    ColTotalTime: Single; // for timing collision detection
    ColCount: Integer;
    procedure AddToTrail(const p : TVector);
	protected
    FLight2: TGLLightSource;
    FDummy2: TGLDummyCube;
    FLines: TGLLines;
    FSpheres: array[0..1] of TGLSphere;
    FFreeForm: TGLFreeForm;
    FArrowLine: TGLArrowLine;
    FPanel: TPanel;
    FLabels: array[0..3] of TLabel;
    FButton: TButton;
    FTrackBar: TTrackBar;
  	procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_Timer(Sender: TObject); override;
    procedure DO_ButtonClick(Sender: TObject);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses SysUtils, GLMisc, GLCrossPlatform;

{ TfrmFunc00037 }

procedure TfrmFunc00037.AddToTrail(const p: TVector);
var
	I, K: Integer;
begin
	FLines.Nodes.Last.AsVector := p;
  FLines.AddNode(0, 0, 0);
  if FLines.Nodes.Count > 20 then // limit trail to 20 points
  	FLines.Nodes[0].Free;
  for I := 0 to 19 do
  begin
  	K := FLines.Nodes.Count - I - 1;
    if K >= 0 then TGLLinesNode(FLines.Nodes[k]).Color.Alpha := 0.95 - i * 0.05;
  end;
end;

procedure TfrmFunc00037.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FLight2 := TGLLightSource.Create(Self);
  FDummy2 := TGLDummyCube.Create(Self);
  FLines := TGLLines.Create(Self);
  for I := 0 to Length(FSpheres)-1 do
  	FSpheres[I] := TGLSphere.Create(Self);
  FFreeForm := TGLFreeForm.Create(Self);
  FArrowLine := TGLArrowLine.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FLight2);
  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FFreeForm);
  FScene.Objects.AddChild(FDummy2);
  FScene.Objects.AddChild(FSpheres[0]);
      FSpheres[0].AddChild(FArrowLine);
  FScene.Objects.AddChild(FSpheres[1]);
  FScene.Objects.AddChild(FLines);

  with FViewer.Buffer do
  begin
    AmbientColor.Color := clrGray20;
    BackgroundColor := $00804000;
    Lighting := True;
    ShadeModel := smFlat;
  end;

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 50;
    TargetObject := FSpheres[1];
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,4,-3);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Ambient.Color := VectorMake(0,0.502,0.753,1);
    Diffuse.Color := clrGray50;
    LightStyle := lsOmni;
    Position.AsVector := VectorMake(50,150,100);
  end;

  with FLight2 do
	begin
  	Ambient.Color := VectorMake(0,0.251,0.502,1);
    Diffuse.Color := clrGray;
    LightStyle := lsSpot;
    Position.AsVector := VectorMake(0,0,0);
  end;

  with FDummy do
	begin
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FDummy2 do
	begin
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FSpheres[0] do
	begin
  	Slices := 6;
    Stacks := 6;
    Radius := 0.1;
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(-0.9486833,1,0.3162278);
    Material.FrontProperties.Emission.Color := clrYellow;
  end;

  with FSpheres[1] do
	begin
  	Slices := 24;
    Stacks := 24;
    Radius := 20;
    Direction.AsVector := VectorMake(0.669438,-0.2276488,-0.7071272);
    Position.AsVector := VectorMake(0,7,-2);
    Up.AsVector := VectorMake(0.3810556,0.9223475,0.06380997);
    Material.FrontProperties.Diffuse.Color := clrRed;
    Material.FrontProperties.Shininess := 128;
    VisibilityCulling := vcObjectBased;
  end;

  with FLines do
	begin
  	Division := 10;
    LinePattern := $FFFF;
    LineWidth := 3;
    NodesAspect := lnaCube;
    NodeSize := 10;
    Options := [loUseNodeColorForLines];
    SplineMode := lsmLines;
    LineColor.Color := VectorMake(0,0,1);
    NodeColor.Color := clrYellow;
    Direction.AsVector := VectorMake(0.669438,-0.2276488,-0.7071272);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FFreeForm do
	begin
  	UseMeshMaterials := True;
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FArrowLine do
	begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.2;
    BottomRadius := 0.05;
    HeadStackingStyle := ahssStacked;
    Height := 1;
    Loops := 1;
    Parts := [alLine,alTopArrow];
    Slices := 16;
    Stacks := 4;
  	TopArrowHeadHeight := 2;
    TopArrowHeadRadius := 0.2;
    TopRadius := 0.1;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0.1);
    Up.AsVector := VectorMake(0,1,0);
    Material.BlendingMode := bmTransparency;
    Material.FrontProperties.Diffuse.Color := VectorMake(0.8,0.8,0.8,0.6);
    Material.FrontProperties.Emission.Color := VectorMake(0.251,0.251,0,1);
  end;

   FFreeForm.LoadFromFile('.\DATA\BoxedIn.3ds');
   FFreeForm.BuildOctree;
   FLines.AddNode(0, 0, 0);
   FLines.ObjectStyle := FLines.ObjectStyle + [osDirectDraw];

end;

procedure TfrmFunc00037.DO_ButtonClick(Sender: TObject);
begin
  //If the ball gets stuck in a pattern, hit the reset button.
  With FSpheres[1].Position do
  begin
   	X := Random;
   	Y := Random;
   	Z := Random;
  end;

  With FSpheres[1].Direction do
  begin
   	X := Random;  if Random > 0.5 then x := -x;
   	Y := Random;  if Random > 0.5 then y := -y;
   	Z := Random;  if Random > 0.5 then z := -z;
  end;
end;

procedure TfrmFunc00037.DO_Timer(Sender: TObject);
const
	FMT: string = '%.2f FPS - %.3f ms for collisions/frame';
var
	T: Single;
begin
	if ColCount > 0 then 	T := ColTotalTime * 1000 / colCount
  else 									T := 0;
	TForm(Owner).Caption := Format(FMT, [FViewer.FramesPerSecond, T]);
	FViewer.ResetPerformanceMonitor;
  ColTotalTime := 0;
  ColCount := 0;
end;

procedure TfrmFunc00037.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  FPanel := TPanel.Create(Self);
  for I := 0 to Length(FLabels)-1 do
  	FLabels[I] := TLabel.Create(Self);
  FButton := TButton.Create(Self);
  FTrackBar := TTrackBar.Create(Self);

  with FPanel do
  begin
  	SetBounds(0,1,200,160);
    Parent := FViewer;
  end;

  for I := 0 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
    	AutoSize := False;
    	Caption := '---';
      if I = 0 then Caption := 'Velocity';
    	case I of
      	0: SetBounds(1,1,198,20);
        else SetBounds(1,90+(I-1)*20,198,20);
      end;
      Parent := FPanel;
    end;

	with FTrackBar do
  begin
  	Frequency := 1;
    LineSize := 1;
    Max := 50;
    Min := 0;
    PageSize := 2;
    Position := 1;
    SliderVisible := True;
    ThumbLength := 20;
    TickMarks := tmBottomRight;
    TickStyle := tsAuto;
    SetBounds(1,20,180,25);
    Parent := FPanel;
  end;

  with FButton do
  begin
  	Caption := 'Reset';
    OnClick := Self.DO_ButtonClick;
    SetBounds(100,50,75, 27);
    Parent := FPanel;
  end;

   FLabels[1].Caption := 'Octree Nodes    : '+ IntToStr(FFreeForm.Octree.NodeCount);
   FLabels[2].Caption := 'Tri Count Octree: '+ IntToStr(FFreeForm.Octree.TriCountOctree);
   FLabels[3].Caption := 'Tri Count Mesh  : '+ IntToStr(FFreeForm.Octree.TriCountMesh);

end;

procedure TfrmFunc00037.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
var
	rayStart, rayVector : TVector;
  velocity : Single;
  pPoint : TVector;
  pNormal : TVector;
  t : Int64;
begin
	Velocity := FTrackbar.Position*deltaTime*50;
  t := StartPrecisionTimer;
  with FFreeForm do
  begin
  	SetVector(rayStart, FSpheres[1].AbsolutePosition);
    SetVector(rayVector, FSpheres[1].AbsoluteDirection);
    NormalizeVector(rayVector);
    //Note: since collision may be performed on multiple meshes, we might need to know which hit
    //      is closest (ie: d:=raystart - pPoint).
    if OctreeSphereSweepIntersect(raystart, rayvector, velocity, FSpheres[1].Radius, @pPoint, @pNormal) then
    begin
    	// Show the polygon intersection point
      NormalizeVector(pNormal);
      FSpheres[0].Position.AsVector := pPoint;
      FSpheres[0].Direction.AsVector := pNormal;
     	// Make it rebound...
      with FSpheres[1].Direction do
      	AsAffineVector:=VectorReflect(AsAffineVector, AffineVectorMake(pNormal));
      // Add some "english"...
      with FSpheres[1].Direction do
      begin
        X := x + random / 10;
        Y := y + random / 10;
        Z := z + random / 10;
      end;
      // Add intersect point to trail
      AddToTrail(pPoint);
    end
    else begin
    	FSpheres[1].Move(velocity); //No collision, so just move the ball.
    end;
  end;
  // Last trail point is always the sphere's current position
  FLines.Nodes.Last.AsVector := FSpheres[1].Position.AsVector;
  colTotalTime := colTotalTime + StopPrecisionTimer(t);
  Inc(colCount);
end;

end.
