unit fFunc00038;

interface

uses fFunc, GLExtrusion, GLObjects, GLMisc;

type
	TfrmFunc00038 = class(TfrmFunc)
  protected
		FExtrusionSolid: TGLExtrusionSolid;
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00038 }

procedure TfrmFunc00038.CREATE_OBJECT;
const
  cSteps = 16;
var
  I: Integer;
  r, x, y: Single;
begin
  inherited;
  FExtrusionSolid := TGLExtrusionSolid.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FExtrusionSolid);

  with FViewer.Buffer do
  begin
    Lighting := True;
  end;

  with FScene do
	begin
  	ObjectsSorting := osRenderBlendedLast;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 150;
    TargetObject := FExtrusionSolid;
    Direction.AsVector := VectorMake(-0.8944272,0.4472136,0);
    Position.AsVector := VectorMake(6,8,10);
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FLight do
	begin
    Position.AsVector := VectorMake(50,40,60);
  end;                                

  with FExtrusionSolid do
	begin
  	Height := 1;
    MinSmoothAngle := 5;
    Normals := nsFlat;
    Parts := [espOutside,espStartPolygon,espStopPolygon];
    Stacks := 1;

    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  // a small star contour
  with FExtrusionSolid.Contours do
  begin
    with Add.Nodes do
    for I := 0 to cSteps do
    	begin
       	r := 2 + (I and 1) * 2;
       	SinCos(i * c2PI / cSteps, y, x);
       	AddNode(x*r, y*r, 0);
    	end;
    // add an empty contour for the square cutout (see progress event)
    Add;
  end;
end;

procedure TfrmFunc00038.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
var
	x, y: Single;
begin
  // Make our Extrusion roll
  FExtrusionSolid.Roll(deltaTime*10);

  // At each frame, we drop the cutout and make a new.
  // Note that we could also have defined it once in the FormCreate and then moved
  // it around with the TGLNodes methods.
  SinCos(newTime, 2, y, x);
  with FExtrusionSolid.Contours do
  begin
    Items[1].Free;
    with Add.Nodes do
    begin
    	AddNode(x-1, y-1, 0);
      AddNode(x+1, y-1, 0);
      AddNode(x+1, y+1, 0);
      AddNode(x-1, y+1, 0);
    end;
  end;
  inherited;
end;

end.
