unit fFunc00040;

interface

uses Classes, Graphics, Controls, StdCtrls, ExtCtrls, fFunc, GLObjects, GLTexture, GLVectorFileObjects;

type
	TfrmFunc00040 = class(TfrmFunc)
  protected
    FReference: TGLActor;
    FMaterialLibrary: TGLMaterialLibrary;
    FActors: TGLDummyCube;
    FSpheres: TGLDummyCube;
    FPanels: array[0..1] of TPanel;
    FLabels: array[0..1] of TLabel;
    FRadioBtnsA: array[0..2] of TRadioButton;
    FRadioBtnsB: array[0..1] of TRadioButton;
    FCheckBox: TCheckBox;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
		procedure DO_RadioBtnAClick(Sender: TObject);
    procedure DO_RadioBtnBClick(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
	end;

implementation

uses VectorGeometry, Jpeg, GLFileMD2, GLMisc;

{ TfrmFunc00040 }

procedure TfrmFunc00040.CREATE_OBJECT;
var
	I, J: Integer;
  NewSphere: TGLSphere;
  NewActor: TGLActor;
begin
  inherited;
  FMaterialLibrary := TGLMaterialLibrary.Create(Self);
  FMaterialLibrary.AddTextureMaterial('ActorTexture', '.\DATA\waste.jpg');
  FMaterialLibrary.Materials[0].Material.Texture.Disabled := False;

  FActors := TGLDummyCube.Create(Self);
  FSpheres := TGLDummyCube.Create(Self);
  FReference := TGLActor.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FActors);
  FDummy.AddChild(FSpheres);
  FDummy.AddChild(FReference);

  with FViewer do
  begin
  	Buffer.BackgroundColor := clBtnFace;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
  	FocalLength := 100;
  	TargetObject := FDummy;
    Position.AsVector := VectorMake(10,5,10);
  end;

  with FLight do Position.AsVector := VectorMake(10,5,10);

  with FDummy do
	begin
  	EdgeColor.Color := clrBlack;
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FActors do Visible := False;

  with FReference do
	begin
  	AnimationMode := aamLoop;
  	FrameInterpolation := afpLinear;
    Interval := 100;
    Options := [aoSkeletonNormalizeNormals];
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0.05,0.05,0.05);
    Scale.AsVector := VectorMake(0.05,0.05,0.05);
    Up.AsVector := VectorMake(0,0,-1);
    Reference := aarMorph;
    UseMeshMaterials := True;
    with Material do
    begin
    	LibMaterialName := 'ActorTexture';
    	MaterialLibrary := FMaterialLibrary;
    end;
  end;

  // Spheres are used as standalone, high-polycount objects
  // that are highly T&L friendly
  for I := -4 to 4 do
  	for j := -4 to 4 do
    begin
	    NewSphere := FSpheres.AddNewChild(TGLSphere) as TGLSphere;
  	  NewSphere.Position.SetPoint(i*2, 0, j*2);
    	NewSphere.Slices := 32;
	    NewSphere.Stacks := 32;
  	end;

   // Actors are used as standalone, med-polycount objects
   // that aren't T&L friendly (all geometry must be sent to
   // the hardware at each frame)
   FReference.LoadFromFile('.\DATA\waste.md2');
   for i := -3 to 3 do
   	for j := -3 to 3 do
    begin
    	NewActor := FActors.AddNewChild(TGLActor) as TGLActor;
      NewActor.Assign(FReference);
      NewActor.Position.SetPoint(i*4, 0, j*4);
      NewActor.CurrentFrame := (i+1) + (j+1) * 5;
   	end;
   FReference.Visible := False;

end;

procedure TfrmFunc00040.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  inherited;
  for I := 0 to Length(FPanels)-1 do FPanels[I] := TPanel.Create(Self);
  for I := 0 to Length(FLabels)-1 do FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FRadioBtnsA)-1 do FRadioBtnsA[I] := TRadioButton.Create(Self);
  for I := 0 to Length(FRadioBtnsB)-1 do FRadioBtnsB[I] := TRadioButton.Create(Self);
  FCheckBox := TCheckBox.Create(Self);

  with FPanels[0] do
  begin
  	SetBounds(0,1,202,90);
    Parent := FViewer;
  end;

  for I := 0 to Length(FLabels)-1 do
    with FLabels[I] do
    begin
      Font.Style := Font.Style + [fsBold];
      case I of
        0: Caption := 'Culling Mode:';
        1: Caption := 'Objects:';
      end;
      SetBounds(I*100+1,1,100,20);
      Parent := FPanels[0];
    end;

	for I := 0 to Length(FRadioBtnsA)-1 do
  	with FRadioBtnsA[I] do
    begin
    	Tag := I;
    	Checked := I = 0;
      OnClick := DO_RadioBtnAClick;
      case I of
      	0: Caption := 'None';
        1: Caption := 'Object based';
        2: Caption := 'Hierarchical';
      end;
      SetBounds(5,21+I*20,95,20);
      Parent := FPanels[0];
    end;

  with FPanels[1] do
  begin
  	BevelInner := bvNone;
    BevelOuter := bvNone;
  	SetBounds(101,21,100,68);
    Parent := FPanels[0];
  end;

	for I := 0 to Length(FRadioBtnsB)-1 do
  	with FRadioBtnsB[I] do
    begin
    	Tag := I;
    	Checked := I = 0;
      OnClick := DO_RadioBtnBClick;
      case I of
      	0: Caption := 'Spheres';
        1: Caption := 'Actors';
      end;
      SetBounds(5,1+I*20,95,20);
      Parent := FPanels[1];
    end;

	with FCheckBox do
  begin
  	Caption := 'Wireframe';
    OnClick := DO_CheckBoxClick;
    SetBounds(5,45,95,20);
    Parent := FPanels[1];
  end;
end;

procedure TfrmFunc00040.DO_RadioBtnAClick(Sender: TObject);
begin
	case TComponent(Sender).Tag of
    0: FScene.VisibilityCulling := vcNone;
  	1: FScene.VisibilityCulling := vcObjectBased;
    2: FScene.VisibilityCulling := vcHierarchical;
  end;
end;

procedure TfrmFunc00040.DO_RadioBtnBClick(Sender: TObject);
begin
  FSpheres.Visible := TComponent(Sender).Tag = 0;
  FActors.Visible := TComponent(Sender).Tag = 1;
end;

procedure TfrmFunc00040.DO_CheckBoxClick(Sender: TObject);
var
	I: Integer;
	Mode: TPolygonMode;
begin
	case TCheckBox(Sender).Checked of
  	True: Mode := pmLines;
    else 	Mode := pmFill;
  end;
  for I := 0 to FActors.Count-1 do
  	if FActors.Children[I] is TGLActor then
    	with TGLActor(FActors.Children[I]) do
      begin
  			Material.FrontProperties.PolygonMode := Mode;
 				Material.BackProperties.PolygonMode := Mode;
      end;
end;

end.
