unit fFunc00041;

interface

uses Forms, Controls, Graphics, StdCtrls, ExtCtrls, ComCtrls, fFunc, GLScene,
	GLTexture, GLVectorFileObjects;

type
	TfrmFunc00041 = class(TfrmFunc)
  protected
    FMaterialLibrary: TGLMaterialLibrary;
    FActor: TGLActor;
    FPanel: TPanel;
    FCheckBoxes: array[0..1] of TCheckBox;
    FTrackBar: TTrackBar;
    FButtons: array[0..1] of TButton;
    FRadioButtons: array[0..1] of TRadioButton;
    FLabel: TLabel;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
		procedure DO_ButtonClick(Sender: TObject);
    procedure DO_RadioButtonClick(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
    procedure DO_SubDivide;
   	procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses SysUtils, VectorLists, VectorGeometry, GLMisc, GLCrossPlatform, MeshUtils;

{ TfrmFunc00041 }

procedure TfrmFunc00041.CREATE_OBJECT;
begin
  inherited;
  FActor := TGLActor.Create(Self);
  FMaterialLibrary := TGLMaterialLibrary.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FActor);

  with FViewer do
	begin
  	Camera := FCamera;
  end;

  with FScene do
	begin
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 90;

  	TargetObject := FDummy;
  	Position.AsVector := VectorMake(5,0,5);
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(10,10,10);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FActor do
  begin
    AnimationMode := aamLoop;
    AutoCentering := [macCenterX,macCenterY,macCenterZ];
    FrameInterpolation := afpLinear;
    Interval := 100;
    UseMeshMaterials := True;
    Scale.AsVector := VectorMake(0.1,0.1,0.1);
    Direction.AsVector := VectorMake(0,1,0);
  end;
  
end;

procedure TfrmFunc00041.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  FPanel := TPanel.Create(Self);
  FTrackBar := TTrackBar.Create(Self);
  FLabel := TLabel.Create(Self);
  for I := 0 to Length(FCheckBoxes)-1 do FCheckBoxes[I] := TCheckBox.Create(Self);
  for I := 0 to Length(FButtons)-1 do FButtons[I] := TButton.Create(Self);
  for I := 0 to Length(FRadioButtons)-1 do FRadioButtons[I] := TRadioButton.Create(Self);

	with FPanel do
  begin
  	SetBounds(0,1,200,100);
    Parent := FViewer;
  end;

	for I := 0 to Length(FButtons)-1 do
  	with FButtons[I] do
    begin
    	Tag := I;
      Enabled := I=0;
      OnClick := DO_ButtonClick;
      case I of
      	0: Caption := 'Load';
        1: Caption := 'Subdivide';
      end;
      SetBounds(I*100+5,5,80,25);
      Parent := FPanel;
    end;

 	for I := 0 to Length(FRadioButtons)-1 do
  	with FRadioButtons[I] do
    begin
    	Tag := I;
      Checked := I=1;
      OnClick := DO_RadioButtonClick;
      case I of
      	0: Caption := 'Wireframe';
        1: Caption := 'Solid';
      end;
      SetBounds(5,30+I*20,80,20);
      Parent := FPanel;
    end;


  for I := 0 to Length(FCheckBoxes)-1 do
		with FCheckBoxes[I] do
  	begin
    	Tag := I;
	  	Checked := True;
    	OnClick := DO_CheckBoxClick;
      case I of
      	0: Caption := 'Animate';
        1: Caption := 'DummyCube';
      end;
	    SetBounds(5+I*75,75,80,20);
  	  Parent := FPanel;
  	end;

  with FTrackBar do
  begin
  	Frequency := 1;
    Max := 10;
    Min := 1;
    Position := 5;
    SliderVisible := True;
    TickMarks := tmBottomRight;
    ThumbLength := 10;
    SetBounds(100,35,95,17);
    Parent := FPanel;
  end;

  with FLabel do
  begin
  	AutoSize := False;
  	Caption := 'LASubdivideTime';
    SetBounds(105,55,85,20);
    Parent := FPanel;
  end;

end;

procedure TfrmFunc00041.DO_ButtonClick(Sender: TObject);
begin
	case TButton(Sender).Tag of
  	0:begin
   			FButtons[1].Enabled := True;
   			FMaterialLibrary.TexturePaths:='.\DATA';
   			FActor.LoadFromFile('.\DATA\waste.md2');
   			FActor.Material.Texture.Image.LoadFromFile('.\DATA\waste.jpg');
   			FActor.Material.Texture.Disabled := False;
   			FActor.SwitchToAnimation(FActor.Animations[0]);
    	end;
    1: DO_SubDivide;
  end;
end;
procedure TfrmFunc00041.DO_RadioButtonClick(Sender: TObject);
var
	Mode: TPolygonMode;
begin
	case TRadioButton(Sender).Tag of
  	0: 		Mode := pmLines;
    else 	Mode := pmFill;
  end;
  FActor.Material.FrontProperties.PolygonMode := Mode;
 	FActor.Material.BackProperties.PolygonMode := Mode;
	case TRadioButton(Sender).Tag of
  	0: 		FViewer.Buffer.BackgroundColor := clSilver;
    else 	FViewer.Buffer.BackgroundColor := clBlack;
  end;
end;

procedure TfrmFunc00041.DO_CheckBoxClick(Sender: TObject);
begin
	case TCheckBox(Sender).Tag of
  	0:begin
        // not only turns on/off animation, but also forces the TGLActor
        // to generate a display list when animation is off
        with FActor do
          if TCheckBox(Sender).Checked then
          begin
            AnimationMode := aamLoop;
            ObjectStyle := ObjectStyle + [osDirectDraw];
            Reference := aarMorph;
          end else
          begin
            AnimationMode := aamNone;
            MeshObjects.MorphTo(0);
            Reference := aarNone;
            StructureChanged;
            ObjectStyle := ObjectStyle - [osDirectDraw];
          end;
    	end;
    1:begin
    		FDummy.ShowAxes := FCheckBoxes[1].Checked;
        FDummy.VisibleAtRunTime := FCheckBoxes[1].Checked;
    	end;
  end;
end;

procedure TfrmFunc00041.DO_SubDivide;
var
  I, J: Integer;
  tris, norms, tex, buf: TAffineVectorList;
  morphTris, morphNorms: TAffineVectorList;	//Unit: VectorLists
  indices, texIndices: TIntegerlist;
  firstRemap, subdivideRemap, bufRemap: TIntegerList;
  t: Int64;
begin
  FButtons[1].Enabled:=False;
  Screen.Cursor := crHourGlass;
  t := StartPrecisionTimer;//Unit: GLCrossPlatform
  for i:=0 to FActor.MeshObjects.Count-1 do
  begin
    tex := TAffineVectorList.Create;
    with FActor.MeshObjects[I] do tris:=ExtractTriangles(tex);
    indices := BuildVectorCountOptimizedIndices(tris);//Unit: MeshUtils
    firstRemap := TIntegerList(indices.CreateClone);
    RemapAndCleanupReferences(tris, indices);
    norms := BuildNormals(tris, indices);
    // subdivide geometry
    SubdivideTriangles(FTrackBar.Position*0.1, tris, indices, norms);
    texIndices := BuildVectorCountOptimizedIndices(tex);
    RemapAndCleanupReferences(tex, texIndices);
    // subdivide texture space
    SubdivideTriangles(0, tex, texIndices);
    // Re-expand everything
    buf := TAffineVectorList.Create;
    try
       ConvertIndexedListToList(tris, indices, buf);
       tris.Assign(buf);
       buf.Count := 0;
       ConvertIndexedListToList(norms, indices, buf);
       norms.Assign(buf);
       buf.Count := 0;
       ConvertIndexedListToList(tex, texIndices, buf);
       tex.Assign(buf);
    finally
       buf.Free;
    end;

    // Pack & Optimize the expanded stuff
    indices.Free;
    indices := BuildVectorCountOptimizedIndices(tris, norms, tex);
    subdivideRemap := TIntegerList(indices.CreateClone);
    RemapReferences(norms, indices);
    RemapReferences(tex, indices);
    RemapAndCleanupReferences(tris, indices);

    IncreaseCoherency(indices, 13);

    with FActor.MeshObjects[i] as TMorphableMeshObject do
    begin
    	bufRemap := TIntegerList.Create;
      for J := 0 to MorphTargets.Count-1 do
      begin
      	MorphTo(J);
        morphTris := ExtractTriangles;
        bufRemap.Assign(firstRemap);
        RemapAndCleanupReferences(morphTris, bufRemap);
        morphNorms := MeshUtils.BuildNormals(morphTris, bufRemap);
        SubdivideTriangles(FTrackBar.Position*0.1, morphTris, bufRemap, morphNorms);
        buf := TAffineVectorList.Create;
        try
        	ConvertIndexedListToList(morphTris, bufRemap, buf);
          morphTris.Assign(buf);
          ConvertIndexedListToList(morphNorms, bufRemap, buf);
          morphNorms.Assign(buf);
        finally
        	buf.Free;
        end;
        RemapReferences(morphTris, subdivideRemap);
        RemapReferences(morphNorms, subdivideRemap);
        MorphTargets[j].Vertices := morphTris;
        MorphTargets[j].Normals := morphNorms;
        morphTris.Free;
        morphNorms.Free;
      end;
      bufRemap.Free;
      Vertices := tris;
      Normals := norms;
      TexCoords := tex;
      FaceGroups.Clear;
      with TFGVertexIndexList.CreateOwned(FaceGroups) do
      begin
        VertexIndices := indices;
        Mode := fgmmTriangles;
      end;
    end;

		texIndices.Free;
    subdivideRemap.Free;
    firstRemap.Free;
    tex.Free;
    indices.Free;
    norms.Free;
    tris.Free;
  end;

  //   (FActor.MeshObjects[0] as TSkeletonMeshObject).PrepareBoneMatrixInvertedMeshes;

  FLabel.Caption := Format('%.1f ms', [StopPrecisionTimer(t)*1000]);
  // Initial perf: 1412 ms
  // Basic Edges Hash: 464 ms
  // Several transfer optims: 377 ms
  // morph & subdivide normals too : 527 ms
  Screen.Cursor := crDefault;

  FActor.StructureChanged;
end;

procedure TfrmFunc00041.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
	if Assigned(FActor) then
  	with FActor do
    	if FrameCount > 1 then
		  	CurrentFrame := Round(CurrentFrame + FrameCount + 1) mod FrameCount;
  inherited;
end;

end.
