unit fFunc00042;

interface

uses Windows, Classes, Forms, Controls, Graphics, StdCtrls, ExtCtrls, ComCtrls, fFunc, GLScene, GLObjects,
	GLShadowPlane, GLTexture, VerletClasses, VerletHairClasses, ODEImport, Keyboard;

const
  cMaxWindMag = 8;
  
type
	TfrmFunc00042 = class(TfrmFunc)
  protected
    FShadowCaster: TGLDummyCube;
    FLines: TGLLines;
    FBall: TGLSphere;
    FSphere: TGLSphere;
    FFloors: array[0..1] of TGLShadowPlane;
    FWalls: array[0..2] of TGLShadowPlane;
    FPanel: TPanel;
    FWindForce: TTrackBar;
    FCheckBoxes: array[0..5] of TCheckBox;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
    procedure DO_MouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean); override;
    procedure DO_Close(Sender: TObject; var Action: TCloseAction); override;
    procedure DO_CheckBoxClick(Sender: TObject);
    procedure DO_TrackbarChange(Sender: TObject);
    procedure INITIAL_VARIABLES;
  public
		space: PdxSpace;
  	PhysicsTime: Single;
  	HairList: TList;
	  Gravity: TVFGravity;
    VerletWorld: TVerletWorld;
    AirResistance: TVFAirResistance;
    odeFurBallBody: PdxBody;
    odeFurBallGeom: PdxGeom;
    VCSphere: TVCSphere;
    procedure CreateBall;
    procedure CreateFur;
	end;

var
  world: PdxWorld;
  contactgroup: TdJointGroupID;

implementation

uses SysUtils, VectorGeometry, GLMisc, GLContext, ODEGL, GLVerletClasses;

const
  cCOL_MAX = 1;
  cOffset = 0.03;

  cRadiusMultiplier = 5;
  cSegmentCount = 4;
  cHairCount = 200;
  cRootDepth = 4;
  
var
  FoldMouseX : integer;
  FoldMouseY : integer;

procedure nearCallback(data : pointer; o1, o2 : PdxGeom); cdecl;
var
  i, numc : integer;
  b1,b2 : PdxBody;
  contact : array[0..cCOL_MAX-1] of TdContact;
  c : TdJointID;
begin
  // exit without doing anything if the two bodies are connected by a joint
  b1 := dGeomGetBody(o1);
  b2 := dGeomGetBody(o2);
  if (Assigned(b1) and Assigned(b2) and (dAreConnected (b1,b2)<>0)) then
    exit;


  for i :=0 to cCOL_MAX-1 do
  begin
    contact[i].surface.mode := dContactBounce;

    // This determines friction, play around with it!
    contact[i].surface.mu := 3;//10e9; //dInfinity; SHOULD BE INFINITY!
    contact[i].surface.mu2 := 0;
    contact[i].surface.bounce := 0.5;//0.5;
    contact[i].surface.bounce_vel := 0.1;
  end;


  numc := dCollide (o1,o2,cCOL_MAX,contact[0].geom,sizeof(TdContact));
  if (numc>0) then
  begin
    for i := 0 to numc-1 do
    begin
      c := dJointCreateContact (world,contactgroup, @contact[i]);
      dJointAttach (c,b1,b2);
    end;
  end;
end;

{ TfrmFunc00042 }

procedure TfrmFunc00042.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FShadowCaster := TGLDummyCube.Create(Self);
  FLines := TGLLines.Create(Self);
  FBall := TGLSphere.Create(Self);
  FSphere := TGLSphere.Create(Self);
  for I := 0 to Length(FFloors)-1 do FFloors[I] := TGLShadowPlane.Create(Self);
  for I := 0 to Length(FWalls)-1 do FWalls[I] := TGLShadowPlane.Create(Self);

	FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FLight);
  FScene.Objects.AddChild(FShadowCaster);
      FShadowCaster.AddChild(FBall);
  FScene.Objects.AddChild(FLines);
  for I := 0 to Length(FFloors)-1 do FScene.Objects.AddChild(FFloors[I]);
  for I := 0 to Length(FWalls)-1 do FScene.Objects.AddChild(FWalls[I]);

	with FViewer do
	begin
  	Camera := FCamera;
    Buffer.AmbientColor.Color := clrGray20;
    Buffer.AntiAliasing := aa4xHQ;
    Buffer.BackgroundColor := clSilver;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 70;
    TargetObject := FBall;//FDummy;
  	Direction.AsVector := VectorMake(0,1,0);
  	Position.AsVector := VectorMake(8,3,3);
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(0,0,5);
  end;

  with FBall do
	begin
  	Radius := 0.25;
    Slices := 16;
    Stacks := 16;
    Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,-1.5,3);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLines do
	begin
  	Division := 10;
    LineColor.Color := clrWhite;
    NodeColor.Color := clrBlue;
    NodesAspect := lnaAxes;
    Options := [];
    SplineMode := lsmLines;
    Direction.AsVector := VectorMake(0,0,1);
  	Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  for I := 0 to Length(FFloors)-1 do
  	with FFloors[I] do
		begin
    	Width := 8;
      ShadowColor.Color := VectorMake(0,0,0,0.5);
      ShadowingObject := FShadowCaster;
      with Material do
      begin
      	BackProperties.Ambient.Color := clrGray20;
        BackProperties.Diffuse.Color := VectorMake(0.5,0.5,0,1);
      	FrontProperties.Ambient.Color := clrGray20;
        FrontProperties.Diffuse.Color := VectorMake(0.5,0.5,0,1);
      end;
    	case I of
      	0:begin
    				Height := 8;
            Direction.AsVector := VectorMake(0,0.1,1);
            Position.AsVector := VectorMake(0,0,0);
            Up.AsVector := VectorMake(0,1,-0.1);
        	end;
        1:begin
    				Height := 10;
            Direction.AsVector := VectorMake(0,-0.1,1);
            Position.AsVector := VectorMake(0,7.5,0);
            Up.AsVector := VectorMake(0,1,0.1);
        	end;
      end;
  	end;

  for I := 0 to Length(FWalls)-1 do
  	with FWalls[I] do
		begin
			Height := 4;
      ShadowColor.Color := VectorMake(0,0,0,0.5);
      ShadowingObject := FShadowCaster;
      Up.AsVector := VectorMake(0,0,-1);
      with Material do
      begin
      	BackProperties.Ambient.Color := clrGray20;
        BackProperties.Diffuse.Color := clrLime;
      	FrontProperties.Ambient.Color := clrGray20;
        FrontProperties.Diffuse.Color := clrYellow;
      end;

      case I of
      	0: Width := 16;
        else Width := 8;
      end;

      case I of
      	0: Direction.AsVector := VectorMake(1,0,0);
        1: Direction.AsVector := VectorMake(0,1,0);
        2: Direction.AsVector := VectorMake(0,-1,0);
      end;

    	case I of
      	0: Position.AsVector := VectorMake(-4,4,1);
        1: Position.AsVector := VectorMake(0,-4,1);
        2: Position.AsVector := VectorMake(0,12,1);
      end;
  	end;
  INITIAL_VARIABLES;
end;

procedure TfrmFunc00042.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  FPanel := TPanel.Create(Self);
  FWindForce := TTrackBar.Create(Self);
  for I := 0 to Length(FCheckBoxes)-1 do
  	FCheckBoxes[I] := TCheckBox.Create(Self);

  with FPanel do
  begin
  	SetBounds(0,1,120,150);
    Parent := FViewer;
  end;

  for I := 0 to Length(FCheckBoxes)-1 do
  	with FCheckBoxes[I] do
    begin
    	Tag := I;
      Checked := I in [1,4,5];
      OnClick := DO_CheckBoxClick;
      case I of
      	0: Caption := '&Lock Ball';
        1: Caption := 'Fur &Gravity';
      	2: Caption := '&Bald';
        3: Caption := '&Shadows';
        4: Caption := 'Fur &Inertia';
        5: Caption := '&Wind Resistence';
			end;
      SetBounds(3,I*20+1,110,20);
      Parent := FPanel;
    end;

	with FWindForce do
  begin
  	Frequency := 10;
  	Max := 100;
    Min := 0;
    Position := 50;
    ThumbLength := 10;
    TickMarks := tmBottomRight;
    TickStyle := tsAuto;
    OnChange := DO_TrackbarChange;
    SetBounds(0,120,118,17);
    Parent := FPanel;
  end;
end;

procedure TfrmFunc00042.DO_CheckBoxClick(Sender: TObject);
var
  I: Integer;
  Light: TGLLightSource;
begin
	if Sender = nil then EXIT;
	with TCheckBox(Sender) do
    case Tag of
      0:;
      1:if not Checked then FreeAndNil(Gravity)
        else begin
          Gravity := TVFGravity.Create(VerletWorld);
          Gravity.Gravity := AffineVectorMake(0,0,-9.81);
        end;
      2:begin
          for I := 0 to HairList.Count -1 do
          begin
            with TVerletHair(HairList[i]) do
            begin
              Anchor.NailedDown := not Checked;
              Anchor.OldLocation := Anchor.Location;
              Root.NailedDown := not Checked;
              Root.OldLocation := Root.Location;
            end;
          end;
          if not Checked then VerletWorld.PauseInertia(5);
        end;
      3:begin
          if Checked then Light := FLight else Light := nil;
          for I := 0 to Length(FFloors)-1 do FFloors[I].ShadowedLight := Light;
          for I := 0 to Length(FWalls)-1 do FWalls[I].ShadowedLight := Light;
        end;
      4:VerletWorld.Inertia := Checked;
      5:begin
          if not Checked then FreeAndNil(AirResistance)
          else begin
            AirResistance := TVFAirResistance.Create(VerletWorld);
            AirResistance.DragCoeff := 0.01;
            AirResistance.WindDirection := AffineVectorMake(1,0,0);
            AirResistance.WindMagnitude := FWindForce.Position/100 * cMaxWindMag;
            AirResistance.WindChaos := 0.4;
          end;
          FWindForce.Enabled := Checked;
        end;
    end;
end;


procedure TfrmFunc00042.DO_TrackbarChange(Sender: TObject);
begin
  if Assigned(AirResistance) then
    AirResistance.WindMagnitude := FWindForce.Position/100 * cMaxWindMag;
end;

var
  angle: double = 0;
  
procedure TfrmFunc00042.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
const
	cTIME_STEP = 0.01;
var
  I, J: Integer;
  Delta: Single;
  Hair: TVerletHair;
  GLLines: TGLLines;
begin
  Delta := deltaTime;
  angle := angle + Delta*3;

  while PhysicsTime<newTime do
  begin
    PhysicsTime := PhysicsTime + cTIME_STEP;

    if not FCheckBoxes[0].Checked then
    begin
      dSpaceCollide(space,nil,nearCallback);
      dWorldStep (world, cTIME_STEP);//}
      // remove all contact joints
      dJointGroupEmpty (contactgroup);

      if IsKeyDown(VK_UP) then
        dBodyAddForce(odeFurBallBody, 0,0,2.5)

      else if IsKeyDown(VK_DOWN) then
        dBodyAddForce(odeFurBallBody, 0,0,-2.5);

      if IsKeyDown('A') then
        dBodyAddForce(odeFurBallBody, 0,-1,0)

      else if IsKeyDown('D') then
        dBodyAddForce(odeFurBallBody, 0,1,0);

      if IsKeyDown('W') then
        dBodyAddForce(odeFurBallBody, -1,0,0)

      else if IsKeyDown('S') then
        dBodyAddForce(odeFurBallBody, 1,0,0);
    end;

    PositionSceneObject(FBall, odeFurBallGeom);
    VCSphere.Location := FBall.Position.AsAffineVector;
    VerletWorld.Progress(cTIME_STEP, PhysicsTime);
  end;

  for i := 0 to HairList.Count -1 do
  begin
    Hair := TVerletHair(HairList[i]);
    GLLines := TGLLines(Hair.Data);
    for j := 1 to Hair.NodeList.Count-1 do
      GLLines.Nodes[j-1].AsAffineVector := Hair.NodeList[j].Location;
  end;
  inherited;
end;

procedure TfrmFunc00042.DO_ViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  FoldMouseX := X;
  FoldMouseY := Y;
  inherited;
end;

procedure TfrmFunc00042.DO_MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
	FCamera.AdjustDistanceToTarget(Power(1.1, WheelDelta/120));
end;

procedure TfrmFunc00042.DO_Close(Sender: TObject; var Action: TCloseAction);
begin
	FCadencer.OnProgress := nil;
	FCadencer.Enabled := False;
  dJointGroupDestroy (contactgroup);
  dSpaceDestroy (space);
  dWorldDestroy (world);
  inherited;
end;

procedure TfrmFunc00042.INITIAL_VARIABLES;
var
	I: Integer;
begin
  world := dWorldCreate();
  space := dHashSpaceCreate(nil);
  contactgroup := dJointGroupCreate (1000000);
  dWorldSetGravity (world,0,0,-9.81);
  for I := 0 to Length(FFloors)-1 do CreateODEPlaneFromGLPlane(FFloors[I], space);
  for I := 0 to Length(FWalls)-1 do CreateODEPlaneFromGLPlane(FWalls[I], space);
  // dCreatePlane (space,0,0,1,0);

  VerletWorld := TVerletWorld.Create;
  VerletWorld.Iterations := 2;
  VerletWorld.VerletNodeClass := TGLVerletNode;

  DO_CheckBoxClick(FCheckBoxes[1]);
  DO_CheckBoxClick(FCheckBoxes[5]);

  for I := 0 to Length(FFloors)-1 do CreateVCPlaneFromGLPlane(FFloors[I], VerletWorld, cOffset);
  for I := 0 to Length(FWalls)-1 do CreateVCPlaneFromGLPlane(FWalls[I], VerletWorld, cOffset);

  HairList := TList.Create;

  CreateBall;

end;

procedure TfrmFunc00042.CreateBall;
var
  m : TdMass;
begin
  dMassSetSphere (m,1,FBall.Radius);

  odeFurBallGeom := dCreateSphere (space,FBall.Radius);
  odeFurBallBody := dBodyCreate(World);

  dGeomSetBody (odeFurBallGeom,odeFurBallBody);
  dBodySetMass (odeFurBallBody, @m);
  dBodySetLinearVel(odeFurBallBody, 0, 14, 0);

  dBodyAddTorque(odeFurBallBody, 0.1,0.1,0.1);

  // Add the GLScene object
  odeFurBallGeom.Data:=FBall;

  CopyPosFromGeomToGL(odeFurBallGeom, FBall);

  VCSphere := TVCSphere.Create(VerletWorld);
  VCSphere.Radius := FBall.Radius * 1.1;
  VCSphere.Location := AffineVectorMake(FBall.AbsolutePosition);

  CreateFur;
end;

procedure TfrmFunc00042.CreateFur;
  // Much, MUCH easier that uniform distribution, and it looks fun.
  procedure CreateRandomHair;
  var
    i : integer;
    Dir : TAffineVector;
    Hair : TVerletHair;
    GLLines : TGLLines;
  begin
    Dir := AffineVectorMake(random-0.5,random-0.5,random-0.5);
    NormalizeVector(Dir);

    Hair := TVerletHair.Create(VerletWorld, FBall.Radius * cRootDepth,
      FBall.Radius*cRadiusMultiplier, cSegmentCount,
      VectorAdd(AffineVectorMake(FBall.AbsolutePosition), VectorScale(Dir, FBall.Radius)),
      Dir, [vhsSkip1Node]);

    //GLLines := TGLLines(GLScene1.Objects.AddNewChild(TGLLines));
    GLLines := TGLLines(FShadowCaster.AddNewChild(TGLLines));
    GLLines.NodesAspect := lnaInvisible;
    GLLines.LineWidth := 2;
    GLLines.LineColor.Color := clrBlack;

    for i := 0 to Hair.NodeList.Count-1 do
      TGLVerletNode(Hair.NodeList[i]).GLBaseSceneObject := FBall;

    for i := 1 to Hair.NodeList.Count-1 do
       GLLines.AddNode(Hair.NodeList[i].Location);

    for i := 0 to GLLines.Nodes.Count-1 do
      TGLLinesNode(GLLines.Nodes[i]).Color.Color := clrBlack;

    GLLines.ObjectStyle:=GLLines.ObjectStyle+[osDirectDraw];
    GLLines.SplineMode := lsmCubicSpline;

    Hair.Data := GLLines;
    HairList.Add(Hair);
  end;
var
  Hair : TVerletHair;
  i : integer;
begin
  for i := 0 to HairList.Count-1 do
  begin
    Hair := TVerletHair(HairList[i]);
    TGLLines(Hair.Data).Free;
    Hair.Free;
  end;

  HairList.Clear;

  for i := 0 to cHairCount-1 do
    CreateRandomHair;
end;

end.
