unit fFunc00043;

interface

uses Graphics, StdCtrls, ExtCtrls, Dialogs, fFunc, GLScene, GLObjects,
	OpenGL1x, GLWin32Viewer;

type
	TfrmFunc00043 = class(TfrmFunc)
  protected
    FCube: TGLCube;
    FMemoryViewer: TGLMemoryViewer;
    FPanel: TPanel;
    FCheckBox: TCheckBox;
    FLabels: array[0..1] of TLabel;
    FRadioButtons: array[0..2] of TRadioButton;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
		procedure DO_ViewerAfterRender(Sender: TObject); override;
    procedure DO_RadioButtonClick(Sender: TObject);
    procedure DO_CheckBoxClick(Sender: TObject);
  public
		n: Integer;
    textureFramerateRatio: Integer;
	end;

implementation

uses VectorGeometry, GLTexture;

{ TfrmFunc00043 }

procedure TfrmFunc00043.CREATE_OBJECT;
begin
  inherited;
  n := 0;
  textureFramerateRatio := 1;
  
  FMemoryViewer := TGLMemoryViewer.Create(Self);
  FCube := TGLCube.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  FDummy.AddChild(FCube);

  with FMemoryViewer do
	begin
  	Camera := FCamera;
    Height := 256;
    Width := 256;
    Buffer.BackgroundColor := clRed;
    Buffer.Lighting := True;
  end;

  with FViewer do
	begin
    Camera := FCamera;
  	Buffer.BackgroundColor := clGray;
  end;

  with FCamera do
	begin
  	DepthOfView := 100;
    FocalLength := 150;
    TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(5,4,5);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Position.AsVector := VectorMake(10,10,10);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FCube do
  begin
  	Material.Texture.Disabled := False;
//    Material.Texture.ImageClassName := 'Blank Image';
  end;

end;

procedure TfrmFunc00043.DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FDummy, 15, newTime, 2);
  inherited;
end;

procedure TfrmFunc00043.DO_ViewerAfterRender(Sender: TObject);
begin

  if not WGL_ARB_pbuffer then
  begin
    ShowMessage( 'WGL_ARB_pbuffer not supported...'#13#10#13#10
                +'Get newer graphics hardware or try updating your drivers!');
    FViewer.AfterRender := nil;
    Exit;
  end;
  Inc(n);

  try
    if n>=textureFramerateRatio then
    begin
      // render to the viewer
      FMemoryViewer.Render;
      // copy result to the textures
      FMemoryViewer.CopyToTexture(FCube.Material.Texture);

      n := 0;
    end;
	except
  	// pbuffer not supported... catchall for exotic ICDs...
    FViewer.AfterRender := nil;
    raise;
  end;

end;

procedure TfrmFunc00043.SETUP_USER_INTERFACE;
var
	I: Integer;
begin
  FPanel := TPanel.Create(Self);
  FCheckBox := TCheckBox.Create(Self);
  for I := 0 to Length(FLabels)-1 do FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FRadioButtons)-1 do FRadioButtons[I] := TRadioButton.Create(Self);

  with FPanel do
  begin
  	SetBounds(0,1,100,125);
    Parent := FViewer;
  end;

  for I := 0 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
    	case I of
      	0:begin
        		Caption := 'Options';
      			Font.Height := 18;
      			Font.Style := Font.Style + [fsBold];
        	end;
      	1:Caption := 'Texture Framerate';
      end;
      SetBounds(1,I*20,80,20);
      Parent := FPanel;
    end;

  for I := 0 to Length(FRadioButtons)-1 do
  	with FRadioButtons[I] do
    begin
    	Tag := I;
      Checked := I=0;
      OnClick := DO_RadioButtonClick;
      case I of
      	0: Caption := '1:1';
        1: Caption := '1:2';
        2: Caption := '1:10';
      end;
      SetBounds(1,I*20+40,60,20);
			Parent := FPanel;
    end;

	with FCheckBox do
  begin
  	Caption := 'VSync';
    OnClick := DO_CheckBoxClick;
    SetBounds(1,100,60,20);
    Parent := FPanel;
  end;

end;

procedure TfrmFunc00043.DO_RadioButtonClick(Sender: TObject);
begin
	textureFramerateRatio := (Sender as TRadioButton).Tag;
	case TRadioButton(Sender).Tag of
  	0: textureFramerateRatio := 1;
    1: textureFramerateRatio := 2;
    2: textureFramerateRatio := 10;
  end;
end;

procedure TfrmFunc00043.DO_CheckBoxClick(Sender: TObject);
begin
	if FCheckBox.Checked then FViewer.VSync := vsmSync else FViewer.VSync := vsmNoSync;
end;

end.
