unit fFunc00044;

interface

uses fFunc, GLObjects, GLGeomObjects, GLTexture;

type
	TfrmFunc00044 = class(TfrmFunc)
  protected
    FDCSphere: TGLDummyCube;
    FDCArrow: TGLDummyCube;
    FSphere: TGLSphere;
    FArrowLine: TGLArrowLine;
    FPlane: TGLPlane;
    FDisks: array[0..1] of TGLDisk;
    FLines: array[0..1] of TGLLines;
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00044 }

procedure TfrmFunc00044.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FDCSphere :=  TGLDummyCube.Create(Self);
  FDCArrow :=  TGLDummyCube.Create(Self);
  FSphere :=  TGLSphere.Create(Self);
  FArrowLine :=  TGLArrowLine.Create(Self);
  FPlane :=  TGLPlane.Create(Self);
  for I := 0 to Length(FDisks)-1 do FDisks[I] := TGLDisk.Create(Self);
  for I := 0 to Length(FLines)-1 do FLines[I] := TGLLines.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
      FDummy.AddChild(FDCArrow);
      		FDCArrow.AddChild(FArrowLine);
          		FArrowLine.AddChild(FLines[0]);
              FArrowLine.AddChild(FPlane);
      FDummy.AddChild(FDCSphere);
          FDCSphere.AddChild(FLines[1]);
      		FDCSphere.AddChild(FSphere);
      FDummy.AddChild(FDisks[0]);
      FDummy.AddChild(FDisks[1]);

  with FCamera do
	begin
  	DepthOfView := 1000;
  	FocalLength := 100;
    TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(5,7,8);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
  	Ambient.Color := clrGray;
    Position.AsVector := VectorMake(20,15,12);
  end;

  with FDummy do
  begin
  	Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FSphere do
	begin
  	Radius := 0.5;
    Slices := 16;
    Stacks := 16;
  	Material.FrontProperties.Diffuse.Color := clrLime;
    Position.AsVector := VectorMake(4,0,0);
  end;

  with FPlane do
	begin
    Height := 1;
    Width := 0.3;
    XOffset := 0;
    XScope := 1;
    XTiles := 1;
    YOffset := 0;
    YScope := 1;
    YTiles := 1;
  	Material.FrontProperties.Diffuse.Color := clrBlue;
    Position.AsVector := VectorMake(0,0,-0.2);
    Up.AsVector := VectorMake(1,0,0);
  end;

  for I := 0 to Length(FDisks)-1 do
  	with FDisks[I] do
    begin
    	case I of
      	0:begin
        		InnerRadius := 0.9;
            OuterRadius := 1;
    				Slices := 16;
            Material.FrontProperties.Diffuse.Color := VectorMake(0,0,0.5,1);
            Material.FrontProperties.Emission.Color := VectorMake(0,0,0.5,1);
        	end;
        1:begin
        		InnerRadius := 3.9;
            OuterRadius := 4;
            Slices := 32;
            Material.FrontProperties.Diffuse.Color := VectorMake(0,0.5,0,1);
            Material.FrontProperties.Emission.Color := VectorMake(0,0.2,0.1);
        	end;
      end;
      Up.AsVector := VectorMake(-0.948,0,0.316);

    end;

  for I := 0 to Length(FLines)-1 do
  	with FLines[I] do
    begin
    	NodeColor.Color := clrBlue;
      NodesAspect := lnaInvisible;
      Options := [];
      SplineMode := lsmLines;
    	Direction.AsVector := VectorMake(0,0,1);
      Position.AsVector := VectorMake(0,0,0);
      Up.AsVector := VectorMake(0,1,0);
    	case I of
      	0:begin
        		LineColor.Color := VectorMake(0.8352941,0.8352941,1,1);
            with TGLLinesNode(Nodes.Add) do
            begin
            	Color.Color := clrBlue;
              X := 0;
              Y := 0;
              Z := -1;
            end;
            with TGLLinesNode(Nodes.Add) do
            begin
            	Color.Color := clrBlue;
              X := 0;
              Y := 0;
              Z := 7;
            end;
        	end;
      	1:begin
        		LineColor.Color := clrLime;
            with TGLLinesNode(Nodes.Add) do
            begin
            	Color.Color := clrBlue;
              X := 4;
              Y := -4;
              Z := 0;
            end;
            with TGLLinesNode(Nodes.Add) do
            begin
            	Color.Color := clrBlue;
              X := 4;
              Y := 4;
              Z := 0;
            end;

        	end;
      end;

    end;

end;

procedure TfrmFunc00044.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
   // Make the blue sphere turn and ride a sin
   FDCSphere.Turn(deltaTime*30);
   FSphere.Position.Y := Sin(DegToRad(newTime*50))*3;

   // Make the arrow turn
   FDCArrow.Turn(-deltaTime*15);

   // Make the arrow point toward the sphere, using Y as up reference
   FArrowLine.PointTo(FSphere, YHmgVector);
  inherited;
end;

end.
