unit fFunc00045;

interface

uses Graphics, fFunc, GLScene, GLObjects, GLExtrusion, GLTexture;

type
	TfrmFunc00045 = class(TfrmFunc)
  protected
    FSphere: TGLSphere;
    FPipes:array[0..4] of TGLPipe;//Unit: GLExtrusion
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
	end;

implementation

uses VectorGeometry;

const
   cNbNodes = 40;

{ TfrmFunc00045 }

procedure TfrmFunc00045.CREATE_OBJECT;
var
	I, K: Integer;
begin
  inherited;
  FSphere := TGLSphere.Create(Self);
  for I := 0 to Length(FPipes)-1 do
	  FPipes[I] := TGLPipe.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FLight);
	FDummy.AddChild(FSphere);
  for I := 0 to Length(FPipes)-1 do
  	FDummy.AddChild(FPipes[I]);

	with FViewer do
  begin
  	Camera := FCamera;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000;
    FocalLength := 70;
    TargetObject := FDummy;
    Position.AsVector := VectorMake(6,5,4);
  end;

  with FDummy do
  begin
  	CubeSize := 4;
  	ShowAxes := True;
    VisibleAtRuntime := True;
  	Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(1,1,0);
  end;

  with FLight do
  begin
		Position.AsVector := VectorMake(50,40,30);
  end;

  with FSphere do
  begin
  	Radius := 1;
    Scale.AsVector := VectorMake(2,0.5,2);
    Slices := 32;
    Stacks := 8;
    Position.AsVector := VectorMake(0,-2,0);
    Material.FrontProperties.Ambient.Color := VectorMake(0,0.2,0,1);
    Material.FrontProperties.Diffuse.Color := clrForestGreen;
    Material.FrontProperties.Emission.Color := VectorMake(0,0.2,0,1);
  end;

  for I := 0 to Length(FPipes)-1 do
  	with FPipes[I] do
    begin
    	Tag := I;
     	Parts := [ppOutside];
      Radius := 1;
      Slices := 16;
    	Position.AsVector := VectorMake(0,-2,0);
    end;


	// prepare the TGLPipe objects (add node, set props...)
  for K :=0 to FDummy.Count-1 do
  	if FDummy[K] is TGLPipe then
      with TGLPipe(FDummy[K]) do
      begin
      	Nodes.Clear;
        for I := 1 to cNbNodes do
        	Nodes.AddNode(0, I/8, 0);
         Radius:=0.1;
         // enable per-node coloring in the TGLPipe
         NodesColorMode := pncmDiffuse;
         // divisions between nodes (for spline interpolation)
         Division := 3;
         // No geometry compilation/cacheing, render directly
         // (geometry changes completely from frame to frame)
         ObjectStyle := ObjectStyle + [osDirectDraw];
      end;
end;

procedure TfrmFunc00045.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
var
  I, K: Integer;
  t, t1, t2, r: Double;
  pipe: TGLPipe;
begin
  t:=newTime;
  for k:=0 to FDummy.Count-1 do if FDummy[k] is TGLPipe then
  begin
    pipe := TGLPipe(FDummy[k]);
    with pipe.Nodes do
    begin
       BeginUpdate;
       for I := 0 to Count-1 do
       	with Items[I] as TGLPipeNode do
        begin
          // don't search any hidden logic behind the formulaes below:
          // they're just here to induce this sickening weirdo movement
          t1 := -t+i*0.1+k*c2PI/5;
          r  := (Sin(3*t+k)+2)*0.5*((2*i+Count)/Count);
          X  := Cos(t1)*r;
          Z  := Sin(t1)*r;
          t2 := 2*(t+i/(Count-1)+k);
          Color.Color := VectorLerp(clrSeaGreen, clrYellow, Sin(t2));
          RadiusFactor := (1+(Sin(t2)*0.5))*ln((Count-i))*0.5;
       end;
       EndUpdate;
    end;
  end;
  FSphere.Radius := 1.1+Sin(2*t)*0.1;
  ROTATE_SCENE_OBJECT(FDummy,30,newTime,5);
  inherited;
end;

end.
