unit fFunc00046;

interface

uses Classes, Controls, StdCtrls, ExtCtrls, fFunc, GLScene, GLObjects, GLGeomObjects,
	GLVectorFileObjects, GLTexture;

type
	TfrmFunc00046 = class(TfrmFunc)
  protected
    FFreeForm: TGLFreeForm;
    FSphere: TGLSphere;
    FArrowLine: TGLArrowLine;
    FPanel: TPanel;
    FLabels: array[0..5] of TLabel;
    FCheckBoxes: array[0..1] of TCheckBox;
    procedure DO_Timer(Sender: TObject); override;
    procedure CREATE_OBJECT; override;
    procedure SETUP_USER_INTERFACE; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
		procedure DO_ViewerMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
		procedure DO_ViewerMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer); override;
  end;

implementation

uses VectorGeometry, SysUtils, GLCrossPlatform;

{ TfrmFunc00046 }

procedure TfrmFunc00046.CREATE_OBJECT;
begin
  inherited;
  FSphere := TGLSphere.Create(Self);
  FFreeForm := TGLFreeForm.Create(Self);
  FArrowLine := TGLArrowLine.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FFreeForm);
  FScene.Objects.AddChild(FSphere);
  		FSphere.AddChild(FArrowLine);

	with FViewer do
	begin
  	Buffer.BackgroundColor := $00804000;
    Camera := FCamera;
  end;

  with FCamera do
	begin
  	TargetObject := FDummy;
//    Position.AsVector := VectorMake(5,-9,20);
    Position.AsVector := VectorMake(20,20,20);
    Up.AsVector := VectorMake(-1,0.2,0);
  end;

  with FLight do
	begin
    LightStyle := lsOmni;
    Specular.Color := clrRed;
  	Ambient.Color := VectorMake(0,0.5,1,1);
    Position.AsVector := VectorMake(50,150,100);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  with FSphere do
	begin
  	Radius := 0.3;
    Slices := 6;
    Stacks := 6;
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(-0.95,0,0.316);
    Material.FrontProperties.Emission.Color := clrYellow;
  end;

  with FFreeForm do
	begin
  	Material.BackProperties.Ambient.Color := clrLime;
  	Scale.AsVector := VectorMake(0.1,0.1,0.1);
    Direction.AsVector := VectorMake(0,1,0);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0,1);
    UseMeshMaterials := True;
  end;

  with FArrowLine do
	begin
  	BottomArrowHeadHeight := 0.5;
    BottomArrowHeadRadius := 0.2;
    BottomRadius := 0.05;
  	TopArrowHeadHeight := 0.5;
    TopArrowHeadRadius := 0.2;
    TopRadius := 0.1;
    Height := 1;
    Loops := 1;
    Parts := [alLine,alTopArrow];
    Slices := 16;
    Stacks := 4;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0.1);
    Up.AsVector := VectorMake(0,1,0);
  end;

  FFreeForm.LoadFromFile('.\DATA\HighPolyObject.3ds');
  
end;

procedure TfrmFunc00046.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
  if FCheckBoxes[0].Checked then
    DO_ViewerMouseDown(Sender, TMouseButton(mbLeft), [], mx, my);
	ROTATE_SCENE_OBJECT(FDummy,15,newTime,7);
  inherited;
end;

procedure TfrmFunc00046.DO_Timer(Sender: TObject);
begin
  with FLabels[0] do Caption := Hint;
  with FLabels[5] do Caption := Hint;
  inherited;
end;

procedure TfrmFunc00046.DO_ViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  rayStart, rayVector, iPoint, iNormal: TVector;
  t: Int64;
begin
	if Shift <> [] then inherited
  else begin
    SetVector(rayStart, FCamera.AbsolutePosition);
    SetVector(rayVector, FViewer.Buffer.ScreenToVector(AffineVectorMake(x, FViewer.Height-y, 0)));
    NormalizeVector(rayVector);

    t:=StartPrecisionTimer;
    if FCheckBoxes[0].Checked then
    begin
      // Octree method (fast)
      if FFreeForm.OctreeRayCastIntersect(raystart, rayvector, @iPoint, @iNormal) then
      begin
         FSphere.Visible:=True;
         FSphere.Position.AsVector:=iPoint;
         FSphere.Direction.AsVector:=VectorNormalize(iNormal);
      end
      else FSphere.Visible:=False;
      FLabels[5].Hint:='# Nodes hit with raycast: '+inttostr(High(FFreeForm.Octree.ResultArray)+1);
    end else begin
      // Brute-Force method (slow)
      if FFreeForm.RayCastIntersect(rayStart, rayVector, @iPoint, @iNormal) then
      begin
         FSphere.Visible:=True;
         FSphere.Position.AsVector:=iPoint;
         FSphere.Direction.AsVector:=VectorNormalize(iNormal);
      end else FSphere.Visible:=False;
    end;
    FLabels[0].Hint:=Format('Intersect Time: %.3f ms', [StopPrecisionTimer(t)*1000]);
  end;
end;

procedure TfrmFunc00046.DO_ViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
	if Shift <> [] then inherited
  else begin
		mx := X;
		my := Y;
  end;
end;

procedure TfrmFunc00046.SETUP_USER_INTERFACE;
var
	I: Integer;
	t: Int64;
begin
  inherited;
  FPanel := TPanel.Create(Self);
  for I := 0 to Length(FLabels)-1 do
  	FLabels[I] := TLabel.Create(Self);
  for I := 0 to Length(FCheckBoxes)-1 do
  	FCheckBoxes[I] := TCheckBox.Create(Self);

	with FPanel do
  begin
  	SetBounds(0,1,152,160);
    Parent := FViewer;
  end;

  for I := 0 to Length(FLabels)-1 do
  	with FLabels[I] do
    begin
    	Tag := I;
    	AutoSize := False;
      Caption := 'Label' + IntToStr(I);
      case I of
      	5: SetBounds(1,140,150,20);
        else SetBounds(1,I*18+1,150,18);
      end;
      Parent := FPanel;
    end;

	for I := 0 to Length(FCheckBoxes)-1 do
  	with FCheckBoxes[I] do
    begin
    	Tag := I;
      Checked := True;
      case I of
      	0: Caption := 'Octree enabled';
        1: Caption := 'Auto collide';
      end;
      SetBounds(1,I*20+100,100,20);
      Parent := FPanel;
    end;

  // Load high poly mesh (10,000 triangles).
  t := StartPrecisionTimer;
  FFreeForm.BuildOctree;
  FLabels[1].Caption := Format('Build time: %.3f ms', [StopPrecisionTimer(t)*1000]);
  with FFreeForm.Octree do
  begin
  	FLabels[2].Caption:='Octree Nodes: '		 + IntToStr(NodeCount);
    FLabels[3].Caption:='Tri Count Octree: ' + IntToStr(TriCountOctree);
    FLabels[4].Caption:='Tri Count Mesh: '	 + IntToStr(TriCountMesh);
  end;
  mx := -1;
  my := -1;

end;

end.

