unit fFunc00047;

interface

uses Graphics, fFunc, GLObjects, GLTimeEventsMgr, GLTexture;

type
	TfrmFunc00047 = class(TfrmFunc)
  protected
    FTimeEventsMGR: TGLTimeEventsMGR;
    FCubes: array[0..2] of TGLCube;
    procedure CREATE_OBJECT; override;
    procedure DO_EVENT(event: TTimeEvent);
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;

	end;

implementation

uses SysUtils, VectorGeometry;

{ TfrmFunc00047 }

procedure TfrmFunc00047.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FTimeEventsMGR := TGLTimeEventsMGR.Create(Self);
  for I := 0 to Length(FCubes)-1 do
  	FCubes[I] := TGLCube.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  for I := 0 to Length(FCubes)-1 do
  	FDummy.AddChild(FCubes[I]);

	with FViewer do
  begin
  	Camera := FCamera;
    Buffer.AmbientColor.Color := clrWhite;
    Buffer.BackgroundColor := clWhite;//cl3DDkShadow;
  end;

  with FCamera do
  begin
  	DepthOfView := 1000000;
    FocalLength := 150;
    TargetObject := FDummy;
    Direction.AsVector := VectorMake(-1,0,0);
    Position.AsVector := VectorMake(15,0,15);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
  begin
  	Position.AsVector := VectorMake(10,10,10);
  end;

  with FDummy do
  begin
  	EdgeColor.Color := clrBlack;
  	ShowAxes := True;
    VisibleAtRuntime := True;
  end;

  for I := 0 to Length(FCubes)-1 do
  	with FCubes[I] do
    begin
    	case I of
	    	0: Scale.AsVector := VectorMake(3,8,2.9);
        1: Scale.AsVector := VectorMake(3,8,3.0);
        2: Scale.AsVector := VectorMake(3,8,3.1);
      end;

      case I of
      	0: Position.AsVector := VectorMake(-6,0,0);
        1: Position.AsVector := VectorMake(+0,0,0);
        2: Position.AsVector := VectorMake(+6,0,0);
      end;
      case I of
      	0: Material.FrontProperties.Diffuse.Color := clrRed;
        1: Material.FrontProperties.Diffuse.Color := clrLime;
        2: Material.FrontProperties.Diffuse.Color := clrBlue;
      end;
      Material.FrontProperties.Shininess := 128;
      Material.BlendingMode := bmTransparency;
//      Material.BlendingMode := bmAdditive;
      Material.FrontProperties.Diffuse.Alpha := 0.7;
    end;

	with FTimeEventsMGR do
  begin
  	Cadencer := FCadencer;
    for I := 0 to 5 do
    	with Events.Add do
      begin
      	Name := 'Event' + IntToStr(I);
      	case I of
        	0:begin StartTime := 02; EndTime := 05; Period := 0000; EventType := etContinuous;end;
        	1:begin StartTime := 05; EndTime := 10; Period := 0.01; EventType := etPeriodic; 	end;
          2:begin StartTime := 10; EndTime := 00; Period := 0000; EventType := etOneShot; 	end;
          3:begin StartTime := 15; EndTime := -1; Period := 0.01; EventType := etPeriodic; 	end;
          4:begin StartTime := 15; EndTime := -1; Period := 0.01; EventType := etPeriodic; 	end;
          5:begin StartTime := 15; EndTime := -1; Period := 0.01; EventType := etPeriodic; 	end;
        end;
        OnEvent := DO_EVENT;
        Enabled := True;
      end;

  end;
end;

procedure TfrmFunc00047.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
	ROTATE_SCENE_OBJECT(FDummy,15,newTime,2);
  inherited;
end;

procedure TfrmFunc00047.DO_EVENT(event: TTimeEvent);
var
	N, I: Integer;
begin
	N := Ord(event.Name[6])-Ord('0');
  case N of
  	0:FCubes[0].RollAngle := event.ElapsedTime * 180 / 3;
    1:FCubes[1].RollAngle := event.TickCount / 499 * 180;
    2:FCubes[2].RollAngle := 90;
		3:for I := 0 to Length(FCubes)-1 do FCubes[I].PitchAngle := Integer(event.TickCount) * (I+1) * 2;
    4:for I := 0 to Length(FCubes)-1 do FCubes[I].TurnAngle := Integer(event.TickCount)  * (I+1) * 2;
    5:for I := 0 to Length(FCubes)-1 do FCubes[I].RollAngle := Integer(event.TickCount)  * (I+1) * 2;
  end;
end;

end.
