unit fFunc00048;

interface

uses Windows, Classes, Graphics, fFunc, GLScene, GLObjects, GLGeomObjects,
	GLHUDObjects, GLPolyhedron, GLTexture;

type
	TfrmFunc00048 = class(TfrmFunc)
  protected
    FHud: TGLHUDSprite;
    FCube: TGLCube;
    FTorus: TGLTorus;
    FDodecahedron: TGLDodecahedron;
    procedure CREATE_OBJECT; override;
    procedure DO_ViewerPostRender(Sender: TObject);
		procedure DO_Resize(Sender: TObject); override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
    procedure DO_KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState); override;
  public
  	Frames: Integer;
	end;

implementation

uses VectorGeometry, GLMisc, GLUtils, GLWin32Viewer;

{ TfrmFunc00048 }

procedure TfrmFunc00048.CREATE_OBJECT;
begin
  inherited;
  FHud := TGLHUDSprite.Create(Self);
  FCube := TGLCube.Create(Self);
  FTorus := TGLTorus.Create(Self);
  FDodecahedron := TGLDodecahedron.Create(Self);

  FScene.Cameras.AddChild(FCamera);
  FScene.Objects.AddChild(FLight);
  FScene.Objects.AddChild(FDummy);
  		FDummy.AddChild(FDodecahedron);
  FScene.Objects.AddChild(FCube);
  		FCube.AddChild(FTorus);
  FScene.Objects.AddChild(FHud);

  with FViewer do
	begin
  	Camera := FCamera;
    Buffer.BackgroundColor := clNavy;
    PostRender := DO_ViewerPostRender;
  end;

  with FScene do
	begin
  	ObjectsSorting := osNone;
    VisibilityCulling := vcNone;
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 70;
  	TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,0,-1);
    Position.AsVector := VectorMake(6,0,1);
    Up.AsVector := VectorMake(0,0,1);
  end;

  with FLight do
	begin
  	LightStyle := lsOmni;
    Position.AsVector := VectorMake(30,0,30);
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
    Direction.AsVector := VectorMake(0,-0.34,0.94);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,0.94,0.34);
  end;

  with FCube do
	begin
		RollAngle := -15;
    Parts := [cpTop,cpBottom,cpFront,cpBack,cpLeft,cpRight];
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0.26,0.966,0);
  	Material.FrontProperties.Diffuse.Color := clrRed;
  end;

  with FHud do
	begin
  	AlphaChannel := 1;
  	Height := 512;
    Width := 512;
    MirrorU := False;
    MirrorV := False;
    NoZWrite := True;
    XTiles := 1;
    YTiles := 1;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(256,256,0);
    Up.AsVector := VectorMake(0,1,0);
    with Material do
    begin
      BlendingMode := bmTransparency;
      FrontProperties.Diffuse.Color := VectorMake(1,1,1,0.1);
      MaterialOptions := [moIgnoreFog,moNoLighting];
      with Texture do
      begin
        Disabled := False;
        MagFilter := maNearest;
        MinFilter := miNearest;
        TextureMode := tmModulate;
        MappingSCoordinates.AsVector := VectorMake(1,0,0,0);
        MappingTCoordinates.AsVector := VectorMake(1,0,0,0);
      end;
		end;

  end;

  with FTorus do
	begin
  	MajorRadius := 0.4;
    MinorRadius := 0.1;
    Rings := 25;
    Sides := 15;
    Scale.AsVector := VectorMake(5,5,5);
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FDodecahedron do
	begin
  	RollAngle := 10;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,3);
    Up.AsVector := VectorMake(0,1,0);
    Material.FrontProperties.Diffuse.Color := clrYellow;
  end;

  Frames := 5;
  FHud.Material.FrontProperties.Diffuse.Alpha := 1-1/Frames;

end;

procedure TfrmFunc00048.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
  ROTATE_SCENE_OBJECT(FDummy,30,newTime,1);
	ROTATE_SCENE_OBJECT(FCube,45,newTime,2);
	ROTATE_SCENE_OBJECT(FDodecahedron,15,newTime,4);
  inherited;
end;

procedure TfrmFunc00048.DO_KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  // turn on/off VSync, this has an obvious impact on framerate,
  // which in turns impacts the motion blur look
  if (Key=Ord('S')) or (Key=Ord('V')) then
    if FViewer.VSync = vsmNoSync then FViewer.VSync := vsmSync
    else 															FViewer.VSync := vsmNoSync;

  // change the number of motion blur frames, and adjust
  // the transparency of the plane accordingly
  if Key = VK_UP then Inc(Frames);
  if (Key = VK_DOWN) and (Frames>0) then Dec(Frames);
  if Frames = 0 then FHUD.Visible := False
  else begin
    FHUD.Visible:=True;
    FHUD.Material.FrontProperties.Diffuse.Alpha:=1-1/(1+Frames);
  end;

end;

procedure TfrmFunc00048.DO_Resize(Sender: TObject);
var
	w, h: Integer;
begin
   // Here we resize our texture and plane to follow window dimension changes
   // Note that we have to stick to power of two texture dimensions if we don't
   // want performance to drop dramatically, this implies we can waste 3/4
   // of our texture memory... (f.i. a 513x513 window will require and use
   // a 1024x1024 texture)
   w := RoundUpToPowerOf2(FViewer.Width);
   h := RoundUpToPowerOf2(FViewer.Height);
   FHUD.Material.Texture.DestroyHandles;
(*
   FHUD.Material.Texture.ImageClassName := 'Blank Image';
   with ((FHUD.Material.Texture.Image) as TGLBlankImage) do
   begin
   	Width := w;
    Height := h;
   end;
*)   
   FHUD.Position.X := w*0.5;
   FHUD.Position.Y := FViewer.Height-h*0.5;
   FHUD.Width := w;
   FHUD.Height := h;
end;

procedure TfrmFunc00048.DO_ViewerPostRender(Sender: TObject);
begin
   // render is done, we transfer it to our hud plane so it can be used
   // in the next frame
   FViewer.Buffer.CopyToTexture(FHUD.Material.Texture);
end;

end.
