unit fFunc00049;

interface

uses fFunc, GLObjects, GLTexture, GLTexCombineShader, GLMultiMaterialShader;

type
	TfrmFunc00049 = class(TfrmFunc)
  protected
    FCube: TGLCube;
    FTexCombineShader: TGLTexCombineShader;
    FMultiMaterialShader: TGLMultiMaterialShader;
    FMaterialLibraries: array[0..1] of TGLMaterialLibrary;
    procedure CREATE_OBJECT; override;
    procedure DO_CadencerProgress(Sender: TObject; const deltaTime, newTime: Double); override;
  end;

implementation

uses VectorGeometry;

{ TfrmFunc00049 }

procedure TfrmFunc00049.CREATE_OBJECT;
var
	I: Integer;
begin
  inherited;
  FCube := TGLCube.Create(Self);
  FTexCombineShader := TGLTexCombineShader.Create(Self);
  FMultiMaterialShader := TGLMultiMaterialShader.Create(Self);
  for I := 0 to Length(FMaterialLibraries)-1 do
	  FMaterialLibraries[I] := TGLMaterialLibrary.Create(Self);

  FScene.Objects.AddChild(FDummy);
  FScene.Objects.AddChild(FCube);
  FDummy.AddChild(FCamera);
  FCamera.AddChild(FLight);

  with FViewer do
	begin
  	Camera := FCamera;
  end;

  with FDummy do
	begin
  	ShowAxes := True;
    VisibleAtRuntime := True;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FCamera do
	begin
  	DepthOfView := 1000;
    FocalLength := 70;
  	TargetObject := FDummy;
    Direction.AsVector := VectorMake(0,1,-1);
    Position.AsVector := VectorMake(1,3,-5);
    Up.AsVector := VectorMake(0,1,0);
  end;

  with FLight do
	begin
    Position.AsVector := VectorMake(0,0,0);
  end;

  with FCube do
	begin
  	CubeDepth := 2;
    CubeHeight := 2;
    CubeWidth := 2;
    Direction.AsVector := VectorMake(0,0,1);
    Position.AsVector := VectorMake(0,0,0);
    Up.AsVector := VectorMake(0,1,0);
  	Material.FrontProperties.Diffuse.Color := clrRed;
    Material.MaterialLibrary := FMaterialLibraries[0];
    Material.LibMaterialName := 'MultipassMat';
  end;

  with FMaterialLibraries[0] do
  begin
    // Add the specular pass
  	with AddTextureMaterial('specular','.\DATA\glscene_alpha.bmp') do
    begin
      // tmBlend for shiny background
      //Material.Texture.TextureMode:=tmBlend;
      // tmModulate for shiny text
      Material.Texture.TextureMode := tmModulate;
      Material.BlendingMode := bmAdditive;
      Texture2Name := 'specular_tex2';
    end;
    with AddTextureMaterial('specular_tex2','.\DATA\chrome_buckle.bmp') do
    begin
      Material.Texture.MappingMode := tmmCubeMapReflection;
      Material.Texture.ImageBrightness := 0.3;
    end;

  end;

  // GLMaterialLibrary2 is the source of the GLMultiMaterialShader
  // passes.
  with FMaterialLibraries[1] do
  begin
    // Pass 1 : Base texture
    AddTextureMaterial('Pass1','.\DATA\glscene.bmp');//}

    // Pass 2 : Add a bit of detail
    with AddTextureMaterial('Pass2','.\DATA\detailmap.jpg') do
    begin
      Material.Texture.TextureMode := tmBlend;
      Material.BlendingMode := bmAdditive;
    end;//}

    // Pass 3 : And a little specular reflection
    with TGLLibMaterial.Create(FMaterialLibraries[1].Materials) do
    begin
      Material.MaterialLibrary := FMaterialLibraries[0];
      Material.LibMaterialName := 'specular';
    end;//}

    // This isn't limited to 3, try adding some more passes!
  end;


end;

procedure TfrmFunc00049.DO_CadencerProgress(Sender: TObject;
  const deltaTime, newTime: Double);
begin
	FCube.Turn(deltaTime * 30);
  inherited;
end;

end.
